
module Make

# 42 "asllib/Parser.mly"
          (Config : ParserConfig.CONFIG)
# 7 "asllib/Parser.ml"

= struct
  
  module MenhirBasics = struct
    
    exception Error
    
    let _eRR =
      fun _s ->
        raise Error
    
    type token = Tokens.token
    
  end
  
  include MenhirBasics
  
# 50 "asllib/Parser.mly"
  

open AST
open ASTUtils
open Desugar

let version = V1

let t_bit =
  T_Bits (E_Literal (L_Int Z.one) |> add_dummy_annotation ~version, [])

let zero = E_Literal (L_Int Z.zero) |> add_dummy_annotation ~version

let make_ldi_vars (xs, ty) =
  let make_one x =
    S_Decl (LDK_Var, LDI_Typed (LDI_Var x, ty), None)
    |> add_dummy_annotation ~version
  in
  List.map make_one xs |> stmt_from_list |> desc

let make_ty_decl_subtype (x, s) =
  let name, _fields = s.desc in
  let ty = ASTUtils.add_pos_from s (T_Named name) in
  D_TypeDecl (x, ty, Some s.desc)

let prec =
  let open AST in
  function
  | BOR | BAND | IMPL | BEQ -> 1
  | EQ_OP | NEQ -> 2
  | PLUS | MINUS | OR | EOR | AND -> 3
  | MUL | DIV | DIVRM | RDIV | MOD | SHL | SHR -> 4
  | POW -> 5
  | _ -> 0 (* Non assoc *)

let check_not_same_prec loc op op' =
  if prec op = prec op' then Error.(fatal_from loc CannotParse)

let check_not_binop_same_prec op e =
  match e.desc with
  | E_Binop (op', _, _) when op != op' -> check_not_same_prec e op op'
  | _ -> ()

let e_binop (e1, op, e2) =
  let () = check_not_binop_same_prec op e1
  and () = check_not_binop_same_prec op e2
  in
  E_Binop (op, e1, e2)

let e_call call = E_Call { call with call_type = ST_Function }
let s_call call = S_Call { call with call_type = ST_Procedure }


# 79 "asllib/Parser.ml"
  
  type ('s, 'r) _menhir_state = 
    | MenhirState0000 : ('s, _menhir_box_ast) _menhir_state
      (** State 0000.
        Stack shape : .
        Start symbol: ast. *)

    | MenhirState0003 : (('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0003.
        Stack shape : VAR MINUS.
        Start symbol: ast. *)

    | MenhirState0005 : (('s, 'r) _menhir_cell1_UNKNOWN, 'r) _menhir_state
      (** State 0005.
        Stack shape : UNKNOWN.
        Start symbol: <undetermined>. *)

    | MenhirState0008 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0008.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0011 : (('s, 'r) _menhir_cell1_INTEGER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0011.
        Stack shape : INTEGER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0015 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
      (** State 0015.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

    | MenhirState0016 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0016.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0017 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0017.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0019 : (('s, 'r) _menhir_cell1_IF, 'r) _menhir_state
      (** State 0019.
        Stack shape : IF.
        Start symbol: <undetermined>. *)

    | MenhirState0021 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0021.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0024 : (('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_state
      (** State 0024.
        Stack shape : BNOT.
        Start symbol: <undetermined>. *)

    | MenhirState0026 : ((('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0026.
        Stack shape : BNOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0027 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LLBRACKET, 'r) _menhir_state
      (** State 0027.
        Stack shape : expr LLBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0028 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LLBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0028.
        Stack shape : expr LLBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0029 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHR, 'r) _menhir_state
      (** State 0029.
        Stack shape : expr SHR.
        Start symbol: <undetermined>. *)

    | MenhirState0030 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0030.
        Stack shape : expr SHR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0031 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_POW, 'r) _menhir_state
      (** State 0031.
        Stack shape : expr POW.
        Start symbol: <undetermined>. *)

    | MenhirState0032 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_POW, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0032.
        Stack shape : expr POW expr.
        Start symbol: <undetermined>. *)

    | MenhirState0033 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_state
      (** State 0033.
        Stack shape : expr LBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0034 : (('s, 'r) _menhir_cell1_COLON, 'r) _menhir_state
      (** State 0034.
        Stack shape : COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0035 : ((('s, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0035.
        Stack shape : COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0036 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHL, 'r) _menhir_state
      (** State 0036.
        Stack shape : expr SHL.
        Start symbol: <undetermined>. *)

    | MenhirState0037 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHL, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0037.
        Stack shape : expr SHL expr.
        Start symbol: <undetermined>. *)

    | MenhirState0039 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IN _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0039.
        Stack shape : expr IN LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0041 : (('s, 'r) _menhir_cell1_UNKNOWN, 'r) _menhir_state
      (** State 0041.
        Stack shape : UNKNOWN.
        Start symbol: <undetermined>. *)

    | MenhirState0045 : (('s, 'r) _menhir_cell1_BITS _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0045.
        Stack shape : BITS LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0046 : ((('s, 'r) _menhir_cell1_BITS _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0046.
        Stack shape : BITS LPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0048 : (((('s, 'r) _menhir_cell1_BITS _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0048.
        Stack shape : BITS LPAR expr RPAR LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0049 : (('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_state
      (** State 0049.
        Stack shape : LBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0053 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0053.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0057 : (((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_loption_non_empty_trailing_list_COMMA_bitfield__ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0057.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER LBRACE loption(non_empty_trailing_list(COMMA,bitfield)) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0059 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0059.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0061 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0061.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0064 : (('s, 'r) _menhir_cell1_ARRAY, 'r) _menhir_state
      (** State 0064.
        Stack shape : ARRAY.
        Start symbol: <undetermined>. *)

    | MenhirState0065 : ((('s, 'r) _menhir_cell1_ARRAY, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0065.
        Stack shape : ARRAY expr.
        Start symbol: <undetermined>. *)

    | MenhirState0066 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RDIV, 'r) _menhir_state
      (** State 0066.
        Stack shape : expr RDIV.
        Start symbol: <undetermined>. *)

    | MenhirState0067 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RDIV, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0067.
        Stack shape : expr RDIV expr.
        Start symbol: <undetermined>. *)

    | MenhirState0069 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOT, 'r) _menhir_state
      (** State 0069.
        Stack shape : expr DOT.
        Start symbol: <undetermined>. *)

    | MenhirState0071 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0071.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0076 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_CONCAT, 'r) _menhir_state
      (** State 0076.
        Stack shape : expr CONCAT.
        Start symbol: <undetermined>. *)

    | MenhirState0077 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_CONCAT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0077.
        Stack shape : expr CONCAT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0079 : (((('s, 'r) _menhir_cell1_ARRAY, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RBRACKET, 'r) _menhir_state
      (** State 0079.
        Stack shape : ARRAY expr RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0081 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS, 'r) _menhir_state
      (** State 0081.
        Stack shape : expr PLUS.
        Start symbol: <undetermined>. *)

    | MenhirState0082 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0082.
        Stack shape : expr PLUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0083 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MUL, 'r) _menhir_state
      (** State 0083.
        Stack shape : expr MUL.
        Start symbol: <undetermined>. *)

    | MenhirState0084 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MUL, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0084.
        Stack shape : expr MUL expr.
        Start symbol: <undetermined>. *)

    | MenhirState0085 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MOD, 'r) _menhir_state
      (** State 0085.
        Stack shape : expr MOD.
        Start symbol: <undetermined>. *)

    | MenhirState0086 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MOD, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0086.
        Stack shape : expr MOD expr.
        Start symbol: <undetermined>. *)

    | MenhirState0087 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIVRM, 'r) _menhir_state
      (** State 0087.
        Stack shape : expr DIVRM.
        Start symbol: <undetermined>. *)

    | MenhirState0088 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIVRM, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0088.
        Stack shape : expr DIVRM expr.
        Start symbol: <undetermined>. *)

    | MenhirState0089 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIV, 'r) _menhir_state
      (** State 0089.
        Stack shape : expr DIV.
        Start symbol: <undetermined>. *)

    | MenhirState0090 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIV, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0090.
        Stack shape : expr DIV expr.
        Start symbol: <undetermined>. *)

    | MenhirState0091 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OR, 'r) _menhir_state
      (** State 0091.
        Stack shape : expr OR.
        Start symbol: <undetermined>. *)

    | MenhirState0092 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0092.
        Stack shape : expr OR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0093 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_NEQ, 'r) _menhir_state
      (** State 0093.
        Stack shape : expr NEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0095 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_NEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0095.
        Stack shape : expr NEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0096 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0096.
        Stack shape : expr MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0097 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0097.
        Stack shape : expr MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0098 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LT, 'r) _menhir_state
      (** State 0098.
        Stack shape : expr LT.
        Start symbol: <undetermined>. *)

    | MenhirState0099 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0099.
        Stack shape : expr LT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0100 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EOR, 'r) _menhir_state
      (** State 0100.
        Stack shape : expr EOR.
        Start symbol: <undetermined>. *)

    | MenhirState0101 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EOR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0101.
        Stack shape : expr EOR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0102 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON_COLON, 'r) _menhir_state
      (** State 0102.
        Stack shape : expr COLON_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0103 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0103.
        Stack shape : expr COLON_COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0104 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AND, 'r) _menhir_state
      (** State 0104.
        Stack shape : expr AND.
        Start symbol: <undetermined>. *)

    | MenhirState0105 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AND, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0105.
        Stack shape : expr AND expr.
        Start symbol: <undetermined>. *)

    | MenhirState0106 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LEQ, 'r) _menhir_state
      (** State 0106.
        Stack shape : expr LEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0107 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0107.
        Stack shape : expr LEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0108 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GT, 'r) _menhir_state
      (** State 0108.
        Stack shape : expr GT.
        Start symbol: <undetermined>. *)

    | MenhirState0109 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0109.
        Stack shape : expr GT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0110 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GEQ, 'r) _menhir_state
      (** State 0110.
        Stack shape : expr GEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0111 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0111.
        Stack shape : expr GEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0112 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IMPL, 'r) _menhir_state
      (** State 0112.
        Stack shape : expr IMPL.
        Start symbol: <undetermined>. *)

    | MenhirState0113 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IMPL, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0113.
        Stack shape : expr IMPL expr.
        Start symbol: <undetermined>. *)

    | MenhirState0114 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EQ_OP, 'r) _menhir_state
      (** State 0114.
        Stack shape : expr EQ_OP.
        Start symbol: <undetermined>. *)

    | MenhirState0116 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EQ_OP, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0116.
        Stack shape : expr EQ_OP expr.
        Start symbol: <undetermined>. *)

    | MenhirState0117 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BOR, 'r) _menhir_state
      (** State 0117.
        Stack shape : expr BOR.
        Start symbol: <undetermined>. *)

    | MenhirState0118 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BOR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0118.
        Stack shape : expr BOR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0119 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BEQ, 'r) _menhir_state
      (** State 0119.
        Stack shape : expr BEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0120 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0120.
        Stack shape : expr BEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0121 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BAND, 'r) _menhir_state
      (** State 0121.
        Stack shape : expr BAND.
        Start symbol: <undetermined>. *)

    | MenhirState0122 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BAND, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0122.
        Stack shape : expr BAND expr.
        Start symbol: <undetermined>. *)

    | MenhirState0123 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AS, 'r) _menhir_state
      (** State 0123.
        Stack shape : expr AS.
        Start symbol: <undetermined>. *)

    | MenhirState0124 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AS, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0124.
        Stack shape : expr AS LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0128 : (('s, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0128.
        Stack shape : expr.
        Start symbol: <undetermined>. *)

    | MenhirState0129 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SLICING, 'r) _menhir_state
      (** State 0129.
        Stack shape : expr SLICING.
        Start symbol: <undetermined>. *)

    | MenhirState0130 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SLICING, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0130.
        Stack shape : expr SLICING expr.
        Start symbol: <undetermined>. *)

    | MenhirState0131 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SLICING, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0131.
        Stack shape : expr SLICING expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0133 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0133.
        Stack shape : expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0137 : (((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0137.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER ty.
        Start symbol: <undetermined>. *)

    | MenhirState0139 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0139.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0141 : (((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0141.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER ty.
        Start symbol: <undetermined>. *)

    | MenhirState0143 : (('s, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0143.
        Stack shape : expr.
        Start symbol: <undetermined>. *)

    | MenhirState0144 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_STAR_COLON, 'r) _menhir_state
      (** State 0144.
        Stack shape : expr STAR_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0145 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_STAR_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0145.
        Stack shape : expr STAR_COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0146 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_STAR_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0146.
        Stack shape : expr STAR_COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0148 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS_COLON, 'r) _menhir_state
      (** State 0148.
        Stack shape : expr PLUS_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0149 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0149.
        Stack shape : expr PLUS_COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0150 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0150.
        Stack shape : expr PLUS_COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0152 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0152.
        Stack shape : expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0154 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON, 'r) _menhir_state
      (** State 0154.
        Stack shape : expr COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0155 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0155.
        Stack shape : expr COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0156 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0156.
        Stack shape : expr COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0162 : (('s, 'r) _menhir_cell1_UNKNOWN, 'r) _menhir_state
      (** State 0162.
        Stack shape : UNKNOWN.
        Start symbol: <undetermined>. *)

    | MenhirState0166 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
      (** State 0166.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

    | MenhirState0167 : ((('s, 'r) _menhir_cell1_NOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0167.
        Stack shape : NOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0168 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0168.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0169 : ((('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0169.
        Stack shape : MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0171 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0171.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0172 : (('s, 'r) _menhir_cell1_LEQ, 'r) _menhir_state
      (** State 0172.
        Stack shape : LEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0173 : ((('s, 'r) _menhir_cell1_LEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0173.
        Stack shape : LEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0174 : (('s, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0174.
        Stack shape : LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0176 : (('s, 'r) _menhir_cell1_IF, 'r) _menhir_state
      (** State 0176.
        Stack shape : IF.
        Start symbol: <undetermined>. *)

    | MenhirState0177 : ((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0177.
        Stack shape : IF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0178 : (((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0178.
        Stack shape : IF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0179 : ((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0179.
        Stack shape : IF expr THEN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0180 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_state
      (** State 0180.
        Stack shape : expr THEN expr ELSIF.
        Start symbol: <undetermined>. *)

    | MenhirState0181 : (((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0181.
        Stack shape : expr THEN expr ELSIF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0182 : ((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0182.
        Stack shape : expr THEN expr ELSIF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0183 : (((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0183.
        Stack shape : expr THEN expr ELSIF expr THEN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0184 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSE, 'r) _menhir_state
      (** State 0184.
        Stack shape : expr THEN expr ELSE.
        Start symbol: <undetermined>. *)

    | MenhirState0185 : (((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0185.
        Stack shape : expr THEN expr ELSE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0189 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0189.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0193 : (('s, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0193.
        Stack shape : expr.
        Start symbol: <undetermined>. *)

    | MenhirState0194 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0194.
        Stack shape : expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0196 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0196.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0199 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0199.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0204 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0204.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0205 : ((('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0205.
        Stack shape : separated_nonempty_list(COMMA,expr) RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0210 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0210.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0211 : ((('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0211.
        Stack shape : IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0212 : (((('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0212.
        Stack shape : IDENTIFIER expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0218 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0218.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0220 : (('s, 'r) _menhir_cell1_GEQ, 'r) _menhir_state
      (** State 0220.
        Stack shape : GEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0221 : ((('s, 'r) _menhir_cell1_GEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0221.
        Stack shape : GEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0223 : (('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_state
      (** State 0223.
        Stack shape : BNOT.
        Start symbol: <undetermined>. *)

    | MenhirState0224 : ((('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0224.
        Stack shape : BNOT LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0229 : (('s, 'r) _menhir_cell1_pattern, 'r) _menhir_state
      (** State 0229.
        Stack shape : pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0232 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0232.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0233 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0233.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0234 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0234.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0235 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0235.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0236 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0236.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0237 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0237.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0238 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0238.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0239 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0239.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0240 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0240.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0241 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0241.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0242 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0242.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0243 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0243.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0244 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0244.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0245 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0245.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0246 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0246.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0248 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0248.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0249 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0249.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0250 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0250.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0251 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0251.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0252 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0252.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0253 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_MINUS, 'r) _menhir_state
      (** State 0253.
        Stack shape : expr_pattern MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0254 : ((('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0254.
        Stack shape : expr_pattern MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0255 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0255.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0256 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0256.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0257 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0257.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0258 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0258.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0260 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_LEQ, 'r) _menhir_state
      (** State 0260.
        Stack shape : expr_pattern LEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0261 : ((('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_LEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0261.
        Stack shape : expr_pattern LEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0262 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0262.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0266 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0266.
        Stack shape : expr_pattern LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0270 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_BNOT _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0270.
        Stack shape : expr_pattern BNOT LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0273 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0273.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0274 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0274.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0275 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0275.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0276 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0276.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0277 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_GEQ, 'r) _menhir_state
      (** State 0277.
        Stack shape : expr_pattern GEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0278 : ((('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_GEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0278.
        Stack shape : expr_pattern GEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0279 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0279.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0281 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0281.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0282 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0282.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0283 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0283.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0285 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0285.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0289 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0289.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0290 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0290.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0291 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0291.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0292 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0292.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0293 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0293.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0294 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0294.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0295 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0295.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0296 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0296.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0297 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0297.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0298 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0298.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0299 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0299.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0300 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0300.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0301 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0301.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0302 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0302.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0303 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0303.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0304 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0304.
        Stack shape : expr_pattern LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0309 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0309.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0310 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0310.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0311 : ((('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0311.
        Stack shape : BNOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0315 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_pattern, 'r) _menhir_state
      (** State 0315.
        Stack shape : LPAR pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0323 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IN _menhir_cell0_BNOT _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0323.
        Stack shape : expr IN BNOT LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0326 : (((('s, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0326.
        Stack shape : COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0333 : ((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0333.
        Stack shape : IF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0334 : (((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0334.
        Stack shape : IF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0335 : ((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0335.
        Stack shape : IF expr THEN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0337 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0337.
        Stack shape : LPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0339 : (((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0339.
        Stack shape : LPAR expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0342 : ((('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0342.
        Stack shape : MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0343 : ((('s, 'r) _menhir_cell1_NOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0343.
        Stack shape : NOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0347 : (('s, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0347.
        Stack shape : ty.
        Start symbol: <undetermined>. *)

    | MenhirState0352 : (('s, 'r) _menhir_cell1_UNKNOWN, 'r) _menhir_state
      (** State 0352.
        Stack shape : UNKNOWN.
        Start symbol: <undetermined>. *)

    | MenhirState0354 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0354.
        Stack shape : VAR MINUS expr.
        Start symbol: ast. *)

    | MenhirState0355 : (((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0355.
        Stack shape : VAR MINUS expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0359 : (('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0359.
        Stack shape : TYPE IDENTIFIER SUBTYPES IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0361 : (('s _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_WITH _menhir_cell0_LBRACE, _menhir_box_ast) _menhir_state
      (** State 0361.
        Stack shape : SUBTYPES IDENTIFIER WITH LBRACE.
        Start symbol: ast. *)

    | MenhirState0363 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0363.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0365 : ((('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0365.
        Stack shape : IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0367 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0367.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0369 : ((('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0369.
        Stack shape : IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0375 : ((('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption___anonymous_22_ _menhir_cell0_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0375.
        Stack shape : TYPE IDENTIFIER SUBTYPES IDENTIFIER loption(__anonymous_22) SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0377 : (('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0377.
        Stack shape : SETTER IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0378 : (('s _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_LBRACE, _menhir_box_ast) _menhir_state
      (** State 0378.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: ast. *)

    | MenhirState0380 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0380.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0382 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0382.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0384 : ((('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0384.
        Stack shape : IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0386 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0386.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0388 : ((('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0388.
        Stack shape : IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0393 : ((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_state
      (** State 0393.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR.
        Start symbol: ast. *)

    | MenhirState0397 : (((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0397.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0399 : ((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0399.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN.
        Start symbol: ast. *)

    | MenhirState0400 : (('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_state
      (** State 0400.
        Stack shape : WHILE.
        Start symbol: <undetermined>. *)

    | MenhirState0401 : ((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0401.
        Stack shape : WHILE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0402 : (((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0402.
        Stack shape : WHILE expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0403 : ((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0403.
        Stack shape : WHILE expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0404 : (((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0404.
        Stack shape : WHILE expr LOOPLIMIT expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0405 : (('s, 'r) _menhir_cell1_VAR, 'r) _menhir_state
      (** State 0405.
        Stack shape : VAR.
        Start symbol: <undetermined>. *)

    | MenhirState0407 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0407.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0409 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0409.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0411 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0411.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0413 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0413.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0415 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0415.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0418 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0418.
        Stack shape : LPAR decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0421 : (((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR, 'r) _menhir_state
      (** State 0421.
        Stack shape : LPAR decl_item separated_nonempty_list(COMMA,decl_item) RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0423 : (((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR, 'r) _menhir_state
      (** State 0423.
        Stack shape : LPAR decl_item separated_nonempty_list(COMMA,decl_item) RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0426 : ((('s, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0426.
        Stack shape : decl_item decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0429 : ((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0429.
        Stack shape : VAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0431 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_state
      (** State 0431.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: <undetermined>. *)

    | MenhirState0433 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0433.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) ty SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0437 : (('s, 'r) _menhir_cell1_UNREACHABLE _menhir_cell0_LPAR _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0437.
        Stack shape : UNREACHABLE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0438 : (('s, 'r) _menhir_cell1_TRY, 'r) _menhir_state
      (** State 0438.
        Stack shape : TRY.
        Start symbol: <undetermined>. *)

    | MenhirState0439 : (('s, 'r) _menhir_cell1_THROW, 'r) _menhir_state
      (** State 0439.
        Stack shape : THROW.
        Start symbol: <undetermined>. *)

    | MenhirState0440 : ((('s, 'r) _menhir_cell1_THROW, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0440.
        Stack shape : THROW SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0441 : (('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_state
      (** State 0441.
        Stack shape : RETURN.
        Start symbol: <undetermined>. *)

    | MenhirState0442 : ((('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0442.
        Stack shape : RETURN SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0443 : (('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_state
      (** State 0443.
        Stack shape : REPEAT.
        Start symbol: <undetermined>. *)

    | MenhirState0445 : (('s, 'r) _menhir_cell1_PRINT _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0445.
        Stack shape : PRINT LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0447 : ((('s, 'r) _menhir_cell1_PRINT _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0447.
        Stack shape : PRINT LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0449 : (('s, 'r) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0449.
        Stack shape : PRAGMA IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0450 : ((('s, 'r) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0450.
        Stack shape : PRAGMA IDENTIFIER SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0452 : (('s, 'r) _menhir_cell1_PASS _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0452.
        Stack shape : PASS SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0454 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0454.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0457 : (('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_state
      (** State 0457.
        Stack shape : lexpr_atom_desc.
        Start symbol: <undetermined>. *)

    | MenhirState0458 : ((('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0458.
        Stack shape : lexpr_atom_desc expr.
        Start symbol: <undetermined>. *)

    | MenhirState0460 : (('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_state
      (** State 0460.
        Stack shape : lexpr_atom_desc.
        Start symbol: <undetermined>. *)

    | MenhirState0464 : (('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_state
      (** State 0464.
        Stack shape : lexpr_atom_desc.
        Start symbol: <undetermined>. *)

    | MenhirState0470 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_lexpr, 'r) _menhir_state
      (** State 0470.
        Stack shape : LPAR lexpr.
        Start symbol: <undetermined>. *)

    | MenhirState0474 : ((('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_cell1_lexpr, 'r) _menhir_state
      (** State 0474.
        Stack shape : lexpr lexpr.
        Start symbol: <undetermined>. *)

    | MenhirState0476 : (('s, 'r) _menhir_cell1_LET, 'r) _menhir_state
      (** State 0476.
        Stack shape : LET.
        Start symbol: <undetermined>. *)

    | MenhirState0478 : ((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0478.
        Stack shape : LET decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0480 : (((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0480.
        Stack shape : LET decl_item IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0482 : ((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0482.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0484 : (((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0484.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0485 : (('s, 'r) _menhir_cell1_IF, 'r) _menhir_state
      (** State 0485.
        Stack shape : IF.
        Start symbol: <undetermined>. *)

    | MenhirState0486 : ((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0486.
        Stack shape : IF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0487 : (((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0487.
        Stack shape : IF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0489 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0489.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0491 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0491.
        Stack shape : IDENTIFIER LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0494 : (('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0494.
        Stack shape : FOR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0495 : ((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0495.
        Stack shape : FOR IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0496 : (((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_state
      (** State 0496.
        Stack shape : FOR IDENTIFIER expr TO.
        Start symbol: <undetermined>. *)

    | MenhirState0497 : ((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0497.
        Stack shape : FOR IDENTIFIER expr TO expr.
        Start symbol: <undetermined>. *)

    | MenhirState0498 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0498.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0499 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0499.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0500 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0500.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0502 : (('s, 'r) _menhir_cell1_DEBUG _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0502.
        Stack shape : DEBUG LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0504 : ((('s, 'r) _menhir_cell1_DEBUG _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0504.
        Stack shape : DEBUG LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0505 : (('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_state
      (** State 0505.
        Stack shape : CONSTANT.
        Start symbol: <undetermined>. *)

    | MenhirState0507 : ((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0507.
        Stack shape : CONSTANT decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0509 : (((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0509.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0511 : ((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0511.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0513 : (((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0513.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0514 : (('s, 'r) _menhir_cell1_CASE, 'r) _menhir_state
      (** State 0514.
        Stack shape : CASE.
        Start symbol: <undetermined>. *)

    | MenhirState0515 : ((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0515.
        Stack shape : CASE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0516 : (((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OF, 'r) _menhir_state
      (** State 0516.
        Stack shape : CASE expr OF.
        Start symbol: <undetermined>. *)

    | MenhirState0517 : (('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_state
      (** State 0517.
        Stack shape : WHEN.
        Start symbol: <undetermined>. *)

    | MenhirState0519 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_state
      (** State 0519.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern).
        Start symbol: <undetermined>. *)

    | MenhirState0520 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0520.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0521 : ((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ARROW, 'r) _menhir_state
      (** State 0521.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr ARROW.
        Start symbol: <undetermined>. *)

    | MenhirState0522 : (('s, 'r) _menhir_cell1_ASSERT, 'r) _menhir_state
      (** State 0522.
        Stack shape : ASSERT.
        Start symbol: <undetermined>. *)

    | MenhirState0523 : ((('s, 'r) _menhir_cell1_ASSERT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0523.
        Stack shape : ASSERT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0524 : (((('s, 'r) _menhir_cell1_ASSERT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0524.
        Stack shape : ASSERT expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0526 : (('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0526.
        Stack shape : ARROBASE_LOOPLIMIT LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0527 : ((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0527.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0529 : (((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_state
      (** State 0529.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE.
        Start symbol: <undetermined>. *)

    | MenhirState0530 : ((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0530.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0531 : (((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0531.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0532 : ((((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0532.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0536 : (((((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0536.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0539 : (('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_state
      (** State 0539.
        Stack shape : lexpr.
        Start symbol: <undetermined>. *)

    | MenhirState0540 : ((('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0540.
        Stack shape : lexpr expr.
        Start symbol: <undetermined>. *)

    | MenhirState0541 : (((('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0541.
        Stack shape : lexpr expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0543 : (((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_state
      (** State 0543.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT.
        Start symbol: <undetermined>. *)

    | MenhirState0545 : ((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0545.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0546 : (((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0546.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0547 : ((((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0547.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt) expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0550 : (((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ARROW, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0550.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr ARROW nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0552 : ((((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ARROW, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_state
      (** State 0552.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr ARROW nonempty_list(stmt) OTHERWISE.
        Start symbol: <undetermined>. *)

    | MenhirState0555 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_state
      (** State 0555.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern).
        Start symbol: <undetermined>. *)

    | MenhirState0556 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0556.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0558 : ((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_state
      (** State 0558.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) nonempty_list(stmt) OTHERWISE.
        Start symbol: <undetermined>. *)

    | MenhirState0561 : ((((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OF, 'r) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, 'r) _menhir_state
      (** State 0561.
        Stack shape : CASE expr OF nlist_opt_terminated(annotated(case_alt),annotated(case_otherwise)).
        Start symbol: <undetermined>. *)

    | MenhirState0562 : (((((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OF, 'r) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0562.
        Stack shape : CASE expr OF nlist_opt_terminated(annotated(case_alt),annotated(case_otherwise)) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0567 : (((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0567.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0569 : ((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0569.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0571 : (((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0571.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0573 : ((((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0573.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0575 : (((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0575.
        Stack shape : CONSTANT decl_item expr.
        Start symbol: <undetermined>. *)

    | MenhirState0576 : ((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0576.
        Stack shape : CONSTANT decl_item expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0581 : ((('s, 'r) _menhir_cell1_DEBUG _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0581.
        Stack shape : DEBUG LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0583 : ((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0583.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0584 : (((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0584.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0586 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0586.
        Stack shape : FOR IDENTIFIER expr TO expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0587 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0587.
        Stack shape : FOR IDENTIFIER expr TO expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0588 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0588.
        Stack shape : FOR IDENTIFIER expr TO expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0590 : (((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_state
      (** State 0590.
        Stack shape : FOR IDENTIFIER expr DOWNTO.
        Start symbol: <undetermined>. *)

    | MenhirState0591 : ((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0591.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr.
        Start symbol: <undetermined>. *)

    | MenhirState0592 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0592.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0593 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0593.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0594 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0594.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0595 : ((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0595.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0596 : (((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0596.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0598 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0598.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0599 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0599.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0600 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0600.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0603 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR, 'r) _menhir_state
      (** State 0603.
        Stack shape : IDENTIFIER LPAR RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0604 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0604.
        Stack shape : IDENTIFIER LPAR RPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0605 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0605.
        Stack shape : IDENTIFIER LPAR RPAR expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0610 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0610.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0613 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_state
      (** State 0613.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0614 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0614.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0615 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0615.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0618 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0618.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0619 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0619.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0620 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0620.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0624 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0624.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0626 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, 'r) _menhir_state
      (** State 0626.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0627 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0627.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0628 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0628.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0633 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0633.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0636 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_state
      (** State 0636.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0637 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0637.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0638 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0638.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0641 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0641.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0642 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0642.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0643 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0643.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0645 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0645.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0647 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0647.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0649 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0649.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0651 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__, 'r) _menhir_state
      (** State 0651.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)).
        Start symbol: <undetermined>. *)

    | MenhirState0652 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0652.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0653 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0653.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0658 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0658.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0661 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_state
      (** State 0661.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0662 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0662.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0663 : ((((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0663.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0666 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0666.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0667 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0667.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0668 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0668.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0670 : ((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0670.
        Stack shape : IF expr THEN nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0671 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_state
      (** State 0671.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF.
        Start symbol: <undetermined>. *)

    | MenhirState0672 : (((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0672.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0673 : ((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0673.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0674 : (((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0674.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF expr THEN nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0675 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSE, 'r) _menhir_state
      (** State 0675.
        Stack shape : expr THEN nonempty_list(stmt) ELSE.
        Start symbol: <undetermined>. *)

    | MenhirState0678 : (((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_s_else, 'r) _menhir_state
      (** State 0678.
        Stack shape : IF expr THEN nonempty_list(stmt) s_else.
        Start symbol: <undetermined>. *)

    | MenhirState0679 : ((((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_s_else, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0679.
        Stack shape : IF expr THEN nonempty_list(stmt) s_else end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0684 : (((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0684.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0686 : ((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0686.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0688 : (((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0688.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0690 : ((((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0690.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0692 : (((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0692.
        Stack shape : LET decl_item expr.
        Start symbol: <undetermined>. *)

    | MenhirState0693 : ((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0693.
        Stack shape : LET decl_item expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0698 : ((('s, 'r) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0698.
        Stack shape : PRAGMA IDENTIFIER separated_nonempty_list(COMMA,expr) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0703 : ((('s, 'r) _menhir_cell1_PRINT _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0703.
        Stack shape : PRINT LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0706 : ((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0706.
        Stack shape : REPEAT nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0707 : (((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0707.
        Stack shape : REPEAT nonempty_list(stmt) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0708 : ((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0708.
        Stack shape : REPEAT nonempty_list(stmt) expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0710 : ((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0710.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0711 : (((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0711.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0712 : ((((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0712.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0715 : ((('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0715.
        Stack shape : RETURN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0716 : (((('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0716.
        Stack shape : RETURN expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0719 : ((('s, 'r) _menhir_cell1_THROW, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0719.
        Stack shape : THROW expr.
        Start symbol: <undetermined>. *)

    | MenhirState0720 : (((('s, 'r) _menhir_cell1_THROW, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0720.
        Stack shape : THROW expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0723 : ((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0723.
        Stack shape : TRY nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0724 : (('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_state
      (** State 0724.
        Stack shape : WHEN.
        Start symbol: <undetermined>. *)

    | MenhirState0726 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0726.
        Stack shape : WHEN IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0728 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0728.
        Stack shape : WHEN IDENTIFIER ty.
        Start symbol: <undetermined>. *)

    | MenhirState0729 : ((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0729.
        Stack shape : WHEN IDENTIFIER ty nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0732 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0732.
        Stack shape : WHEN ty.
        Start symbol: <undetermined>. *)

    | MenhirState0733 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_ty, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0733.
        Stack shape : WHEN ty nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0735 : (((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_state
      (** State 0735.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher).
        Start symbol: <undetermined>. *)

    | MenhirState0737 : ((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_state
      (** State 0737.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE.
        Start symbol: <undetermined>. *)

    | MenhirState0738 : (((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0738.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0739 : ((((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0739.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0741 : ((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0741.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0745 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_state
      (** State 0745.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: <undetermined>. *)

    | MenhirState0747 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0747.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) ty SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0750 : ((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0750.
        Stack shape : VAR decl_item SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0752 : ((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0752.
        Stack shape : VAR decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0754 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0754.
        Stack shape : VAR decl_item IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0756 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0756.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0758 : (((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0758.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0762 : (((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0762.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0764 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0764.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0766 : (((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0766.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0768 : ((((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0768.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0770 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0770.
        Stack shape : VAR decl_item expr.
        Start symbol: <undetermined>. *)

    | MenhirState0771 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0771.
        Stack shape : VAR decl_item expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0773 : ((((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0773.
        Stack shape : WHILE expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0774 : (((((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0774.
        Stack shape : WHILE expr LOOPLIMIT expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0776 : (((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0776.
        Stack shape : WHILE expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0777 : ((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0777.
        Stack shape : WHILE expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0778 : (((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0778.
        Stack shape : WHILE expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0780 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0780.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0781 : ((((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0781.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0783 : (('s, _menhir_box_ast) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0783.
        Stack shape : PRAGMA IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0784 : ((('s, _menhir_box_ast) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0784.
        Stack shape : PRAGMA IDENTIFIER SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0787 : (('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0787.
        Stack shape : LET MINUS.
        Start symbol: ast. *)

    | MenhirState0788 : ((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0788.
        Stack shape : LET MINUS expr.
        Start symbol: ast. *)

    | MenhirState0789 : (((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0789.
        Stack shape : LET MINUS expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0791 : (('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0791.
        Stack shape : GETTER IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0793 : ((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_state
      (** State 0793.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR.
        Start symbol: ast. *)

    | MenhirState0795 : (((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_state
      (** State 0795.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR.
        Start symbol: ast. *)

    | MenhirState0797 : ((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0797.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN.
        Start symbol: ast. *)

    | MenhirState0798 : (((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0798.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0799 : ((((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0799.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0801 : (('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0801.
        Stack shape : FUNC IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0803 : ((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_state
      (** State 0803.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR.
        Start symbol: ast. *)

    | MenhirState0805 : (((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0805.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN.
        Start symbol: ast. *)

    | MenhirState0806 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0806.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0807 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0807.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0810 : (('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0810.
        Stack shape : CONSTANT MINUS.
        Start symbol: ast. *)

    | MenhirState0811 : ((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0811.
        Stack shape : CONSTANT MINUS expr.
        Start symbol: ast. *)

    | MenhirState0812 : (((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0812.
        Stack shape : CONSTANT MINUS expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0815 : (('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0815.
        Stack shape : CONFIG MINUS.
        Start symbol: ast. *)

    | MenhirState0816 : ((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0816.
        Stack shape : CONFIG MINUS expr.
        Start symbol: ast. *)

    | MenhirState0817 : (((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0817.
        Stack shape : CONFIG MINUS expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0819 : (('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0819.
        Stack shape : CONFIG MINUS.
        Start symbol: ast. *)

    | MenhirState0821 : ((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0821.
        Stack shape : CONFIG MINUS ty.
        Start symbol: ast. *)

    | MenhirState0822 : (((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0822.
        Stack shape : CONFIG MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState0823 : ((((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0823.
        Stack shape : CONFIG MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0825 : (('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0825.
        Stack shape : CONFIG MINUS.
        Start symbol: ast. *)

    | MenhirState0827 : ((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0827.
        Stack shape : CONFIG MINUS ty.
        Start symbol: ast. *)

    | MenhirState0828 : (((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0828.
        Stack shape : CONFIG MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState0829 : ((((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0829.
        Stack shape : CONFIG MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0832 : (('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0832.
        Stack shape : CONFIG IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0833 : ((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0833.
        Stack shape : CONFIG IDENTIFIER expr.
        Start symbol: ast. *)

    | MenhirState0834 : (((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0834.
        Stack shape : CONFIG IDENTIFIER expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0836 : (('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0836.
        Stack shape : CONFIG IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0838 : ((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0838.
        Stack shape : CONFIG IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0839 : (((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0839.
        Stack shape : CONFIG IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState0840 : ((((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0840.
        Stack shape : CONFIG IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0842 : (('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0842.
        Stack shape : CONFIG IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0844 : ((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0844.
        Stack shape : CONFIG IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0845 : (((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0845.
        Stack shape : CONFIG IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState0846 : ((((('s, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0846.
        Stack shape : CONFIG IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0849 : (('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0849.
        Stack shape : CONSTANT MINUS.
        Start symbol: ast. *)

    | MenhirState0851 : ((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0851.
        Stack shape : CONSTANT MINUS ty.
        Start symbol: ast. *)

    | MenhirState0852 : (((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0852.
        Stack shape : CONSTANT MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState0853 : ((((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0853.
        Stack shape : CONSTANT MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0855 : (('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0855.
        Stack shape : CONSTANT MINUS.
        Start symbol: ast. *)

    | MenhirState0857 : ((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0857.
        Stack shape : CONSTANT MINUS ty.
        Start symbol: ast. *)

    | MenhirState0858 : (((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0858.
        Stack shape : CONSTANT MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState0859 : ((((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0859.
        Stack shape : CONSTANT MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0862 : (('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0862.
        Stack shape : CONSTANT IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0863 : ((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0863.
        Stack shape : CONSTANT IDENTIFIER expr.
        Start symbol: ast. *)

    | MenhirState0864 : (((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0864.
        Stack shape : CONSTANT IDENTIFIER expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0866 : (('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0866.
        Stack shape : CONSTANT IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0868 : ((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0868.
        Stack shape : CONSTANT IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0869 : (((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0869.
        Stack shape : CONSTANT IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState0870 : ((((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0870.
        Stack shape : CONSTANT IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0872 : (('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0872.
        Stack shape : CONSTANT IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0874 : ((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0874.
        Stack shape : CONSTANT IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0875 : (((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0875.
        Stack shape : CONSTANT IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState0876 : ((((('s, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0876.
        Stack shape : CONSTANT IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0879 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0879.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0881 : (((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_state
      (** State 0881.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR.
        Start symbol: ast. *)

    | MenhirState0883 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0883.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty.
        Start symbol: ast. *)

    | MenhirState0884 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0884.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr.
        Start symbol: ast. *)

    | MenhirState0885 : ((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0885.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN.
        Start symbol: ast. *)

    | MenhirState0886 : (((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0886.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0887 : ((((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0887.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0889 : (((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0889.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0891 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0891.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN.
        Start symbol: ast. *)

    | MenhirState0892 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0892.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0893 : ((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0893.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0895 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0895.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0898 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0898.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0900 : ((('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0900.
        Stack shape : IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0902 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0902.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0904 : ((('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0904.
        Stack shape : IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0908 : (((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0908.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN.
        Start symbol: ast. *)

    | MenhirState0909 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0909.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0910 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0910.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0912 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0912.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0914 : (((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_state
      (** State 0914.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR.
        Start symbol: ast. *)

    | MenhirState0916 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0916.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty.
        Start symbol: ast. *)

    | MenhirState0917 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0917.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr.
        Start symbol: ast. *)

    | MenhirState0918 : ((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0918.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN.
        Start symbol: ast. *)

    | MenhirState0919 : (((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0919.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0920 : ((((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0920.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0922 : (((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0922.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0924 : ((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0924.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN.
        Start symbol: ast. *)

    | MenhirState0925 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0925.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0926 : ((((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0926.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0928 : (((((('s, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0928.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0931 : (((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0931.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0935 : (((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_state
      (** State 0935.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR.
        Start symbol: ast. *)

    | MenhirState0937 : ((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0937.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN.
        Start symbol: ast. *)

    | MenhirState0938 : (((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0938.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0939 : ((((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0939.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0941 : (((((('s, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0941.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0944 : (('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0944.
        Stack shape : LET MINUS.
        Start symbol: ast. *)

    | MenhirState0946 : ((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0946.
        Stack shape : LET MINUS ty.
        Start symbol: ast. *)

    | MenhirState0947 : (((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0947.
        Stack shape : LET MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState0948 : ((((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0948.
        Stack shape : LET MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0950 : (('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 0950.
        Stack shape : LET MINUS.
        Start symbol: ast. *)

    | MenhirState0952 : ((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0952.
        Stack shape : LET MINUS ty.
        Start symbol: ast. *)

    | MenhirState0953 : (((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0953.
        Stack shape : LET MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState0954 : ((((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0954.
        Stack shape : LET MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0957 : (('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0957.
        Stack shape : LET IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0958 : ((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0958.
        Stack shape : LET IDENTIFIER expr.
        Start symbol: ast. *)

    | MenhirState0959 : (((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0959.
        Stack shape : LET IDENTIFIER expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0961 : (('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0961.
        Stack shape : LET IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0963 : ((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0963.
        Stack shape : LET IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0964 : (((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0964.
        Stack shape : LET IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState0965 : ((((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0965.
        Stack shape : LET IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0967 : (('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0967.
        Stack shape : LET IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0969 : ((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 0969.
        Stack shape : LET IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState0970 : (((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 0970.
        Stack shape : LET IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState0971 : ((((('s, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0971.
        Stack shape : LET IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0975 : ((('s, _menhir_box_ast) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 0975.
        Stack shape : PRAGMA IDENTIFIER separated_nonempty_list(COMMA,expr) SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState0978 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0978.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0980 : (((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0980.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0982 : ((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0982.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN.
        Start symbol: ast. *)

    | MenhirState0983 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0983.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0984 : ((((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0984.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0986 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0986.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState0992 : (((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 0992.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState0994 : ((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 0994.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN.
        Start symbol: ast. *)

    | MenhirState0995 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 0995.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState0996 : ((((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0996.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState0998 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 0998.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState1000 : (((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 1000.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState1002 : ((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_state
      (** State 1002.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN.
        Start symbol: ast. *)

    | MenhirState1003 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_state
      (** State 1003.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: ast. *)

    | MenhirState1004 : ((((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 1004.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: ast. *)

    | MenhirState1006 : (((((('s, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon, _menhir_box_ast) _menhir_state
      (** State 1006.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: ast. *)

    | MenhirState1009 : (('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 1009.
        Stack shape : TYPE IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState1011 : ((('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_RECORD _menhir_cell0_LBRACE, _menhir_box_ast) _menhir_state
      (** State 1011.
        Stack shape : TYPE IDENTIFIER RECORD LBRACE.
        Start symbol: ast. *)

    | MenhirState1015 : ((('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_EXCEPTION _menhir_cell0_LBRACE, _menhir_box_ast) _menhir_state
      (** State 1015.
        Stack shape : TYPE IDENTIFIER EXCEPTION LBRACE.
        Start symbol: ast. *)

    | MenhirState1019 : ((('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ENUMERATION _menhir_cell0_LBRACE, _menhir_box_ast) _menhir_state
      (** State 1019.
        Stack shape : TYPE IDENTIFIER ENUMERATION LBRACE.
        Start symbol: ast. *)

    | MenhirState1021 : (('s, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 1021.
        Stack shape : IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState1027 : ((('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty_decl _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 1027.
        Stack shape : TYPE IDENTIFIER ty_decl SUBTYPES IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState1030 : ((('s, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty_decl _menhir_cell0_option_subtype_ _menhir_cell0_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1030.
        Stack shape : TYPE IDENTIFIER ty_decl option(subtype) SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1034 : (('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 1034.
        Stack shape : VAR MINUS.
        Start symbol: ast. *)

    | MenhirState1036 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1036.
        Stack shape : VAR MINUS ty SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1038 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 1038.
        Stack shape : VAR MINUS ty.
        Start symbol: ast. *)

    | MenhirState1039 : (((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 1039.
        Stack shape : VAR MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState1040 : ((((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1040.
        Stack shape : VAR MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1042 : (('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_state
      (** State 1042.
        Stack shape : VAR MINUS.
        Start symbol: ast. *)

    | MenhirState1044 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1044.
        Stack shape : VAR MINUS ty SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1046 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 1046.
        Stack shape : VAR MINUS ty.
        Start symbol: ast. *)

    | MenhirState1047 : (((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 1047.
        Stack shape : VAR MINUS ty expr.
        Start symbol: ast. *)

    | MenhirState1048 : ((((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1048.
        Stack shape : VAR MINUS ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1051 : (('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 1051.
        Stack shape : VAR IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState1052 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 1052.
        Stack shape : VAR IDENTIFIER expr.
        Start symbol: ast. *)

    | MenhirState1053 : (((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1053.
        Stack shape : VAR IDENTIFIER expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1055 : (('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 1055.
        Stack shape : VAR IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState1057 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1057.
        Stack shape : VAR IDENTIFIER ty SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1059 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 1059.
        Stack shape : VAR IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState1060 : (((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 1060.
        Stack shape : VAR IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState1061 : ((((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1061.
        Stack shape : VAR IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1063 : (('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_state
      (** State 1063.
        Stack shape : VAR IDENTIFIER.
        Start symbol: ast. *)

    | MenhirState1065 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1065.
        Stack shape : VAR IDENTIFIER ty SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1067 : ((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_state
      (** State 1067.
        Stack shape : VAR IDENTIFIER ty.
        Start symbol: ast. *)

    | MenhirState1068 : (((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_state
      (** State 1068.
        Stack shape : VAR IDENTIFIER ty expr.
        Start symbol: ast. *)

    | MenhirState1069 : ((((('s, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON, _menhir_box_ast) _menhir_state
      (** State 1069.
        Stack shape : VAR IDENTIFIER ty expr SEMI_COLON.
        Start symbol: ast. *)

    | MenhirState1074 : ('s, _menhir_box_opn) _menhir_state
      (** State 1074.
        Stack shape : .
        Start symbol: opn. *)

    | MenhirState1075 : (('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_state
      (** State 1075.
        Stack shape : WHILE.
        Start symbol: opn. *)

    | MenhirState1076 : ((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1076.
        Stack shape : WHILE expr.
        Start symbol: opn. *)

    | MenhirState1077 : (((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 1077.
        Stack shape : WHILE expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState1078 : ((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1078.
        Stack shape : WHILE expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState1079 : (((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1079.
        Stack shape : WHILE expr LOOPLIMIT expr DO.
        Start symbol: opn. *)

    | MenhirState1080 : ((((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1080.
        Stack shape : WHILE expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1083 : (((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1083.
        Stack shape : WHILE expr DO.
        Start symbol: opn. *)

    | MenhirState1084 : ((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1084.
        Stack shape : WHILE expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1087 : (('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_state
      (** State 1087.
        Stack shape : VAR.
        Start symbol: opn. *)

    | MenhirState1089 : ((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1089.
        Stack shape : VAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1091 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, _menhir_box_opn) _menhir_state
      (** State 1091.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: opn. *)

    | MenhirState1095 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, _menhir_box_opn) _menhir_state
      (** State 1095.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: opn. *)

    | MenhirState1102 : ((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_state
      (** State 1102.
        Stack shape : VAR decl_item.
        Start symbol: opn. *)

    | MenhirState1104 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 1104.
        Stack shape : VAR decl_item IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState1106 : ((((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1106.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: opn. *)

    | MenhirState1114 : ((((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_state
      (** State 1114.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: opn. *)

    | MenhirState1116 : (((((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 1116.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState1120 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1120.
        Stack shape : VAR decl_item expr.
        Start symbol: opn. *)

    | MenhirState1128 : (('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_state
      (** State 1128.
        Stack shape : TRY.
        Start symbol: opn. *)

    | MenhirState1130 : ((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1130.
        Stack shape : TRY nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1131 : (((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_state
      (** State 1131.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher).
        Start symbol: opn. *)

    | MenhirState1133 : ((((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE, _menhir_box_opn) _menhir_state
      (** State 1133.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE.
        Start symbol: opn. *)

    | MenhirState1134 : (((((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1134.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1139 : (('s, _menhir_box_opn) _menhir_cell1_THROW, _menhir_box_opn) _menhir_state
      (** State 1139.
        Stack shape : THROW.
        Start symbol: opn. *)

    | MenhirState1142 : ((('s, _menhir_box_opn) _menhir_cell1_THROW, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1142.
        Stack shape : THROW expr.
        Start symbol: opn. *)

    | MenhirState1145 : (('s, _menhir_box_opn) _menhir_cell1_RETURN, _menhir_box_opn) _menhir_state
      (** State 1145.
        Stack shape : RETURN.
        Start symbol: opn. *)

    | MenhirState1148 : ((('s, _menhir_box_opn) _menhir_cell1_RETURN, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1148.
        Stack shape : RETURN expr.
        Start symbol: opn. *)

    | MenhirState1151 : (('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_state
      (** State 1151.
        Stack shape : REPEAT.
        Start symbol: opn. *)

    | MenhirState1153 : ((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1153.
        Stack shape : REPEAT nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1154 : (((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1154.
        Stack shape : REPEAT nonempty_list(stmt) expr.
        Start symbol: opn. *)

    | MenhirState1157 : ((((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 1157.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState1158 : (((((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1158.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState1162 : (('s, _menhir_box_opn) _menhir_cell1_PRINT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1162.
        Stack shape : PRINT LPAR.
        Start symbol: opn. *)

    | MenhirState1171 : (('s, _menhir_box_opn) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1171.
        Stack shape : PRAGMA IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1180 : (('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_state
      (** State 1180.
        Stack shape : LET.
        Start symbol: opn. *)

    | MenhirState1182 : ((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_state
      (** State 1182.
        Stack shape : LET decl_item.
        Start symbol: opn. *)

    | MenhirState1184 : (((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 1184.
        Stack shape : LET decl_item IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState1186 : ((((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1186.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: opn. *)

    | MenhirState1194 : ((((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_state
      (** State 1194.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: opn. *)

    | MenhirState1196 : (((((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 1196.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState1200 : (((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1200.
        Stack shape : LET decl_item expr.
        Start symbol: opn. *)

    | MenhirState1203 : (('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_state
      (** State 1203.
        Stack shape : IF.
        Start symbol: opn. *)

    | MenhirState1204 : ((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1204.
        Stack shape : IF expr.
        Start symbol: opn. *)

    | MenhirState1205 : (((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_state
      (** State 1205.
        Stack shape : IF expr THEN.
        Start symbol: opn. *)

    | MenhirState1206 : ((((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1206.
        Stack shape : IF expr THEN nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1207 : (((((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_s_else, _menhir_box_opn) _menhir_state
      (** State 1207.
        Stack shape : IF expr THEN nonempty_list(stmt) s_else.
        Start symbol: opn. *)

    | MenhirState1211 : (('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1211.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: opn. *)

    | MenhirState1215 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR, _menhir_box_opn) _menhir_state
      (** State 1215.
        Stack shape : IDENTIFIER LPAR RPAR.
        Start symbol: opn. *)

    | MenhirState1216 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1216.
        Stack shape : IDENTIFIER LPAR RPAR expr.
        Start symbol: opn. *)

    | MenhirState1222 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1222.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1225 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_state
      (** State 1225.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: opn. *)

    | MenhirState1226 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1226.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: opn. *)

    | MenhirState1230 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1230.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1231 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1231.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState1238 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, _menhir_box_opn) _menhir_state
      (** State 1238.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR.
        Start symbol: opn. *)

    | MenhirState1239 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1239.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR expr.
        Start symbol: opn. *)

    | MenhirState1245 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1245.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1248 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_state
      (** State 1248.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: opn. *)

    | MenhirState1249 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1249.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: opn. *)

    | MenhirState1253 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1253.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1254 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1254.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState1257 : (('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 1257.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState1259 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 1259.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState1263 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__, _menhir_box_opn) _menhir_state
      (** State 1263.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)).
        Start symbol: opn. *)

    | MenhirState1264 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1264.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) expr.
        Start symbol: opn. *)

    | MenhirState1270 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1270.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1273 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_state
      (** State 1273.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: opn. *)

    | MenhirState1274 : (((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1274.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: opn. *)

    | MenhirState1278 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1278.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1279 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1279.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState1284 : (('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 1284.
        Stack shape : FOR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState1285 : ((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1285.
        Stack shape : FOR IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState1286 : (((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_state
      (** State 1286.
        Stack shape : FOR IDENTIFIER expr TO.
        Start symbol: opn. *)

    | MenhirState1287 : ((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1287.
        Stack shape : FOR IDENTIFIER expr TO expr.
        Start symbol: opn. *)

    | MenhirState1288 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 1288.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState1289 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1289.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState1290 : (((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1290.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO.
        Start symbol: opn. *)

    | MenhirState1291 : ((((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1291.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1294 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1294.
        Stack shape : FOR IDENTIFIER expr TO expr DO.
        Start symbol: opn. *)

    | MenhirState1295 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1295.
        Stack shape : FOR IDENTIFIER expr TO expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1298 : (((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_state
      (** State 1298.
        Stack shape : FOR IDENTIFIER expr DOWNTO.
        Start symbol: opn. *)

    | MenhirState1299 : ((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1299.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr.
        Start symbol: opn. *)

    | MenhirState1300 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 1300.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState1301 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1301.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState1302 : (((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1302.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO.
        Start symbol: opn. *)

    | MenhirState1303 : ((((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1303.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1306 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1306.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO.
        Start symbol: opn. *)

    | MenhirState1307 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1307.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1311 : (('s, _menhir_box_opn) _menhir_cell1_DEBUG _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1311.
        Stack shape : DEBUG LPAR.
        Start symbol: opn. *)

    | MenhirState1319 : (('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_state
      (** State 1319.
        Stack shape : CONSTANT.
        Start symbol: opn. *)

    | MenhirState1321 : ((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_state
      (** State 1321.
        Stack shape : CONSTANT decl_item.
        Start symbol: opn. *)

    | MenhirState1323 : (((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 1323.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState1325 : ((((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1325.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: opn. *)

    | MenhirState1333 : ((((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_state
      (** State 1333.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: opn. *)

    | MenhirState1335 : (((((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 1335.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState1339 : (((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1339.
        Stack shape : CONSTANT decl_item expr.
        Start symbol: opn. *)

    | MenhirState1342 : (('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_state
      (** State 1342.
        Stack shape : CASE.
        Start symbol: opn. *)

    | MenhirState1343 : ((('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1343.
        Stack shape : CASE expr.
        Start symbol: opn. *)

    | MenhirState1344 : (((('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF, _menhir_box_opn) _menhir_state
      (** State 1344.
        Stack shape : CASE expr OF.
        Start symbol: opn. *)

    | MenhirState1345 : ((((('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF, _menhir_box_opn) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, _menhir_box_opn) _menhir_state
      (** State 1345.
        Stack shape : CASE expr OF nlist_opt_terminated(annotated(case_alt),annotated(case_otherwise)).
        Start symbol: opn. *)

    | MenhirState1348 : (('s, _menhir_box_opn) _menhir_cell1_ASSERT, _menhir_box_opn) _menhir_state
      (** State 1348.
        Stack shape : ASSERT.
        Start symbol: opn. *)

    | MenhirState1349 : ((('s, _menhir_box_opn) _menhir_cell1_ASSERT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1349.
        Stack shape : ASSERT expr.
        Start symbol: opn. *)

    | MenhirState1353 : (('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1353.
        Stack shape : ARROBASE_LOOPLIMIT LPAR.
        Start symbol: opn. *)

    | MenhirState1354 : ((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1354.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr.
        Start symbol: opn. *)

    | MenhirState1356 : (((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_state
      (** State 1356.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE.
        Start symbol: opn. *)

    | MenhirState1357 : ((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1357.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr.
        Start symbol: opn. *)

    | MenhirState1358 : (((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1358.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO.
        Start symbol: opn. *)

    | MenhirState1359 : ((((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1359.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1362 : (((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_state
      (** State 1362.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT.
        Start symbol: opn. *)

    | MenhirState1364 : ((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1364.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1365 : (((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1365.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt) expr.
        Start symbol: opn. *)

    | MenhirState1370 : (('s, _menhir_box_opn) _menhir_cell1_lexpr, _menhir_box_opn) _menhir_state
      (** State 1370.
        Stack shape : lexpr.
        Start symbol: opn. *)

    | MenhirState1371 : ((('s, _menhir_box_opn) _menhir_cell1_lexpr, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1371.
        Stack shape : lexpr expr.
        Start symbol: opn. *)

    | MenhirState1374 : ('s, _menhir_box_stmts) _menhir_state
      (** State 1374.
        Stack shape : .
        Start symbol: stmts. *)

  
  and ('s, 'r) _menhir_cell1_decl_item = 
    | MenhirCell1_decl_item of 's * ('s, 'r) _menhir_state * (AST.local_decl_item)
  
  and ('s, 'r) _menhir_cell1_end_semicolon = 
    | MenhirCell1_end_semicolon of 's * ('s, 'r) _menhir_state * (unit) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_expr = 
    | MenhirCell1_expr of 's * ('s, 'r) _menhir_state * (AST.expr) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_expr_pattern = 
    | MenhirCell1_expr_pattern of 's * ('s, 'r) _menhir_state * (AST.expr) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_lexpr = 
    | MenhirCell1_lexpr of 's * ('s, 'r) _menhir_state * (AST.lexpr) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_lexpr_atom_desc = 
    | MenhirCell1_lexpr_atom_desc of 's * ('s, 'r) _menhir_state * (AST.lexpr_desc) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_loption___anonymous_22_ = 
    | MenhirCell1_loption___anonymous_22_ of 's * ('s, 'r) _menhir_state * (AST.field list) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ = 
    | MenhirCell1_loption_braced_nclist_opt_typed_identifier___ of 's * ('s, 'r) _menhir_state * ((string * AST.ty option) list)
  
  and ('s, 'r) _menhir_cell1_loption_non_empty_trailing_list_COMMA_bitfield__ = 
    | MenhirCell1_loption_non_empty_trailing_list_COMMA_bitfield__ of 's * ('s, 'r) _menhir_state * (AST.bitfield list)
  
  and ('s, 'r) _menhir_cell1_loption_plist_expr__ = 
    | MenhirCell1_loption_plist_expr__ of 's * ('s, 'r) _menhir_state * (AST.expr list) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ = 
    | MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ of 's * ('s, 'r) _menhir_state * (AST.case_alt list)
  
  and ('s, 'r) _menhir_cell1_nonempty_list_catcher_ = 
    | MenhirCell1_nonempty_list_catcher_ of 's * ('s, 'r) _menhir_state * (AST.catcher list)
  
  and ('s, 'r) _menhir_cell1_nonempty_list_stmt_ = 
    | MenhirCell1_nonempty_list_stmt_ of 's * ('s, 'r) _menhir_state * (AST.stmt list) * Lexing.position
  
  and 's _menhir_cell0_option_subtype_ = 
    | MenhirCell0_option_subtype_ of 's * ((string * AST.field list) option)
  
  and ('s, 'r) _menhir_cell1_pattern = 
    | MenhirCell1_pattern of 's * ('s, 'r) _menhir_state * (AST.pattern) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_s_else = 
    | MenhirCell1_s_else of 's * ('s, 'r) _menhir_state * (AST.stmt) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ of 's * ('s, 'r) _menhir_state * (string list)
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_decl_item_ of 's * ('s, 'r) _menhir_state * (AST.local_decl_item list)
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_expr_ of 's * ('s, 'r) _menhir_state * (AST.expr list) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_pattern_ of 's * ('s, 'r) _menhir_state * (AST.pattern list) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_slice_ of 's * ('s, 'r) _menhir_state * (AST.slice list)
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ of 's * ('s, 'r) _menhir_state * (AST.typed_identifier list)
  
  and ('s, 'r) _menhir_cell1_ty = 
    | MenhirCell1_ty of 's * ('s, 'r) _menhir_state * (AST.ty) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ty_decl = 
    | MenhirCell1_ty_decl of 's * ('s, 'r) _menhir_state * (AST.ty)
  
  and ('s, 'r) _menhir_cell1_AND = 
    | MenhirCell1_AND of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ARRAY = 
    | MenhirCell1_ARRAY of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT = 
    | MenhirCell1_ARROBASE_LOOPLIMIT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ARROW = 
    | MenhirCell1_ARROW of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_AS = 
    | MenhirCell1_AS of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ASSERT = 
    | MenhirCell1_ASSERT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BAND = 
    | MenhirCell1_BAND of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_BEGIN = 
    | MenhirCell1_BEGIN of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_BEGIN = 
    | MenhirCell0_BEGIN of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BEQ = 
    | MenhirCell1_BEQ of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_BITS = 
    | MenhirCell1_BITS of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BNOT = 
    | MenhirCell1_BNOT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_BNOT = 
    | MenhirCell0_BNOT of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BOR = 
    | MenhirCell1_BOR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_CASE = 
    | MenhirCell1_CASE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_COLON = 
    | MenhirCell1_COLON of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_COLON_COLON = 
    | MenhirCell1_COLON_COLON of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_COMMA = 
    | MenhirCell1_COMMA of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_CONCAT = 
    | MenhirCell1_CONCAT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_CONFIG = 
    | MenhirCell1_CONFIG of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_CONSTANT = 
    | MenhirCell1_CONSTANT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_DEBUG = 
    | MenhirCell1_DEBUG of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_DIV = 
    | MenhirCell1_DIV of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DIVRM = 
    | MenhirCell1_DIVRM of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DO = 
    | MenhirCell1_DO of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DOT = 
    | MenhirCell1_DOT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DOWNTO = 
    | MenhirCell1_DOWNTO of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ELSE = 
    | MenhirCell1_ELSE of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ELSIF = 
    | MenhirCell1_ELSIF of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_END = 
    | MenhirCell1_END of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ENUMERATION = 
    | MenhirCell1_ENUMERATION of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_EOR = 
    | MenhirCell1_EOR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_EQ_OP = 
    | MenhirCell1_EQ_OP of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_EXCEPTION = 
    | MenhirCell1_EXCEPTION of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_FOR = 
    | MenhirCell1_FOR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_FUNC = 
    | MenhirCell1_FUNC of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_GEQ = 
    | MenhirCell1_GEQ of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_GEQ = 
    | MenhirCell0_GEQ of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_GETTER = 
    | MenhirCell1_GETTER of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_GT = 
    | MenhirCell1_GT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_IDENTIFIER = 
    | MenhirCell1_IDENTIFIER of 's * ('s, 'r) _menhir_state * (
# 40 "asllib/Tokens.mly"
       (string)
# 3849 "asllib/Parser.ml"
  ) * Lexing.position * Lexing.position
  
  and 's _menhir_cell0_IDENTIFIER = 
    | MenhirCell0_IDENTIFIER of 's * (
# 40 "asllib/Tokens.mly"
       (string)
# 3856 "asllib/Parser.ml"
  ) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_IF = 
    | MenhirCell1_IF of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_IMPL = 
    | MenhirCell1_IMPL of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_IN = 
    | MenhirCell1_IN of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_INTEGER = 
    | MenhirCell1_INTEGER of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LBRACE = 
    | MenhirCell1_LBRACE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_LBRACE = 
    | MenhirCell0_LBRACE of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LBRACKET = 
    | MenhirCell1_LBRACKET of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_LEQ = 
    | MenhirCell1_LEQ of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_LEQ = 
    | MenhirCell0_LEQ of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LET = 
    | MenhirCell1_LET of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LLBRACKET = 
    | MenhirCell1_LLBRACKET of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_LOOPLIMIT = 
    | MenhirCell1_LOOPLIMIT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_LPAR = 
    | MenhirCell1_LPAR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_LPAR = 
    | MenhirCell0_LPAR of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LT = 
    | MenhirCell1_LT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_MINUS = 
    | MenhirCell1_MINUS of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and 's _menhir_cell0_MINUS = 
    | MenhirCell0_MINUS of 's * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_MOD = 
    | MenhirCell1_MOD of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_MUL = 
    | MenhirCell1_MUL of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_NEQ = 
    | MenhirCell1_NEQ of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_NOT = 
    | MenhirCell1_NOT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_OF = 
    | MenhirCell1_OF of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_OR = 
    | MenhirCell1_OR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_OTHERWISE = 
    | MenhirCell1_OTHERWISE of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_PASS = 
    | MenhirCell1_PASS of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_PLUS = 
    | MenhirCell1_PLUS of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_PLUS_COLON = 
    | MenhirCell1_PLUS_COLON of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_POW = 
    | MenhirCell1_POW of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_PRAGMA = 
    | MenhirCell1_PRAGMA of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_PRINT = 
    | MenhirCell1_PRINT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RBRACE = 
    | MenhirCell1_RBRACE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_RBRACE = 
    | MenhirCell0_RBRACE of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RBRACKET = 
    | MenhirCell1_RBRACKET of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_RBRACKET = 
    | MenhirCell0_RBRACKET of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RDIV = 
    | MenhirCell1_RDIV of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_RECORD = 
    | MenhirCell1_RECORD of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_REPEAT = 
    | MenhirCell1_REPEAT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_REPEAT = 
    | MenhirCell0_REPEAT of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RETURN = 
    | MenhirCell1_RETURN of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RPAR = 
    | MenhirCell1_RPAR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_RPAR = 
    | MenhirCell0_RPAR of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_SEMI_COLON = 
    | MenhirCell1_SEMI_COLON of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_SEMI_COLON = 
    | MenhirCell0_SEMI_COLON of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_SETTER = 
    | MenhirCell1_SETTER of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_SHL = 
    | MenhirCell1_SHL of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_SHR = 
    | MenhirCell1_SHR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_SLICING = 
    | MenhirCell1_SLICING of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_STAR_COLON = 
    | MenhirCell1_STAR_COLON of 's * ('s, 'r) _menhir_state
  
  and 's _menhir_cell0_SUBTYPES = 
    | MenhirCell0_SUBTYPES of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_THEN = 
    | MenhirCell1_THEN of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_THROW = 
    | MenhirCell1_THROW of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_TO = 
    | MenhirCell1_TO of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_TRY = 
    | MenhirCell1_TRY of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_TYPE = 
    | MenhirCell1_TYPE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_UNKNOWN = 
    | MenhirCell1_UNKNOWN of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_UNREACHABLE = 
    | MenhirCell1_UNREACHABLE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_VAR = 
    | MenhirCell1_VAR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_WHEN = 
    | MenhirCell1_WHEN of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_WHILE = 
    | MenhirCell1_WHILE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_WHILE = 
    | MenhirCell0_WHILE of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_WITH = 
    | MenhirCell1_WITH of 's * ('s, 'r) _menhir_state
  
  and _menhir_box_stmts = 
    | MenhirBox_stmts of (AST.stmt) [@@unboxed]
  
  and _menhir_box_opn = 
    | MenhirBox_opn of (AST.t) [@@unboxed]
  
  and _menhir_box_ast = 
    | MenhirBox_ast of (AST.t) [@@unboxed]
  
  let _menhir_action_003 =
    fun x ->
      (
# 196 "<standard.mly>"
    ( x )
# 4056 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_004 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item =
        let _1_1 = _1 in
        
# 415 "asllib/Parser.mly"
                              LDI_Var      
# 4070 "asllib/Parser.ml"
         _1_1
      in
      (
# 411 "asllib/Parser.mly"
                                      LDI_Typed    
# 4076 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_005 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item =
        let _1_1 = _1 in
        
# 415 "asllib/Parser.mly"
                              LDI_Var      
# 4090 "asllib/Parser.ml"
         _1_1
      in
      (
# 411 "asllib/Parser.mly"
                                      LDI_Typed    
# 4096 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_006 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item = 
# 416 "asllib/Parser.mly"
                            ( LDI_Discard )
# 4108 "asllib/Parser.ml"
       in
      (
# 411 "asllib/Parser.mly"
                                      LDI_Typed    
# 4113 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_007 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item = 
# 416 "asllib/Parser.mly"
                            ( LDI_Discard )
# 4125 "asllib/Parser.ml"
       in
      (
# 411 "asllib/Parser.mly"
                                      LDI_Typed    
# 4130 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_008 =
    fun _1_inlined1 _2 _2_inlined1 li x ->
      let as_ty =
        let (_2, _1) = (_2_inlined1, _1_inlined1) in
        _2
      in
      let untyped_decl_item =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 4144 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 4149 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 417 "asllib/Parser.mly"
                              LDI_Tuple    
# 4156 "asllib/Parser.ml"
         _1_1
      in
      (
# 411 "asllib/Parser.mly"
                                      LDI_Typed    
# 4162 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_009 =
    fun _1_inlined1 _2 _2_inlined1 li x ->
      let as_ty =
        let (_2, _1) = (_2_inlined1, _1_inlined1) in
        _2
      in
      let untyped_decl_item =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 4176 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 4181 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 417 "asllib/Parser.mly"
                              LDI_Tuple    
# 4188 "asllib/Parser.ml"
         _1_1
      in
      (
# 411 "asllib/Parser.mly"
                                      LDI_Typed    
# 4194 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_010 =
    fun _1 ->
      let _1_1 = _1 in
      (
# 415 "asllib/Parser.mly"
                              LDI_Var      
# 4203 "asllib/Parser.ml"
       _1_1 : (AST.local_decl_item))
  
  let _menhir_action_011 =
    fun _1 ->
      (
# 416 "asllib/Parser.mly"
                            ( LDI_Discard )
# 4211 "asllib/Parser.ml"
       : (AST.local_decl_item))
  
  let _menhir_action_012 =
    fun _2 li x ->
      let _1 =
        let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 4220 "asllib/Parser.ml"
         in
        
# 205 "<standard.mly>"
    ( x )
# 4225 "asllib/Parser.ml"
        
      in
      let _1_1 = _1 in
      (
# 417 "asllib/Parser.mly"
                              LDI_Tuple    
# 4232 "asllib/Parser.ml"
       _1_1 : (AST.local_decl_item))
  
  let _menhir_action_013 =
    fun _1 _2 ->
      (_2 : (AST.expr))
  
  let _menhir_action_014 =
    fun _1 _3 _endpos_e_else_ _startpos__1_ c e e_else ->
      let desc = 
# 252 "asllib/Parser.mly"
                                                        E_Cond 
# 4244 "asllib/Parser.ml"
       (c, e, e_else) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_else_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4252 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_015 =
    fun _1 _endpos_opt_ _startpos__1_ opt ->
      let _endpos = _endpos_opt_ in
      let _startpos = _startpos__1_ in
      (
# 187 "asllib/Parser.mly"
                                                  (
      if Option.is_none opt && not Config.allow_no_end_semicolon then
          Error.fatal_here _startpos _endpos @@ Error.ObsoleteSyntax "Missing ';' after 'end' keyword.";
  )
# 4265 "asllib/Parser.ml"
       : (unit))
  
  let _menhir_action_016 =
    fun _endpos_i_ _startpos_i_ i ->
      let desc =
        let value = 
# 200 "asllib/Parser.mly"
                        L_Int          
# 4274 "asllib/Parser.ml"
         i in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4279 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_i_, _startpos_i_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4288 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_017 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 201 "asllib/Parser.mly"
                        L_Bool         
# 4297 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4302 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4311 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_018 =
    fun _endpos_r_ _startpos_r_ r ->
      let desc =
        let value = 
# 202 "asllib/Parser.mly"
                        L_Real         
# 4320 "asllib/Parser.ml"
         r in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4325 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_r_, _startpos_r_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4334 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_019 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 203 "asllib/Parser.mly"
                        L_BitVector    
# 4343 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4348 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4357 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_020 =
    fun _endpos_s_ _startpos_s_ s ->
      let desc =
        let value = 
# 204 "asllib/Parser.mly"
                        L_String       
# 4366 "asllib/Parser.ml"
         s in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4371 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_s_, _startpos_s_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4380 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_021 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let _1_1 = _1 in
        
# 265 "asllib/Parser.mly"
                                                                    E_Var                 
# 4390 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4399 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_022 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 214 "asllib/Parser.mly"
                  ( AND    )
# 4408 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4413 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4422 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_023 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 215 "asllib/Parser.mly"
                  ( BAND   )
# 4431 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4436 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4445 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_024 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 216 "asllib/Parser.mly"
                  ( BOR    )
# 4454 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4459 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4468 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_025 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 217 "asllib/Parser.mly"
                  ( EQ_OP  )
# 4477 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4482 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4491 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_026 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 218 "asllib/Parser.mly"
                  ( DIV    )
# 4500 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4505 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4514 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_027 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 219 "asllib/Parser.mly"
                  ( DIVRM  )
# 4523 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4528 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4537 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_028 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 220 "asllib/Parser.mly"
                  ( EOR    )
# 4546 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4551 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4560 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_029 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 221 "asllib/Parser.mly"
                  ( EQ_OP  )
# 4569 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4574 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4583 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_030 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 222 "asllib/Parser.mly"
                  ( NEQ    )
# 4592 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4597 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4606 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_031 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 223 "asllib/Parser.mly"
                  ( GT     )
# 4615 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4620 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4629 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_032 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 224 "asllib/Parser.mly"
                  ( GEQ    )
# 4638 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4643 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4652 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_033 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 225 "asllib/Parser.mly"
                  ( IMPL   )
# 4661 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4666 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4675 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_034 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 226 "asllib/Parser.mly"
                  ( LT     )
# 4684 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4689 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4698 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_035 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 227 "asllib/Parser.mly"
                  ( LEQ    )
# 4707 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4712 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4721 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_036 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 228 "asllib/Parser.mly"
                  ( PLUS   )
# 4730 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4735 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4744 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_037 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 229 "asllib/Parser.mly"
                  ( MINUS  )
# 4753 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4758 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4767 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_038 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 230 "asllib/Parser.mly"
                  ( MOD    )
# 4776 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4781 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4790 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_039 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 231 "asllib/Parser.mly"
                  ( MUL    )
# 4799 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4804 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4813 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_040 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 232 "asllib/Parser.mly"
                  ( OR     )
# 4822 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4827 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4836 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_041 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 233 "asllib/Parser.mly"
                  ( RDIV   )
# 4845 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4850 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4859 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_042 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 234 "asllib/Parser.mly"
                  ( SHL    )
# 4868 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4873 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4882 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_043 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 235 "asllib/Parser.mly"
                  ( SHR    )
# 4891 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4896 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4905 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_044 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 236 "asllib/Parser.mly"
                  ( POW    )
# 4914 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4919 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4928 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_045 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 237 "asllib/Parser.mly"
                  ( BV_CONCAT )
# 4937 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4942 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4951 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_046 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op =
          let _1 = () in
          
# 211 "asllib/Parser.mly"
                                  ( PLUS )
# 4962 "asllib/Parser.ml"
          
        in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4968 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4977 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_047 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 207 "asllib/Parser.mly"
            ( BNOT )
# 4986 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 4991 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5000 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_048 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 208 "asllib/Parser.mly"
            ( NEG )
# 5009 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 5014 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5023 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_049 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 209 "asllib/Parser.mly"
            ( NOT )
# 5032 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 5037 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5046 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_050 =
    fun _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else ->
      let desc = 
# 268 "asllib/Parser.mly"
                                                                    E_Cond                
# 5054 "asllib/Parser.ml"
       (e1, e2, e_else) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_else_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5062 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_051 =
    fun _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 5073 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 5078 "asllib/Parser.ml"
            
          in
          
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 5084 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 5090 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5099 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_052 =
    fun _1_inlined1 _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 5111 "asllib/Parser.ml"
            
          in
          
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 5117 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 5123 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5132 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_053 =
    fun _1_inlined1 _endpos_args_ _startpos_name_ args name ->
      let desc =
        let call =
          let params =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 5144 "asllib/Parser.ml"
            
          in
          
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 5150 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 5156 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5165 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_054 =
    fun _1_inlined1 _endpos__3_ _startpos_e_ e ->
      let desc =
        let slices =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 5176 "asllib/Parser.ml"
          
        in
        
# 270 "asllib/Parser.mly"
                                                                    E_Slice               
# 5182 "asllib/Parser.ml"
         (e, slices)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5191 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_055 =
    fun _2 _4 _endpos__4_ _startpos_e1_ e1 e2 ->
      let desc = 
# 271 "asllib/Parser.mly"
                                                                    E_GetArray            
# 5199 "asllib/Parser.ml"
       (e1, e2) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__4_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5207 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_056 =
    fun _2 _endpos_x_ _startpos_e_ e x ->
      let desc = 
# 272 "asllib/Parser.mly"
                                                                    E_GetField            
# 5215 "asllib/Parser.ml"
       (e, x) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_x_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5223 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_057 =
    fun _1_inlined1 _2 _endpos__3_ _startpos_e_ e ->
      let desc =
        let fs =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 5234 "asllib/Parser.ml"
          
        in
        
# 273 "asllib/Parser.mly"
                                                                    E_GetFields           
# 5240 "asllib/Parser.ml"
         (e, fs)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5249 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_058 =
    fun _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty ->
      let desc = 
# 275 "asllib/Parser.mly"
                                                                    E_ATC                 
# 5257 "asllib/Parser.ml"
       (sub_expr, ty) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5265 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_059 =
    fun _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 5278 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 5283 "asllib/Parser.ml"
                
              in
              let _1_1 = _1 in
              
# 300 "asllib/Parser.mly"
                                                          WellConstrained  
# 5290 "asllib/Parser.ml"
               _1_1
            in
            
# 380 "asllib/Parser.mly"
                                                       T_Int 
# 5296 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5305 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 5311 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5320 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_060 =
    fun _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 5334 "asllib/Parser.ml"
                
              in
              let _1_1 = _1 in
              
# 300 "asllib/Parser.mly"
                                                          WellConstrained  
# 5341 "asllib/Parser.ml"
               _1_1
            in
            
# 380 "asllib/Parser.mly"
                                                       T_Int 
# 5347 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5356 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 5362 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5371 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_061 =
    fun _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let x =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 5385 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5394 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 5400 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 5406 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5415 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_062 =
    fun _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let desc =
            let _2 =
              let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) = (_endpos__1_inlined2_, _startpos__1_inlined2_, _1_inlined2) in
              let x =
                let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
                let desc =
                  let _1_1 = _1 in
                  
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 5432 "asllib/Parser.ml"
                   _1_1
                in
                let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
                let _endpos = _endpos_desc_ in
                let _symbolstartpos = _startpos_desc_ in
                
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5441 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 5447 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 326 "asllib/Parser.mly"
                                      Pattern_Not  
# 5454 "asllib/Parser.ml"
             _2_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5463 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 5469 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5478 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_063 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_mask =
          let desc =
            let _1_1 = _1 in
            
# 308 "asllib/Parser.mly"
                                            Pattern_Mask  
# 5490 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5499 "asllib/Parser.ml"
          
        in
        
# 279 "asllib/Parser.mly"
                                                                    E_Pattern             
# 5505 "asllib/Parser.ml"
         (sub_expr, pattern_mask)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5514 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_064 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e ->
      let desc =
        let p =
          let desc =
            let _1_1 = _1 in
            
# 308 "asllib/Parser.mly"
                                            Pattern_Mask  
# 5526 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5535 "asllib/Parser.ml"
          
        in
        
# 280 "asllib/Parser.mly"
                                                                  ( E_Pattern (e, Pattern_Not (p) |> add_pos_from p) )
# 5541 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5550 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_065 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                    E_Unknown             
# 5560 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5569 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_066 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                    E_Unknown             
# 5579 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5588 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_067 =
    fun _endpos__3_ _startpos__1_ x ->
      let desc =
        let e = 
# 205 "<standard.mly>"
    ( x )
# 5597 "asllib/Parser.ml"
         in
        
# 283 "asllib/Parser.mly"
                                                                  ( E_Tuple [ e ]        )
# 5602 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5611 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_068 =
    fun _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 5621 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 5626 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5635 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 5641 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5650 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_069 =
    fun _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 5661 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5670 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 5676 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5685 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_070 =
    fun _2 _endpos__3_ _startpos__1_ li x ->
      let desc =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 5695 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 5700 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 257 "asllib/Parser.mly"
                                                                    E_Tuple               
# 5707 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5716 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_071 =
    fun _endpos_i_ _startpos_i_ i ->
      let desc =
        let value = 
# 200 "asllib/Parser.mly"
                        L_Int          
# 5725 "asllib/Parser.ml"
         i in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5730 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_i_, _startpos_i_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5739 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_072 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 201 "asllib/Parser.mly"
                        L_Bool         
# 5748 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5753 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5762 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_073 =
    fun _endpos_r_ _startpos_r_ r ->
      let desc =
        let value = 
# 202 "asllib/Parser.mly"
                        L_Real         
# 5771 "asllib/Parser.ml"
         r in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5776 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_r_, _startpos_r_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5785 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_074 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 203 "asllib/Parser.mly"
                        L_BitVector    
# 5794 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5799 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5808 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_075 =
    fun _endpos_s_ _startpos_s_ s ->
      let desc =
        let value = 
# 204 "asllib/Parser.mly"
                        L_String       
# 5817 "asllib/Parser.ml"
         s in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5822 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_s_, _startpos_s_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5831 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_076 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let _1_1 = _1 in
        
# 265 "asllib/Parser.mly"
                                                                    E_Var                 
# 5841 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5850 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_077 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 214 "asllib/Parser.mly"
                  ( AND    )
# 5859 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5864 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5873 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_078 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 215 "asllib/Parser.mly"
                  ( BAND   )
# 5882 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5887 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5896 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_079 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 216 "asllib/Parser.mly"
                  ( BOR    )
# 5905 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5910 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5919 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_080 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 217 "asllib/Parser.mly"
                  ( EQ_OP  )
# 5928 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5933 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5942 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_081 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 218 "asllib/Parser.mly"
                  ( DIV    )
# 5951 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5956 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5965 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_082 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 219 "asllib/Parser.mly"
                  ( DIVRM  )
# 5974 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5979 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5988 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_083 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 220 "asllib/Parser.mly"
                  ( EOR    )
# 5997 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6002 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6011 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_084 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 221 "asllib/Parser.mly"
                  ( EQ_OP  )
# 6020 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6025 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6034 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_085 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 222 "asllib/Parser.mly"
                  ( NEQ    )
# 6043 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6048 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6057 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_086 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 223 "asllib/Parser.mly"
                  ( GT     )
# 6066 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6071 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6080 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_087 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 224 "asllib/Parser.mly"
                  ( GEQ    )
# 6089 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6094 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6103 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_088 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 225 "asllib/Parser.mly"
                  ( IMPL   )
# 6112 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6117 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6126 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_089 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 226 "asllib/Parser.mly"
                  ( LT     )
# 6135 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6140 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6149 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_090 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 227 "asllib/Parser.mly"
                  ( LEQ    )
# 6158 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6163 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6172 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_091 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 228 "asllib/Parser.mly"
                  ( PLUS   )
# 6181 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6186 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6195 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_092 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 229 "asllib/Parser.mly"
                  ( MINUS  )
# 6204 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6209 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6218 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_093 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 230 "asllib/Parser.mly"
                  ( MOD    )
# 6227 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6232 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6241 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_094 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 231 "asllib/Parser.mly"
                  ( MUL    )
# 6250 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6255 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6264 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_095 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 232 "asllib/Parser.mly"
                  ( OR     )
# 6273 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6278 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6287 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_096 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 233 "asllib/Parser.mly"
                  ( RDIV   )
# 6296 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6301 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6310 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_097 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 234 "asllib/Parser.mly"
                  ( SHL    )
# 6319 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6324 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6333 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_098 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 235 "asllib/Parser.mly"
                  ( SHR    )
# 6342 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6347 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6356 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_099 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 236 "asllib/Parser.mly"
                  ( POW    )
# 6365 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6370 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6379 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_100 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 237 "asllib/Parser.mly"
                  ( BV_CONCAT )
# 6388 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6393 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6402 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_101 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op =
          let _1 = () in
          
# 211 "asllib/Parser.mly"
                                  ( PLUS )
# 6413 "asllib/Parser.ml"
          
        in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6419 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6428 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_102 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 207 "asllib/Parser.mly"
            ( BNOT )
# 6437 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 6442 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6451 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_103 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 208 "asllib/Parser.mly"
            ( NEG )
# 6460 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 6465 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6474 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_104 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 209 "asllib/Parser.mly"
            ( NOT )
# 6483 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 6488 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6497 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_105 =
    fun _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else ->
      let desc = 
# 268 "asllib/Parser.mly"
                                                                    E_Cond                
# 6505 "asllib/Parser.ml"
       (e1, e2, e_else) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_else_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6513 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_106 =
    fun _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 6524 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 6529 "asllib/Parser.ml"
            
          in
          
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 6535 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 6541 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6550 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_107 =
    fun _1_inlined1 _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 6562 "asllib/Parser.ml"
            
          in
          
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 6568 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 6574 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6583 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_108 =
    fun _1_inlined1 _endpos_args_ _startpos_name_ args name ->
      let desc =
        let call =
          let params =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 6595 "asllib/Parser.ml"
            
          in
          
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 6601 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 6607 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6616 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_109 =
    fun _1_inlined1 _endpos__3_ _startpos_e_ e ->
      let desc =
        let slices =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 6627 "asllib/Parser.ml"
          
        in
        
# 270 "asllib/Parser.mly"
                                                                    E_Slice               
# 6633 "asllib/Parser.ml"
         (e, slices)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6642 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_110 =
    fun _2 _4 _endpos__4_ _startpos_e1_ e1 e2 ->
      let desc = 
# 271 "asllib/Parser.mly"
                                                                    E_GetArray            
# 6650 "asllib/Parser.ml"
       (e1, e2) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__4_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6658 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_111 =
    fun _2 _endpos_x_ _startpos_e_ e x ->
      let desc = 
# 272 "asllib/Parser.mly"
                                                                    E_GetField            
# 6666 "asllib/Parser.ml"
       (e, x) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_x_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6674 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_112 =
    fun _1_inlined1 _2 _endpos__3_ _startpos_e_ e ->
      let desc =
        let fs =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 6685 "asllib/Parser.ml"
          
        in
        
# 273 "asllib/Parser.mly"
                                                                    E_GetFields           
# 6691 "asllib/Parser.ml"
         (e, fs)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6700 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_113 =
    fun _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty ->
      let desc = 
# 275 "asllib/Parser.mly"
                                                                    E_ATC                 
# 6708 "asllib/Parser.ml"
       (sub_expr, ty) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6716 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_114 =
    fun _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 6729 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 6734 "asllib/Parser.ml"
                
              in
              let _1_1 = _1 in
              
# 300 "asllib/Parser.mly"
                                                          WellConstrained  
# 6741 "asllib/Parser.ml"
               _1_1
            in
            
# 380 "asllib/Parser.mly"
                                                       T_Int 
# 6747 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6756 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 6762 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6771 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_115 =
    fun _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 6785 "asllib/Parser.ml"
                
              in
              let _1_1 = _1 in
              
# 300 "asllib/Parser.mly"
                                                          WellConstrained  
# 6792 "asllib/Parser.ml"
               _1_1
            in
            
# 380 "asllib/Parser.mly"
                                                       T_Int 
# 6798 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6807 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 6813 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6822 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_116 =
    fun _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let x =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 6836 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6845 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 6851 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 6857 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6866 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_117 =
    fun _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let desc =
            let _2 =
              let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) = (_endpos__1_inlined2_, _startpos__1_inlined2_, _1_inlined2) in
              let x =
                let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
                let desc =
                  let _1_1 = _1 in
                  
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 6883 "asllib/Parser.ml"
                   _1_1
                in
                let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
                let _endpos = _endpos_desc_ in
                let _symbolstartpos = _startpos_desc_ in
                
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6892 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 6898 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 326 "asllib/Parser.mly"
                                      Pattern_Not  
# 6905 "asllib/Parser.ml"
             _2_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6914 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 6920 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6929 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_118 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_mask =
          let desc =
            let _1_1 = _1 in
            
# 308 "asllib/Parser.mly"
                                            Pattern_Mask  
# 6941 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6950 "asllib/Parser.ml"
          
        in
        
# 279 "asllib/Parser.mly"
                                                                    E_Pattern             
# 6956 "asllib/Parser.ml"
         (sub_expr, pattern_mask)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6965 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_119 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e ->
      let desc =
        let p =
          let desc =
            let _1_1 = _1 in
            
# 308 "asllib/Parser.mly"
                                            Pattern_Mask  
# 6977 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6986 "asllib/Parser.ml"
          
        in
        
# 280 "asllib/Parser.mly"
                                                                  ( E_Pattern (e, Pattern_Not (p) |> add_pos_from p) )
# 6992 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7001 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_120 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                    E_Unknown             
# 7011 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7020 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_121 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                    E_Unknown             
# 7030 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7039 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_122 =
    fun _endpos__3_ _startpos__1_ x ->
      let desc =
        let e = 
# 205 "<standard.mly>"
    ( x )
# 7048 "asllib/Parser.ml"
         in
        
# 283 "asllib/Parser.mly"
                                                                  ( E_Tuple [ e ]        )
# 7053 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7062 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_123 =
    fun _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7072 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 7077 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7086 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 7092 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7101 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_124 =
    fun _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 7112 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7121 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 7127 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7136 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_125 =
    fun _endpos_desc_ _startpos_desc_ desc ->
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7146 "asllib/Parser.ml"
       : (AST.lexpr))
  
  let _menhir_action_126 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 392 "asllib/Parser.mly"
                      ( LE_Discard )
# 7154 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7162 "asllib/Parser.ml"
       : (AST.lexpr))
  
  let _menhir_action_127 =
    fun _1_inlined1 _2 _endpos__3_ _startpos__1_ x ->
      let desc =
        let _1 =
          let x =
            let _1 = _1_inlined1 in
            let li = _1 in
            
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 7175 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 7181 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 394 "asllib/Parser.mly"
                               LE_Destructuring 
# 7188 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7197 "asllib/Parser.ml"
       : (AST.lexpr))
  
  let _menhir_action_128 =
    fun _1 ->
      let _1_1 = _1 in
      (
# 398 "asllib/Parser.mly"
                    LE_Var 
# 7206 "asllib/Parser.ml"
       _1_1 : (AST.lexpr_desc))
  
  let _menhir_action_129 =
    fun _1_inlined1 _endpos_desc_ _startpos_desc_ desc ->
      let slices =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 7216 "asllib/Parser.ml"
        
      in
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7225 "asllib/Parser.ml"
        
      in
      (
# 399 "asllib/Parser.mly"
                              LE_Slice 
# 7231 "asllib/Parser.ml"
       (le, slices) : (AST.lexpr_desc))
  
  let _menhir_action_130 =
    fun _2 _4 _endpos_desc_ _startpos_desc_ desc expr ->
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7242 "asllib/Parser.ml"
        
      in
      (
# 400 "asllib/Parser.mly"
                                                  LE_SetArray 
# 7248 "asllib/Parser.ml"
       (le, expr) : (AST.lexpr_desc))
  
  let _menhir_action_131 =
    fun _2 _endpos_desc_ _startpos_desc_ desc field ->
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7259 "asllib/Parser.ml"
        
      in
      (
# 401 "asllib/Parser.mly"
                                           LE_SetField 
# 7265 "asllib/Parser.ml"
       (le, field) : (AST.lexpr_desc))
  
  let _menhir_action_132 =
    fun _2 _endpos_desc_ _startpos_desc_ desc ->
      let li =
        let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7274 "asllib/Parser.ml"
         in
        
# 205 "<standard.mly>"
    ( x )
# 7279 "asllib/Parser.ml"
        
      in
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7288 "asllib/Parser.ml"
        
      in
      (
# 402 "asllib/Parser.mly"
                                                         ( LE_SetFields (le, li, []) )
# 7294 "asllib/Parser.ml"
       : (AST.lexpr_desc))
  
  let _menhir_action_133 =
    fun _1_inlined1 _2 _endpos_desc_ _startpos_desc_ desc ->
      let li =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 7304 "asllib/Parser.ml"
        
      in
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7313 "asllib/Parser.ml"
        
      in
      (
# 402 "asllib/Parser.mly"
                                                         ( LE_SetFields (le, li, []) )
# 7319 "asllib/Parser.ml"
       : (AST.lexpr_desc))
  
  let _menhir_action_134 =
    fun () ->
      (
# 216 "<standard.mly>"
    ( [] )
# 7327 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_135 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 7342 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7348 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7354 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7364 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7369 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7387 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7396 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7402 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_136 =
    fun _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7416 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7424 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7430 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7436 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7446 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7451 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7469 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7478 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7484 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_137 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 7499 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7505 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined4) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7514 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7525 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7530 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7548 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7557 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7563 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_138 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _2_inlined1 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7577 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7585 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7591 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined4) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7600 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7611 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7616 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7634 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7643 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7649 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_139 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 7664 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7670 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7676 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7688 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7706 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7715 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7721 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_140 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7735 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7743 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7749 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7755 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7767 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7785 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7794 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7800 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_141 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _1_inlined7 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined7 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 7815 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7821 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined5) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7830 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7843 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7861 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7870 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7876 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_142 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _2 _2_inlined1 _endpos__1_inlined6_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined6_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7890 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7898 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7904 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined5) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7913 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7926 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 544 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7944 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7953 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7959 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_143 =
    fun _1 _1_inlined1 _1_inlined4 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined4 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 7974 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7980 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7987 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7992 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 559 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8010 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8019 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8025 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_144 =
    fun _1 _1_inlined1 _endpos__1_inlined3_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined3_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8039 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8047 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8053 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8060 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8065 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 559 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8083 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8092 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8098 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_145 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 8113 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8119 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8128 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 559 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8146 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8155 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8161 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_146 =
    fun _1 _1_inlined1 _1_inlined3 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8175 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8183 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8189 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8198 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 559 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8216 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8225 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8231 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_147 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 8246 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8252 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8263 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8268 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 575 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8287 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8296 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8302 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_148 =
    fun _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8316 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8324 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8330 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8341 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8346 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 575 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8365 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8374 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8380 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_149 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 8395 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8401 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8414 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 575 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8433 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8442 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8448 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_150 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8462 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8470 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8476 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8489 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 575 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8508 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8517 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8523 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_151 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 8538 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8544 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8553 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8560 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8565 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8584 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8593 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8599 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_152 =
    fun _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8613 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8621 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8627 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8636 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8643 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8648 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8667 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8676 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8682 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_153 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 8697 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8703 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8712 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8719 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8724 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8743 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8752 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8758 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_154 =
    fun _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8772 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8780 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8786 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8795 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8802 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8807 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8826 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8835 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8841 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_155 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 8856 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8862 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8871 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8880 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8899 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8908 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8914 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_156 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8928 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8936 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8942 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8951 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8960 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8979 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8988 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8994 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_157 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 9009 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 9015 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 9024 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 9033 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 9052 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9061 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9067 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_158 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 533 "asllib/Parser.mly"
                                                   ( S_Pass )
# 9081 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9089 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 9095 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 9104 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 9113 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 592 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 9132 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9141 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9147 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_159 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ t x xs ->
      let x =
        let desc =
          let x =
            let subtype_opt = _1_inlined1 in
            
# 608 "asllib/Parser.mly"
                                                            D_TypeDecl            
# 9159 "asllib/Parser.ml"
             (x, t, subtype_opt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 9165 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9174 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9180 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_160 =
    fun _1 _1_inlined1 _2_inlined1 _3 _endpos__2_ _endpos__3_ _startpos__1_ _startpos__1_inlined1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let s =
              let (_startpos__1_, _1) = (_startpos__1_inlined1_, _1_inlined1) in
              let desc =
                let _2_1 = _2 in
                let _3_1 = _3 in
                (_2_1, _3_1)
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9202 "asllib/Parser.ml"
              
            in
            
# 611 "asllib/Parser.mly"
                                                            make_ty_decl_subtype  
# 9208 "asllib/Parser.ml"
             (x, s)
          in
          
# 196 "<standard.mly>"
    ( x )
# 9214 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9223 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9229 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_161 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9240 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9245 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9252 "asllib/Parser.ml"
              
            in
            let keyword = 
# 430 "asllib/Parser.mly"
                ( GDK_Let      )
# 9258 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9263 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9269 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9278 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9284 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_162 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9296 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9304 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9312 "asllib/Parser.ml"
              
            in
            let keyword = 
# 430 "asllib/Parser.mly"
                ( GDK_Let      )
# 9318 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9323 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9329 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9338 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9344 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_163 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9356 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9364 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9372 "asllib/Parser.ml"
              
            in
            let keyword = 
# 430 "asllib/Parser.mly"
                ( GDK_Let      )
# 9378 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9383 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9389 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9398 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9404 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_164 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9415 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9420 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 430 "asllib/Parser.mly"
                ( GDK_Let      )
# 9426 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9431 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9437 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9446 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9452 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_165 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9464 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9472 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 430 "asllib/Parser.mly"
                ( GDK_Let      )
# 9479 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9484 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9490 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9499 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9505 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_166 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9517 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9525 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 430 "asllib/Parser.mly"
                ( GDK_Let      )
# 9532 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9537 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9543 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9552 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9558 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_167 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9569 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9574 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9581 "asllib/Parser.ml"
              
            in
            let keyword = 
# 431 "asllib/Parser.mly"
                ( GDK_Constant )
# 9587 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9592 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9598 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9607 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9613 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_168 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9625 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9633 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9641 "asllib/Parser.ml"
              
            in
            let keyword = 
# 431 "asllib/Parser.mly"
                ( GDK_Constant )
# 9647 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9652 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9658 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9667 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9673 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_169 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9685 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9693 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9701 "asllib/Parser.ml"
              
            in
            let keyword = 
# 431 "asllib/Parser.mly"
                ( GDK_Constant )
# 9707 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9712 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9718 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9727 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9733 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_170 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9744 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9749 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 431 "asllib/Parser.mly"
                ( GDK_Constant )
# 9755 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9760 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9766 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9775 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9781 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_171 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9793 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9801 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 431 "asllib/Parser.mly"
                ( GDK_Constant )
# 9808 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9813 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9819 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9828 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9834 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_172 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9846 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9854 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 431 "asllib/Parser.mly"
                ( GDK_Constant )
# 9861 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9866 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9872 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9881 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9887 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_173 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9898 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9903 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9910 "asllib/Parser.ml"
              
            in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Var      )
# 9916 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9921 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9927 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9936 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9942 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_174 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9954 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9962 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 9970 "asllib/Parser.ml"
              
            in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Var      )
# 9976 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9981 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9987 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9996 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10002 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_175 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10014 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10022 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 10030 "asllib/Parser.ml"
              
            in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Var      )
# 10036 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10041 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10047 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10056 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10062 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_176 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10073 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 10078 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Var      )
# 10084 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10089 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10095 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10104 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10110 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_177 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10122 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10130 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Var      )
# 10137 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10142 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10148 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10157 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10163 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_178 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10175 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10183 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Var      )
# 10190 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10195 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10201 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10210 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10216 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_179 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10227 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 10232 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 10239 "asllib/Parser.ml"
              
            in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Config   )
# 10245 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10250 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10256 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10265 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10271 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_180 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10283 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10291 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 10299 "asllib/Parser.ml"
              
            in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Config   )
# 10305 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10310 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10316 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10325 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10331 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_181 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10343 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10351 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 10359 "asllib/Parser.ml"
              
            in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Config   )
# 10365 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10370 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10376 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10385 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10391 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_182 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10402 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 10407 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Config   )
# 10413 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10418 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10424 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10433 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10439 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_183 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10451 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10459 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Config   )
# 10466 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10471 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10477 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10486 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10492 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_184 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10504 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10512 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Config   )
# 10519 "asllib/Parser.ml"
             in
            
# 616 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10524 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10530 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10539 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10545 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_185 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10560 "asllib/Parser.ml"
               x
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 10568 "asllib/Parser.ml"
              
            in
            
# 620 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10574 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10580 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10589 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10595 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_186 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10610 "asllib/Parser.ml"
               x
            in
            let name =
              let _1 = _1_inlined1 in
              
# 537 "asllib/Parser.mly"
           ( global_ignored () )
# 10618 "asllib/Parser.ml"
              
            in
            
# 620 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10624 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10630 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10639 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10645 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_187 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10660 "asllib/Parser.ml"
               x
            in
            let name = _1_inlined1 in
            
# 620 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10667 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10673 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10682 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10688 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_188 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10703 "asllib/Parser.ml"
               x
            in
            let name = _1_inlined1 in
            
# 620 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10710 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10716 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10725 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10731 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_189 =
    fun _1 _2_inlined1 _endpos__2_ _endpos__2_inlined1_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let (_endpos__2_, _2) = (_endpos__2_inlined1_, _2_inlined1) in
            let loc =
              let desc =
                let _3 = 
# 161 "asllib/Parser.mly"
                ( [] )
# 10745 "asllib/Parser.ml"
                 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10755 "asllib/Parser.ml"
              
            in
            
# 624 "asllib/Parser.mly"
        ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragma in declarations")
# 10761 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10767 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10776 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10782 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_190 =
    fun _1 _1_inlined1 _2_inlined1 _endpos__1_inlined1_ _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loc =
              let desc =
                let _3 = _1_inlined1 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined1_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10802 "asllib/Parser.ml"
              
            in
            
# 624 "asllib/Parser.mly"
        ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragma in declarations")
# 10808 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10814 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10823 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10829 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_191 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10837 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_192 =
    fun _1 _1_inlined2 ->
      let x =
        let _1_inlined1 = _1_inlined2 in
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10848 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10854 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_193 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10862 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_194 =
    fun _1_inlined1 ->
      let x =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10872 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10878 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_195 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10886 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_196 =
    fun _1_inlined1 ->
      let x =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10896 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10902 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_197 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10910 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_198 =
    fun x ->
      (
# 148 "<standard.mly>"
    ( x )
# 10918 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_199 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10926 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_200 =
    fun x ->
      (
# 148 "<standard.mly>"
    ( x )
# 10934 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_201 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10942 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_202 =
    fun () ->
      let x =
        let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 10951 "asllib/Parser.ml"
         in
        
# 205 "<standard.mly>"
    ( x )
# 10956 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10962 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_203 =
    fun _1_inlined1 ->
      let x =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10972 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10978 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_204 =
    fun _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined2 in
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 10991 "asllib/Parser.ml"
             _1_1
          in
          let where = 
# 123 "<standard.mly>"
    ( None )
# 10997 "asllib/Parser.ml"
           in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11006 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11015 "asllib/Parser.ml"
            
          in
          
# 441 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11021 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11030 "asllib/Parser.ml"
        
      in
      (
# 183 "asllib/Parser.mly"
         ( [ x ] )
# 11036 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_205 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined3 in
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11049 "asllib/Parser.ml"
             _1_1
          in
          let where =
            let _1 = _1_inlined2 in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 11058 "asllib/Parser.ml"
            
          in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11068 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11077 "asllib/Parser.ml"
            
          in
          
# 441 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11083 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined3_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11092 "asllib/Parser.ml"
        
      in
      (
# 183 "asllib/Parser.mly"
         ( [ x ] )
# 11098 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_206 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _endpos__1_inlined3_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc ->
      let y =
        let (_endpos__1_, _1) = (_endpos__1_inlined3_, _1_inlined3) in
        let desc =
          let stmt =
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11111 "asllib/Parser.ml"
             _1_1
          in
          let loc =
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11120 "asllib/Parser.ml"
            
          in
          
# 445 "asllib/Parser.mly"
  ( { pattern = add_pos_from loc Pattern_All; where = None; stmt } )
# 11126 "asllib/Parser.ml"
          
        in
        let _endpos_desc_ = _endpos__1_ in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11135 "asllib/Parser.ml"
        
      in
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined2 in
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11146 "asllib/Parser.ml"
             _1_1
          in
          let where = 
# 123 "<standard.mly>"
    ( None )
# 11152 "asllib/Parser.ml"
           in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11161 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11170 "asllib/Parser.ml"
            
          in
          
# 441 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11176 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11185 "asllib/Parser.ml"
        
      in
      (
# 184 "asllib/Parser.mly"
              ( [ x; y ] )
# 11191 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_207 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _1_inlined4 _2 _2_inlined1 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _endpos__1_inlined4_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc ->
      let y =
        let (_endpos__1_, _1, _2) = (_endpos__1_inlined4_, _1_inlined4, _2_inlined1) in
        let desc =
          let stmt =
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11204 "asllib/Parser.ml"
             _1_1
          in
          let loc =
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11213 "asllib/Parser.ml"
            
          in
          
# 445 "asllib/Parser.mly"
  ( { pattern = add_pos_from loc Pattern_All; where = None; stmt } )
# 11219 "asllib/Parser.ml"
          
        in
        let _endpos_desc_ = _endpos__1_ in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11228 "asllib/Parser.ml"
        
      in
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined3 in
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11239 "asllib/Parser.ml"
             _1_1
          in
          let where =
            let _1 = _1_inlined2 in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 11248 "asllib/Parser.ml"
            
          in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11258 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11267 "asllib/Parser.ml"
            
          in
          
# 441 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11273 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined3_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11282 "asllib/Parser.ml"
        
      in
      (
# 184 "asllib/Parser.mly"
              ( [ x; y ] )
# 11288 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_208 =
    fun _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ l ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined2 in
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11301 "asllib/Parser.ml"
             _1_1
          in
          let where = 
# 123 "<standard.mly>"
    ( None )
# 11307 "asllib/Parser.ml"
           in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11316 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11325 "asllib/Parser.ml"
            
          in
          
# 441 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11331 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11340 "asllib/Parser.ml"
        
      in
      (
# 185 "asllib/Parser.mly"
                                       ( x :: l )
# 11346 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_209 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ l ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined3 in
            let _1_1 = _1 in
            
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11359 "asllib/Parser.ml"
             _1_1
          in
          let where =
            let _1 = _1_inlined2 in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 11368 "asllib/Parser.ml"
            
          in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11378 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11387 "asllib/Parser.ml"
            
          in
          
# 441 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11393 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined3_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11402 "asllib/Parser.ml"
        
      in
      (
# 185 "asllib/Parser.mly"
                                       ( x :: l )
# 11408 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_210 =
    fun x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11416 "asllib/Parser.ml"
       in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11421 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_211 =
    fun x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11431 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11437 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_212 =
    fun _2 h t ->
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11445 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_213 =
    fun _1_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11453 "asllib/Parser.ml"
       in
      let x =
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11461 "asllib/Parser.ml"
          
        in
        
# 345 "asllib/Parser.mly"
                                              ( BitField_Simple (x, s)     )
# 11467 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11473 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_214 =
    fun _1_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11483 "asllib/Parser.ml"
        
      in
      let x =
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11492 "asllib/Parser.ml"
          
        in
        
# 345 "asllib/Parser.mly"
                                              ( BitField_Simple (x, s)     )
# 11498 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11504 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_215 =
    fun _1_inlined1 _1_inlined3 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11512 "asllib/Parser.ml"
       in
      let x =
        let bf =
          let _1_inlined1 = _1_inlined3 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11521 "asllib/Parser.ml"
          
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11529 "asllib/Parser.ml"
          
        in
        
# 346 "asllib/Parser.mly"
                                              ( BitField_Nested (x, s, bf) )
# 11535 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11541 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_216 =
    fun _1_inlined1 _1_inlined3 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11551 "asllib/Parser.ml"
        
      in
      let x =
        let bf =
          let _1_inlined1 = _1_inlined3 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11561 "asllib/Parser.ml"
          
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11569 "asllib/Parser.ml"
          
        in
        
# 346 "asllib/Parser.mly"
                                              ( BitField_Nested (x, s, bf) )
# 11575 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11581 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_217 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11589 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11602 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11608 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11614 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_218 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11624 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11638 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11644 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11650 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_219 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11658 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11671 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11677 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11683 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_220 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11693 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11707 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11713 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11719 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_221 =
    fun _1_inlined1 _2 t x ->
      let h =
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11730 "asllib/Parser.ml"
          
        in
        
# 345 "asllib/Parser.mly"
                                              ( BitField_Simple (x, s)     )
# 11736 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11742 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_222 =
    fun _1_inlined1 _1_inlined3 _2 t x ->
      let h =
        let bf =
          let _1_inlined1 = _1_inlined3 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11754 "asllib/Parser.ml"
          
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11762 "asllib/Parser.ml"
          
        in
        
# 346 "asllib/Parser.mly"
                                              ( BitField_Nested (x, s, bf) )
# 11768 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11774 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_223 =
    fun _1_inlined1 _1_inlined2 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11790 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11796 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11802 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_224 =
    fun _1_inlined1 _1_inlined2 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11818 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11824 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11830 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_225 =
    fun _1 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11838 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11846 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11852 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_226 =
    fun _1 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11862 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11871 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11877 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_227 =
    fun _1 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11885 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11893 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11899 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_228 =
    fun _1 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11909 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11918 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11924 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_229 =
    fun _1 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11935 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11941 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_230 =
    fun _1 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11952 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11958 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_231 =
    fun _1 _1_inlined1 _4 ty ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11970 "asllib/Parser.ml"
           _1_1
        in
        let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11976 "asllib/Parser.ml"
         in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 11984 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_232 =
    fun _1 _1_inlined1 _4 ty x ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 11996 "asllib/Parser.ml"
           _1_1
        in
        let _2 =
          let x = 
# 196 "<standard.mly>"
    ( x )
# 12003 "asllib/Parser.ml"
           in
          
# 126 "<standard.mly>"
    ( Some x )
# 12008 "asllib/Parser.ml"
          
        in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12017 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_233 =
    fun _1 _1_inlined1 _4 ty xs ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12029 "asllib/Parser.ml"
           _1_1
        in
        let _2 = 
# 123 "<standard.mly>"
    ( None )
# 12035 "asllib/Parser.ml"
         in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 12043 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_234 =
    fun _1 _1_inlined1 _4 ty x xs ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12055 "asllib/Parser.ml"
           _1_1
        in
        let _2 =
          let x = 
# 196 "<standard.mly>"
    ( x )
# 12062 "asllib/Parser.ml"
           in
          
# 126 "<standard.mly>"
    ( Some x )
# 12067 "asllib/Parser.ml"
          
        in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 12076 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_235 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 ->
      let x =
        let desc =
          let x =
            let s1 =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12090 "asllib/Parser.ml"
               _1_1
            in
            
# 458 "asllib/Parser.mly"
                                                       S_Cond 
# 12096 "asllib/Parser.ml"
             (e, s1, s2)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12102 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12111 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12117 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_236 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let alt = _1_inlined1 in
            
# 459 "asllib/Parser.mly"
                                                       S_Case 
# 12129 "asllib/Parser.ml"
             (expr, alt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12135 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12144 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12150 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_237 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12164 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 12170 "asllib/Parser.ml"
             in
            
# 460 "asllib/Parser.mly"
                                                       S_While 
# 12175 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12181 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12190 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12196 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_238 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let stmt_list =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12211 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 12220 "asllib/Parser.ml"
              
            in
            
# 460 "asllib/Parser.mly"
                                                       S_While 
# 12226 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12232 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12241 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12247 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_239 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12261 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 12267 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                       ( AST.Up )
# 12274 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12280 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12286 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12295 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12301 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_240 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12316 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 12325 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                       ( AST.Up )
# 12333 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12339 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12345 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12354 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12360 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_241 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12374 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 12380 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                                            ( AST.Down )
# 12387 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12393 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12399 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12408 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12414 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_242 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12429 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 12438 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                                            ( AST.Down )
# 12446 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12452 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12458 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12467 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12473 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_243 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c ->
      let x =
        let desc =
          let x =
            let o = 
# 123 "<standard.mly>"
    ( None )
# 12484 "asllib/Parser.ml"
             in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12492 "asllib/Parser.ml"
               _1_1
            in
            
# 464 "asllib/Parser.mly"
                                                                              S_Try  
# 12498 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12504 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12513 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12519 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_244 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let o =
              let (_1_inlined1, _1) = (_1_inlined3, _1_inlined2) in
              let x =
                let _1 = _1_inlined1 in
                let _1_1 = _1 in
                
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12536 "asllib/Parser.ml"
                 _1_1
              in
              
# 126 "<standard.mly>"
    ( Some x )
# 12542 "asllib/Parser.ml"
              
            in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12551 "asllib/Parser.ml"
               _1_1
            in
            
# 464 "asllib/Parser.mly"
                                                                              S_Try  
# 12557 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12563 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12572 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12578 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_245 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 12592 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 12598 "asllib/Parser.ml"
             in
            
# 466 "asllib/Parser.mly"
          ( S_While (cond, Some looplimit, body) )
# 12603 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12609 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12618 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12624 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_246 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x = 
# 435 "asllib/Parser.mly"
            ( S_Pass )
# 12634 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 12639 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12648 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12654 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_247 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let _2 = 
# 123 "<standard.mly>"
    ( None )
# 12665 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 470 "asllib/Parser.mly"
                                                               S_Return  
# 12671 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 12677 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12686 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12692 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_248 =
    fun _1 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let _2 = 
# 126 "<standard.mly>"
    ( Some x )
# 12703 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 470 "asllib/Parser.mly"
                                                               S_Return  
# 12709 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 12715 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12724 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12730 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_249 =
    fun _endpos__2_ _startpos_name_ name ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 12743 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 12748 "asllib/Parser.ml"
                
              in
              
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 12754 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 12760 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 12766 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12775 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12781 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_250 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ name ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 12795 "asllib/Parser.ml"
                
              in
              
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 12801 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 12807 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 12813 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12822 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12828 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_251 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ args name ->
      let x =
        let desc =
          let x =
            let call =
              let params =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 12842 "asllib/Parser.ml"
                
              in
              
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 12848 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 12854 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 12860 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12869 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12875 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_252 =
    fun _1 _endpos__2_ _startpos__1_ e ->
      let x =
        let desc =
          let x = 
# 472 "asllib/Parser.mly"
                                                               S_Assert  
# 12885 "asllib/Parser.ml"
           e in
          
# 196 "<standard.mly>"
    ( x )
# 12890 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12899 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12905 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_253 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 12916 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 12921 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 473 "asllib/Parser.mly"
                                                               S_Decl    
# 12927 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12933 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12942 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12948 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_254 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 12959 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 12964 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 473 "asllib/Parser.mly"
                                                               S_Decl    
# 12970 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12976 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12985 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12991 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_255 =
    fun _2_inlined1 _endpos__2_ _startpos_le_ e le ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 474 "asllib/Parser.mly"
                                                               S_Assign  
# 13003 "asllib/Parser.ml"
             (le, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 13009 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_le_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13018 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13024 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_256 =
    fun _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13039 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13044 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13050 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13059 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 13065 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13071 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13080 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13086 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_257 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13102 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13108 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13117 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 13123 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13129 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13138 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13144 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_258 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13160 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13166 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13175 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 13181 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13187 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13196 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13202 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_259 =
    fun _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13217 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13222 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13228 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13237 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 13243 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13249 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13258 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13264 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_260 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13280 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13286 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13295 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 13301 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13307 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13316 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13322 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_261 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13338 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13344 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13353 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 13359 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13365 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13374 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13380 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_262 =
    fun _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined2, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 13397 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 13403 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13412 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13417 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13423 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13432 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 13438 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13444 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13453 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13459 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_263 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 13476 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 13482 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13492 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13498 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13507 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 13513 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13519 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13528 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13534 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_264 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 13551 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 13557 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13567 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13573 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13582 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 13588 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13594 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13603 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13609 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_265 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13625 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13630 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13636 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13645 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 13651 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13656 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13662 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13671 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13677 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_266 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13694 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13700 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13709 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 13715 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13720 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13726 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13735 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13741 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_267 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13757 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13766 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 13772 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13777 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13783 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13792 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13798 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_268 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13814 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13819 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13825 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13834 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 13840 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13845 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13851 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13860 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13866 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_269 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13883 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13889 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13898 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 13904 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13909 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13915 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13924 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13930 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_270 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13946 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13955 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 13961 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13966 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13972 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13981 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13987 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_271 =
    fun _1 _endpos__2_ _startpos__1_ ldi ->
      let x =
        let desc =
          let x =
            let e = 
# 123 "<standard.mly>"
    ( None )
# 13998 "asllib/Parser.ml"
             in
            
# 483 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 14003 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14009 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14018 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14024 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_272 =
    fun _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let e =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 14039 "asllib/Parser.ml"
              
            in
            
# 483 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 14045 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14051 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14060 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14066 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_273 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 14085 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 484 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 14092 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14098 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14107 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14113 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_274 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 14132 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 484 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 14139 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14145 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14154 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14160 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_275 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14176 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 14181 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 14187 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14196 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 14202 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14208 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14217 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14223 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_276 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 14240 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 14246 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14255 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 14261 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14267 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14276 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14282 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_277 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 14298 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14307 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 14313 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14319 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14328 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14334 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_278 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14346 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 14351 "asllib/Parser.ml"
              
            in
            
# 487 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 14357 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14363 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14372 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14378 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_279 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 14392 "asllib/Parser.ml"
              
            in
            
# 487 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 14398 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14404 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14413 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14419 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_280 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14431 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 14436 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 14442 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14448 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14457 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14463 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_281 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 14477 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 14483 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14489 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14498 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14504 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_282 =
    fun _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Unreachable )
# 14516 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14522 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14531 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14537 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_283 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 14548 "asllib/Parser.ml"
             in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 14556 "asllib/Parser.ml"
               _1_1
            in
            
# 490 "asllib/Parser.mly"
                                                               S_Repeat  
# 14562 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14568 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14577 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14583 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_284 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loop_limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 14598 "asllib/Parser.ml"
              
            in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 14607 "asllib/Parser.ml"
               _1_1
            in
            
# 490 "asllib/Parser.mly"
                                                               S_Repeat  
# 14613 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14619 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14628 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14634 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_285 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 14648 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 14654 "asllib/Parser.ml"
             in
            
# 492 "asllib/Parser.mly"
          ( S_Repeat (body, cond, Some looplimit) )
# 14659 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14665 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14674 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14680 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_286 =
    fun _1 _endpos__2_ _startpos__1_ e ->
      let x =
        let desc =
          let x = 
# 493 "asllib/Parser.mly"
                                                             ( S_Throw (Some (e, None)) )
# 14690 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 14695 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14704 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14710 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_287 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x = 
# 494 "asllib/Parser.mly"
                                                             ( S_Throw None             )
# 14720 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 14725 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14734 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14740 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_288 =
    fun _1 _2_inlined1 _endpos__2_ _endpos__2_inlined1_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let (_endpos__2_, _2) = (_endpos__2_inlined1_, _2_inlined1) in
            let loc =
              let desc =
                let _3 = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14754 "asllib/Parser.ml"
                 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14764 "asllib/Parser.ml"
              
            in
            
# 496 "asllib/Parser.mly"
          ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragmas in statements" )
# 14770 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14776 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14785 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14791 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_289 =
    fun _1 _1_inlined1 _2_inlined1 _endpos__1_inlined1_ _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loc =
              let desc =
                let _3 = _1_inlined1 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined1_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14811 "asllib/Parser.ml"
              
            in
            
# 496 "asllib/Parser.mly"
          ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragmas in statements" )
# 14817 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14823 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14832 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14838 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_290 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 xs ->
      let x =
        let desc =
          let x =
            let s1 =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 14852 "asllib/Parser.ml"
               _1_1
            in
            
# 458 "asllib/Parser.mly"
                                                       S_Cond 
# 14858 "asllib/Parser.ml"
             (e, s1, s2)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14864 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14873 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14879 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_291 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let alt = _1_inlined1 in
            
# 459 "asllib/Parser.mly"
                                                       S_Case 
# 14891 "asllib/Parser.ml"
             (expr, alt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14897 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14906 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14912 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_292 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 14926 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 14932 "asllib/Parser.ml"
             in
            
# 460 "asllib/Parser.mly"
                                                       S_While 
# 14937 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14943 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14952 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14958 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_293 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let stmt_list =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 14973 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 14982 "asllib/Parser.ml"
              
            in
            
# 460 "asllib/Parser.mly"
                                                       S_While 
# 14988 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14994 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15003 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15009 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_294 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15023 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 15029 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                       ( AST.Up )
# 15036 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 15042 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15048 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15057 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15063 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_295 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15078 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 15087 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                       ( AST.Up )
# 15095 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 15101 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15107 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15116 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15122 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_296 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15136 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 15142 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                                            ( AST.Down )
# 15149 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 15155 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15161 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15170 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15176 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_297 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15191 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 15200 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                                            ( AST.Down )
# 15208 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 15214 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15220 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15229 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15235 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_298 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c xs ->
      let x =
        let desc =
          let x =
            let o = 
# 123 "<standard.mly>"
    ( None )
# 15246 "asllib/Parser.ml"
             in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15254 "asllib/Parser.ml"
               _1_1
            in
            
# 464 "asllib/Parser.mly"
                                                                              S_Try  
# 15260 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15266 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15275 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15281 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_299 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let o =
              let (_1_inlined1, _1) = (_1_inlined3, _1_inlined2) in
              let x =
                let _1 = _1_inlined1 in
                let _1_1 = _1 in
                
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15298 "asllib/Parser.ml"
                 _1_1
              in
              
# 126 "<standard.mly>"
    ( Some x )
# 15304 "asllib/Parser.ml"
              
            in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15313 "asllib/Parser.ml"
               _1_1
            in
            
# 464 "asllib/Parser.mly"
                                                                              S_Try  
# 15319 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15325 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15334 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15340 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_300 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 15354 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 15360 "asllib/Parser.ml"
             in
            
# 466 "asllib/Parser.mly"
          ( S_While (cond, Some looplimit, body) )
# 15365 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15371 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15380 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15386 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_301 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x = 
# 435 "asllib/Parser.mly"
            ( S_Pass )
# 15396 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 15401 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15410 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15416 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_302 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = 
# 123 "<standard.mly>"
    ( None )
# 15427 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 470 "asllib/Parser.mly"
                                                               S_Return  
# 15433 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 15439 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15448 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15454 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_303 =
    fun _1 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = 
# 126 "<standard.mly>"
    ( Some x )
# 15465 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 470 "asllib/Parser.mly"
                                                               S_Return  
# 15471 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 15477 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15486 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15492 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_304 =
    fun _endpos__2_ _startpos_name_ name xs ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 15505 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 15510 "asllib/Parser.ml"
                
              in
              
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15516 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 15522 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 15528 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15537 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15543 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_305 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ name xs ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 15557 "asllib/Parser.ml"
                
              in
              
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15563 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 15569 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 15575 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15584 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15590 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_306 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ args name xs ->
      let x =
        let desc =
          let x =
            let call =
              let params =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 15604 "asllib/Parser.ml"
                
              in
              
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 15610 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 15616 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 15622 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15631 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15637 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_307 =
    fun _1 _endpos__2_ _startpos__1_ e xs ->
      let x =
        let desc =
          let x = 
# 472 "asllib/Parser.mly"
                                                               S_Assert  
# 15647 "asllib/Parser.ml"
           e in
          
# 196 "<standard.mly>"
    ( x )
# 15652 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15661 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15667 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_308 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x xs ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 15678 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 15683 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 473 "asllib/Parser.mly"
                                                               S_Decl    
# 15689 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15695 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15704 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15710 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_309 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x xs ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 15721 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 15726 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 473 "asllib/Parser.mly"
                                                               S_Decl    
# 15732 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15738 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15747 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15753 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_310 =
    fun _2_inlined1 _endpos__2_ _startpos_le_ e le xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 474 "asllib/Parser.mly"
                                                               S_Assign  
# 15765 "asllib/Parser.ml"
             (le, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15771 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_le_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15780 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15786 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_311 =
    fun _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 15801 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 15806 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15812 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15821 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 15827 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15833 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15842 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15848 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_312 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 15864 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15870 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15879 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 15885 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15891 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15900 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15906 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_313 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 15922 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 15928 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15937 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 15943 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15949 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15958 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15964 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_314 =
    fun _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 15979 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 15984 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15990 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15999 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 16005 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16011 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16020 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16026 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_315 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16042 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16048 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16057 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 16063 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16069 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16078 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16084 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_316 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16100 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16106 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16115 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 16121 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16127 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16136 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16142 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_317 =
    fun _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined2, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16159 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 16165 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16174 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16179 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16185 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16194 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 16200 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16206 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16215 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16221 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_318 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16238 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 16244 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16254 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16260 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16269 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 16275 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16281 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16290 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16296 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_319 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16313 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 16319 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16329 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16335 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16344 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 16350 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16356 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16365 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16371 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_320 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16387 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16392 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16398 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16407 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 16413 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16418 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16424 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16433 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16439 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_321 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16456 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16462 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16471 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 16477 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16482 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16488 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16497 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16503 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_322 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16519 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16528 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 16534 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16539 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16545 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16554 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16560 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_323 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16576 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16581 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16587 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16596 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 16602 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16607 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16613 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16622 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16628 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_324 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16645 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16651 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16660 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 16666 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16671 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16677 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16686 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16692 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_325 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16708 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16717 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 16723 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16728 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16734 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16743 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16749 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_326 =
    fun _1 _endpos__2_ _startpos__1_ ldi xs ->
      let x =
        let desc =
          let x =
            let e = 
# 123 "<standard.mly>"
    ( None )
# 16760 "asllib/Parser.ml"
             in
            
# 483 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 16765 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16771 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16780 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16786 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_327 =
    fun _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let e =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 16801 "asllib/Parser.ml"
              
            in
            
# 483 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 16807 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16813 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16822 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16828 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_328 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16847 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 484 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 16854 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 16860 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16869 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16875 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_329 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16894 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 484 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 16901 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 16907 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16916 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16922 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_330 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16938 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16943 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16949 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16958 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 16964 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16970 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16979 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16985 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_331 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 17002 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 17008 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17017 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 17023 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17029 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17038 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17044 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_332 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 17060 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17069 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 17075 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17081 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17090 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17096 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_333 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 17108 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 17113 "asllib/Parser.ml"
              
            in
            
# 487 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 17119 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17125 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17134 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17140 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_334 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 17154 "asllib/Parser.ml"
              
            in
            
# 487 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 17160 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17166 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17175 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17181 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_335 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 17193 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 17198 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 17204 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17210 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17219 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17225 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_336 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 17239 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 17245 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17251 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17260 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17266 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_337 =
    fun _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Unreachable )
# 17278 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17284 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17293 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17299 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_338 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 17310 "asllib/Parser.ml"
             in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17318 "asllib/Parser.ml"
               _1_1
            in
            
# 490 "asllib/Parser.mly"
                                                               S_Repeat  
# 17324 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17330 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17339 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17345 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_339 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loop_limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 17360 "asllib/Parser.ml"
              
            in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17369 "asllib/Parser.ml"
               _1_1
            in
            
# 490 "asllib/Parser.mly"
                                                               S_Repeat  
# 17375 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17381 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17390 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17396 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_340 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17410 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 17416 "asllib/Parser.ml"
             in
            
# 492 "asllib/Parser.mly"
          ( S_Repeat (body, cond, Some looplimit) )
# 17421 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17427 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17436 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17442 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_341 =
    fun _1 _endpos__2_ _startpos__1_ e xs ->
      let x =
        let desc =
          let x = 
# 493 "asllib/Parser.mly"
                                                             ( S_Throw (Some (e, None)) )
# 17452 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 17457 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17466 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17472 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_342 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x = 
# 494 "asllib/Parser.mly"
                                                             ( S_Throw None             )
# 17482 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 17487 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17496 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17502 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_343 =
    fun _1 _2_inlined1 _endpos__2_ _endpos__2_inlined1_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let (_endpos__2_, _2) = (_endpos__2_inlined1_, _2_inlined1) in
            let loc =
              let desc =
                let _3 = 
# 161 "asllib/Parser.mly"
                ( [] )
# 17516 "asllib/Parser.ml"
                 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17526 "asllib/Parser.ml"
              
            in
            
# 496 "asllib/Parser.mly"
          ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragmas in statements" )
# 17532 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17538 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17547 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17553 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_344 =
    fun _1 _1_inlined1 _2_inlined1 _endpos__1_inlined1_ _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loc =
              let desc =
                let _3 = _1_inlined1 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined1_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17573 "asllib/Parser.ml"
              
            in
            
# 496 "asllib/Parser.mly"
          ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragmas in statements" )
# 17579 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17585 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17594 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17600 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_345 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ e s2 ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let s1 =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17615 "asllib/Parser.ml"
               _1_1
            in
            
# 458 "asllib/Parser.mly"
                                                       S_Cond 
# 17621 "asllib/Parser.ml"
             (e, s1, s2)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17627 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17636 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17657 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_346 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let alt = _1_inlined1 in
            
# 459 "asllib/Parser.mly"
                                                       S_Case 
# 17670 "asllib/Parser.ml"
             (expr, alt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17676 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17685 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17706 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_347 =
    fun _1 _1_inlined1 _2 _4 _endpos__2_inlined1_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17721 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 17727 "asllib/Parser.ml"
             in
            
# 460 "asllib/Parser.mly"
                                                       S_While 
# 17732 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17738 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17747 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17768 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_348 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _4 _endpos__2_inlined2_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let stmt_list =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17784 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 17793 "asllib/Parser.ml"
              
            in
            
# 460 "asllib/Parser.mly"
                                                       S_While 
# 17799 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17805 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17814 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17835 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_349 =
    fun _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17850 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 17856 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                       ( AST.Up )
# 17863 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 17869 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17875 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17884 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17905 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_350 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17921 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 17930 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                       ( AST.Up )
# 17938 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 17944 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17950 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17959 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17980 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_351 =
    fun _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 17995 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 18001 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                                            ( AST.Down )
# 18008 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 18014 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18020 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18029 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18050 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_352 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 18066 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 18075 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 437 "asllib/Parser.mly"
                                            ( AST.Down )
# 18083 "asllib/Parser.ml"
              
            in
            
# 463 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 18089 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18095 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18104 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18125 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_353 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ c ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let o = 
# 123 "<standard.mly>"
    ( None )
# 18137 "asllib/Parser.ml"
             in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 18145 "asllib/Parser.ml"
               _1_1
            in
            
# 464 "asllib/Parser.mly"
                                                                              S_Try  
# 18151 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18157 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18166 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18187 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_354 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ c ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let o =
              let (_1_inlined1, _1) = (_1_inlined3, _1_inlined2) in
              let x =
                let _1 = _1_inlined1 in
                let _1_1 = _1 in
                
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 18205 "asllib/Parser.ml"
                 _1_1
              in
              
# 126 "<standard.mly>"
    ( Some x )
# 18211 "asllib/Parser.ml"
              
            in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 18220 "asllib/Parser.ml"
               _1_1
            in
            
# 464 "asllib/Parser.mly"
                                                                              S_Try  
# 18226 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18232 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18241 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18262 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_355 =
    fun _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 18277 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 18283 "asllib/Parser.ml"
             in
            
# 466 "asllib/Parser.mly"
          ( S_While (cond, Some looplimit, body) )
# 18288 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18294 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18303 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18324 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_356 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 435 "asllib/Parser.mly"
            ( S_Pass )
# 18335 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 18340 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18349 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18370 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_357 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _2 = 
# 123 "<standard.mly>"
    ( None )
# 18382 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 470 "asllib/Parser.mly"
                                                               S_Return  
# 18388 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 18394 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18403 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18424 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_358 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _2 = 
# 126 "<standard.mly>"
    ( Some x )
# 18436 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 470 "asllib/Parser.mly"
                                                               S_Return  
# 18442 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 18448 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18457 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18478 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_359 =
    fun _2 _endpos__2_inlined1_ _startpos_name_ name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let call =
              let args =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 18492 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 18497 "asllib/Parser.ml"
                
              in
              
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18503 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 18509 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 18515 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18524 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18545 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_360 =
    fun _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let call =
              let args =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 18560 "asllib/Parser.ml"
                
              in
              
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18566 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 18572 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 18578 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18587 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18608 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_361 =
    fun _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ args name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let call =
              let params =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 18623 "asllib/Parser.ml"
                
              in
              
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 18629 "asllib/Parser.ml"
              
            in
            
# 471 "asllib/Parser.mly"
                                                               s_call  
# 18635 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 18641 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18650 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18671 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_362 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 472 "asllib/Parser.mly"
                                                               S_Assert  
# 18682 "asllib/Parser.ml"
           e in
          
# 196 "<standard.mly>"
    ( x )
# 18687 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18696 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18717 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_363 =
    fun _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 18729 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 18734 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 473 "asllib/Parser.mly"
                                                               S_Decl    
# 18740 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18746 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18755 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18776 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_364 =
    fun _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 18788 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 18793 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 473 "asllib/Parser.mly"
                                                               S_Decl    
# 18799 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18805 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18814 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18835 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_365 =
    fun _2 _2_inlined1 _endpos__2_inlined2_ _startpos_le_ e le ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 474 "asllib/Parser.mly"
                                                               S_Assign  
# 18848 "asllib/Parser.ml"
             (le, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18854 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_le_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18863 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18884 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_366 =
    fun _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 18900 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 18905 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18911 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18920 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 18926 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18932 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18941 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18962 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_367 =
    fun _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 18979 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18985 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18994 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 19000 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19006 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19015 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19036 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_368 =
    fun _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19053 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19059 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19068 "asllib/Parser.ml"
              
            in
            
# 476 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 19074 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19080 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19089 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19110 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_369 =
    fun _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19126 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19131 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19137 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19146 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 19152 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19158 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19167 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19188 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_370 =
    fun _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19205 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19211 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19220 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 19226 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19232 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19241 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19262 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_371 =
    fun _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args fld name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19279 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19285 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19294 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 19300 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19306 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19315 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19336 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_372 =
    fun _1_inlined2 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined2, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 19354 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 19360 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19369 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19374 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19380 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19389 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 19395 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19401 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19410 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19431 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_373 =
    fun _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 19449 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 19455 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19465 "asllib/Parser.ml"
                  
                in
                
# 524 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19471 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19480 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 19486 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19492 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19501 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19522 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_374 =
    fun _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos_args_ _startpos_name_ args name rhs x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 19540 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 19546 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19556 "asllib/Parser.ml"
                  
                in
                
# 526 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19562 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19571 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 19577 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19583 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19592 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19613 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_375 =
    fun _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19630 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19635 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19641 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19650 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 19656 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19661 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19667 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19676 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19697 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_376 =
    fun _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19715 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19721 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19730 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 19736 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19741 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19747 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19756 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19777 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_377 =
    fun _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19794 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19803 "asllib/Parser.ml"
              
            in
            let ldk = 
# 423 "asllib/Parser.mly"
                ( LDK_Let       )
# 19809 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19814 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19820 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19829 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19850 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_378 =
    fun _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19867 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19872 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19878 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19887 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 19893 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19898 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19904 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19913 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19934 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_379 =
    fun _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19952 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19958 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19967 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 19973 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19978 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19984 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19993 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20014 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_380 =
    fun _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 20031 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20040 "asllib/Parser.ml"
              
            in
            let ldk = 
# 424 "asllib/Parser.mly"
                ( LDK_Constant  )
# 20046 "asllib/Parser.ml"
             in
            
# 482 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 20051 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20057 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20066 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20087 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_381 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ldi ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let e = 
# 123 "<standard.mly>"
    ( None )
# 20099 "asllib/Parser.ml"
             in
            
# 483 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 20104 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20110 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20119 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20140 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_382 =
    fun _1 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _startpos__1_ ldi ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let e =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 20156 "asllib/Parser.ml"
              
            in
            
# 483 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 20162 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20168 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20177 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20198 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_383 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 20218 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 484 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 20225 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20231 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20240 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20261 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_384 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 20281 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 484 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 20288 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20294 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20303 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20324 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_385 =
    fun _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 20341 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 20346 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 20352 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20361 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 20367 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20373 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20382 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20403 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_386 =
    fun _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 20421 "asllib/Parser.ml"
                  
                in
                
# 529 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 20427 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20436 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 20442 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20448 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20457 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20478 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_387 =
    fun _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 531 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 20495 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20504 "asllib/Parser.ml"
              
            in
            
# 486 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 20510 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20516 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20525 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20546 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_388 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 20559 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 20564 "asllib/Parser.ml"
              
            in
            
# 487 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 20570 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20576 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20585 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20606 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_389 =
    fun _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 20621 "asllib/Parser.ml"
              
            in
            
# 487 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 20627 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20633 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20642 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20663 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_390 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 20676 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 20681 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 20687 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20693 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20702 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20723 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_391 =
    fun _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 20738 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 20744 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20750 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20759 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20780 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_392 =
    fun _1 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Unreachable )
# 20793 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20799 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20808 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20829 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_393 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 20841 "asllib/Parser.ml"
             in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 20849 "asllib/Parser.ml"
               _1_1
            in
            
# 490 "asllib/Parser.mly"
                                                               S_Repeat  
# 20855 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20861 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20870 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20891 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_394 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loop_limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 20907 "asllib/Parser.ml"
              
            in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 20916 "asllib/Parser.ml"
               _1_1
            in
            
# 490 "asllib/Parser.mly"
                                                               S_Repeat  
# 20922 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20928 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20937 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20958 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_395 =
    fun _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 20973 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 20979 "asllib/Parser.ml"
             in
            
# 492 "asllib/Parser.mly"
          ( S_Repeat (body, cond, Some looplimit) )
# 20984 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20990 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20999 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21020 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_396 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 493 "asllib/Parser.mly"
                                                             ( S_Throw (Some (e, None)) )
# 21031 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 21036 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21045 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21066 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_397 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 494 "asllib/Parser.mly"
                                                             ( S_Throw None             )
# 21077 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 21082 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21091 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21112 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_398 =
    fun _1 _2 _2_inlined1 _endpos__2_inlined1_ _endpos__2_inlined2_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let (_endpos__2_, _2) = (_endpos__2_inlined1_, _2_inlined1) in
            let loc =
              let desc =
                let _3 = 
# 161 "asllib/Parser.mly"
                ( [] )
# 21127 "asllib/Parser.ml"
                 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21137 "asllib/Parser.ml"
              
            in
            
# 496 "asllib/Parser.mly"
          ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragmas in statements" )
# 21143 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 21149 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21158 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21179 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_399 =
    fun _1 _1_inlined1 _2 _2_inlined1 _endpos__1_inlined1_ _endpos__2_inlined2_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loc =
              let desc =
                let _3 = _1_inlined1 in
                ()
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined1_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21200 "asllib/Parser.ml"
              
            in
            
# 496 "asllib/Parser.mly"
          ( Error.fatal_from loc @@ Error.NotYetImplemented "Pragmas in statements" )
# 21206 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 21212 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21221 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21242 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_400 =
    fun () ->
      (
# 111 "<standard.mly>"
    ( None )
# 21250 "asllib/Parser.ml"
       : (unit option))
  
  let _menhir_action_401 =
    fun x ->
      (
# 114 "<standard.mly>"
    ( Some x )
# 21258 "asllib/Parser.ml"
       : (unit option))
  
  let _menhir_action_402 =
    fun () ->
      (
# 111 "<standard.mly>"
    ( None )
# 21266 "asllib/Parser.ml"
       : ((string * AST.field list) option))
  
  let _menhir_action_403 =
    fun _1 _2 _3 ->
      let x =
        let _2_1 = _2 in
        let _3_1 = _3 in
        (_2_1, _3_1)
      in
      (
# 114 "<standard.mly>"
    ( Some x )
# 21279 "asllib/Parser.ml"
       : ((string * AST.field list) option))
  
  let _menhir_action_404 =
    fun _endpos_expr_pattern_ _startpos_expr_pattern_ expr_pattern ->
      let desc = 
# 313 "asllib/Parser.mly"
                        Pattern_Single  
# 21287 "asllib/Parser.ml"
       expr_pattern in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_expr_pattern_, _startpos_expr_pattern_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21295 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_405 =
    fun _2 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc = 
# 314 "asllib/Parser.mly"
                                           Pattern_Range  
# 21303 "asllib/Parser.ml"
       (e1, e2) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21311 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_406 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 315 "asllib/Parser.mly"
             ( Pattern_All )
# 21319 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21327 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_407 =
    fun _1 _endpos_expr_ _startpos__1_ expr ->
      let desc = 
# 316 "asllib/Parser.mly"
                     Pattern_Leq  
# 21335 "asllib/Parser.ml"
       expr in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_expr_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21343 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_408 =
    fun _1 _endpos_expr_ _startpos__1_ expr ->
      let desc = 
# 317 "asllib/Parser.mly"
                     Pattern_Geq  
# 21351 "asllib/Parser.ml"
       expr in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_expr_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21359 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_409 =
    fun _2 _endpos__3_ _startpos__1_ li x ->
      let desc =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 21369 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 21374 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 318 "asllib/Parser.mly"
                           Pattern_Tuple  
# 21381 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21390 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_410 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let _1_1 = _1 in
        
# 308 "asllib/Parser.mly"
                                            Pattern_Mask  
# 21400 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21409 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_411 =
    fun _1_inlined1 _endpos__1_inlined1_ _startpos__1_inlined1_ ->
      let x =
        let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
        let desc =
          let _1_1 = _1 in
          
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 21421 "asllib/Parser.ml"
           _1_1
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21430 "asllib/Parser.ml"
        
      in
      (
# 205 "<standard.mly>"
    ( x )
# 21436 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_412 =
    fun _1 _1_inlined2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ ->
      let desc =
        let _2 =
          let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) = (_endpos__1_inlined2_, _startpos__1_inlined2_, _1_inlined2) in
          let x =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 309 "asllib/Parser.mly"
                                                   Pattern_Any  
# 21451 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21460 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 21466 "asllib/Parser.ml"
          
        in
        let _2_1 = _2 in
        
# 326 "asllib/Parser.mly"
                                      Pattern_Not  
# 21473 "asllib/Parser.ml"
         _2_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21482 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_413 =
    fun _1 _1_inlined1 _3 _endpos_s2_ _startpos__1_ e s2 ->
      let desc =
        let s1 =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 21494 "asllib/Parser.ml"
           _1_1
        in
        
# 504 "asllib/Parser.mly"
                                                     S_Cond 
# 21500 "asllib/Parser.ml"
         (e, s1, s2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_s2_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = if _startpos_desc_ != _endpos_desc_ then
        _startpos_desc_
      else
        _endpos in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21512 "asllib/Parser.ml"
       : (AST.stmt))
  
  let _menhir_action_414 =
    fun _endpos__0_ ->
      let desc = 
# 435 "asllib/Parser.mly"
            ( S_Pass )
# 21520 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__0_, _endpos__0_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = if _startpos_desc_ != _endpos_desc_ then
        _startpos_desc_
      else
        _endpos in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21531 "asllib/Parser.ml"
       : (AST.stmt))
  
  let _menhir_action_415 =
    fun _1 _1_inlined1 ->
      let _1 = _1_inlined1 in
      let _1_1 = _1 in
      (
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 21541 "asllib/Parser.ml"
       _1_1 : (AST.stmt))
  
  let _menhir_action_416 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21549 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_417 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21557 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_418 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21565 "asllib/Parser.ml"
       : (AST.local_decl_item list))
  
  let _menhir_action_419 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21573 "asllib/Parser.ml"
       : (AST.local_decl_item list))
  
  let _menhir_action_420 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21581 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_421 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21589 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_422 =
    fun x y ->
      let x = 
# 180 "<standard.mly>"
    ( (x, y) )
# 21597 "asllib/Parser.ml"
       in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21602 "asllib/Parser.ml"
       : ((string * AST.expr) list))
  
  let _menhir_action_423 =
    fun x xs y ->
      let x = 
# 180 "<standard.mly>"
    ( (x, y) )
# 21610 "asllib/Parser.ml"
       in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21615 "asllib/Parser.ml"
       : ((string * AST.expr) list))
  
  let _menhir_action_424 =
    fun expr ->
      let x = 
# 303 "asllib/Parser.mly"
                                  Constraint_Exact  
# 21623 "asllib/Parser.ml"
       expr in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21628 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_425 =
    fun _2 e1 e2 ->
      let x = 
# 304 "asllib/Parser.mly"
                                  Constraint_Range  
# 21636 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21641 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_426 =
    fun expr xs ->
      let x = 
# 303 "asllib/Parser.mly"
                                  Constraint_Exact  
# 21649 "asllib/Parser.ml"
       expr in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21654 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_427 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 304 "asllib/Parser.mly"
                                  Constraint_Range  
# 21664 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21670 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_428 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21678 "asllib/Parser.ml"
       : (AST.lexpr list))
  
  let _menhir_action_429 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21686 "asllib/Parser.ml"
       : (AST.lexpr list))
  
  let _menhir_action_430 =
    fun x ->
      let x =
        let y = 
# 123 "<standard.mly>"
    ( None )
# 21695 "asllib/Parser.ml"
         in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21700 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21706 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_431 =
    fun _1 _2 x ->
      let x =
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21717 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21723 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21729 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_432 =
    fun _1 _2 x ->
      let x =
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21740 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21746 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21752 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_433 =
    fun x xs ->
      let x =
        let y = 
# 123 "<standard.mly>"
    ( None )
# 21761 "asllib/Parser.ml"
         in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21766 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21772 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_434 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21784 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21790 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21796 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_435 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21808 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21814 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21820 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_436 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21828 "asllib/Parser.ml"
       : (AST.pattern list))
  
  let _menhir_action_437 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21836 "asllib/Parser.ml"
       : (AST.pattern list))
  
  let _menhir_action_438 =
    fun expr ->
      let x = 
# 335 "asllib/Parser.mly"
                                    Slice_Single   
# 21844 "asllib/Parser.ml"
       expr in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21849 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_439 =
    fun _2 e1 e2 ->
      let x = 
# 336 "asllib/Parser.mly"
                                    Slice_Range    
# 21857 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21862 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_440 =
    fun _2 e1 e2 ->
      let x = 
# 337 "asllib/Parser.mly"
                                    Slice_Length   
# 21870 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21875 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_441 =
    fun _1 e ->
      let x = 
# 338 "asllib/Parser.mly"
                                  ( Slice_Length(zero, e) )
# 21883 "asllib/Parser.ml"
       in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21888 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_442 =
    fun _2 e1 e2 ->
      let x = 
# 339 "asllib/Parser.mly"
                                    Slice_Star     
# 21896 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21901 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_443 =
    fun expr xs ->
      let x = 
# 335 "asllib/Parser.mly"
                                    Slice_Single   
# 21909 "asllib/Parser.ml"
       expr in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21914 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_444 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 336 "asllib/Parser.mly"
                                    Slice_Range    
# 21924 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21930 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_445 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 337 "asllib/Parser.mly"
                                    Slice_Length   
# 21940 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21946 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_446 =
    fun _1 e xs ->
      let x = 
# 338 "asllib/Parser.mly"
                                  ( Slice_Length(zero, e) )
# 21954 "asllib/Parser.ml"
       in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21959 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_447 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 339 "asllib/Parser.mly"
                                    Slice_Star     
# 21969 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21975 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_448 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21983 "asllib/Parser.ml"
       : (AST.ty list))
  
  let _menhir_action_449 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21991 "asllib/Parser.ml"
       : (AST.ty list))
  
  let _menhir_action_450 =
    fun _1 _2 x ->
      let x =
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 22001 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 22007 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_451 =
    fun _1 _2 x ->
      let x =
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 22017 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 22023 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_452 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 22034 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 22040 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_453 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 22051 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 22057 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_454 =
    fun _1 ->
      let x =
        let _1_1 = _1 in
        
# 500 "asllib/Parser.mly"
                                            stmt_from_list 
# 22067 "asllib/Parser.ml"
         _1_1
      in
      (
# 196 "<standard.mly>"
    ( x )
# 22073 "asllib/Parser.ml"
       : (AST.stmt))
  
  let _menhir_action_455 =
    fun _1 _endpos__3_ _startpos__1_ ->
      let desc =
        let c =
          let _1 =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 22084 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 22089 "asllib/Parser.ml"
            
          in
          let _1_1 = _1 in
          
# 300 "asllib/Parser.mly"
                                                          WellConstrained  
# 22096 "asllib/Parser.ml"
           _1_1
        in
        
# 352 "asllib/Parser.mly"
                                                          T_Int        
# 22102 "asllib/Parser.ml"
         c
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22111 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_456 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let c =
          let _1_inlined1 = _1_inlined2 in
          let _1 =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 22124 "asllib/Parser.ml"
            
          in
          let _1_1 = _1 in
          
# 300 "asllib/Parser.mly"
                                                          WellConstrained  
# 22131 "asllib/Parser.ml"
           _1_1
        in
        
# 352 "asllib/Parser.mly"
                                                          T_Int        
# 22137 "asllib/Parser.ml"
         c
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22146 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_457 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let c = 
# 299 "asllib/Parser.mly"
                                             ( UnConstrained )
# 22155 "asllib/Parser.ml"
         in
        
# 352 "asllib/Parser.mly"
                                                          T_Int        
# 22160 "asllib/Parser.ml"
         c
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22169 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_458 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 353 "asllib/Parser.mly"
                                                        ( T_Real      )
# 22177 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22185 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_459 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 354 "asllib/Parser.mly"
                                                        ( T_Bool      )
# 22193 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22201 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_460 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 355 "asllib/Parser.mly"
                                                        ( T_String    )
# 22209 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22217 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_461 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 356 "asllib/Parser.mly"
                                                        ( t_bit       )
# 22225 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22233 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_462 =
    fun _1 _1_inlined2 _endpos__1_inlined2_ _startpos__1_ x ->
      let desc =
        let bitfields_opt = _1_inlined2 in
        let _2 = 
# 205 "<standard.mly>"
    ( x )
# 22243 "asllib/Parser.ml"
         in
        let _2_1 = _2 in
        
# 357 "asllib/Parser.mly"
                                                          T_Bits       
# 22249 "asllib/Parser.ml"
         (_2_1, bitfields_opt)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22258 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_463 =
    fun _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 22268 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 22273 "asllib/Parser.ml"
          
        in
        
# 358 "asllib/Parser.mly"
                                                          T_Tuple      
# 22279 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22288 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_464 =
    fun _1_inlined1 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22299 "asllib/Parser.ml"
          
        in
        
# 358 "asllib/Parser.mly"
                                                          T_Tuple      
# 22305 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22314 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_465 =
    fun _endpos_name_ _startpos_name_ name ->
      let desc = 
# 359 "asllib/Parser.mly"
                                                          T_Named      
# 22322 "asllib/Parser.ml"
       name in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_name_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22330 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_466 =
    fun _1 _3 _endpos_t_ _startpos__1_ t x ->
      let desc =
        let e = 
# 205 "<standard.mly>"
    ( x )
# 22339 "asllib/Parser.ml"
         in
        
# 360 "asllib/Parser.mly"
                                                        ( T_Array (ArrayLength_Expr e, t) )
# 22344 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_t_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22353 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_467 =
    fun _1 ->
      (_1 : (AST.ty))
  
  let _menhir_action_468 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let _1_inlined1 = _1_inlined2 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22369 "asllib/Parser.ml"
          
        in
        
# 365 "asllib/Parser.mly"
                                                          T_Enum       
# 22375 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22384 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_469 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let l = 
# 330 "asllib/Parser.mly"
                  ( [] )
# 22393 "asllib/Parser.ml"
         in
        
# 366 "asllib/Parser.mly"
                                                          T_Record     
# 22398 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22407 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_470 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let _1_inlined1 = _1_inlined2 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22419 "asllib/Parser.ml"
          
        in
        
# 366 "asllib/Parser.mly"
                                                          T_Record     
# 22425 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22434 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_471 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let l = 
# 330 "asllib/Parser.mly"
                  ( [] )
# 22443 "asllib/Parser.ml"
         in
        
# 367 "asllib/Parser.mly"
                                                          T_Exception  
# 22448 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22457 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_472 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let _1_inlined1 = _1_inlined2 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22469 "asllib/Parser.ml"
          
        in
        
# 367 "asllib/Parser.mly"
                                                          T_Exception  
# 22475 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22484 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_print_token : token -> string =
    fun _tok ->
      match _tok with
      | Tokens.AND ->
          "AND"
      | Tokens.ARRAY ->
          "ARRAY"
      | Tokens.ARROBASE_LOOPLIMIT ->
          "ARROBASE_LOOPLIMIT"
      | Tokens.ARROW ->
          "ARROW"
      | Tokens.AS ->
          "AS"
      | Tokens.ASSERT ->
          "ASSERT"
      | Tokens.BAND ->
          "BAND"
      | Tokens.BEGIN ->
          "BEGIN"
      | Tokens.BEQ ->
          "BEQ"
      | Tokens.BIT ->
          "BIT"
      | Tokens.BITS ->
          "BITS"
      | Tokens.BITVECTOR_LIT _ ->
          "BITVECTOR_LIT"
      | Tokens.BNOT ->
          "BNOT"
      | Tokens.BOOLEAN ->
          "BOOLEAN"
      | Tokens.BOOL_LIT _ ->
          "BOOL_LIT"
      | Tokens.BOR ->
          "BOR"
      | Tokens.CASE ->
          "CASE"
      | Tokens.CATCH ->
          "CATCH"
      | Tokens.COLON ->
          "COLON"
      | Tokens.COLON_COLON ->
          "COLON_COLON"
      | Tokens.COMMA ->
          "COMMA"
      | Tokens.CONCAT ->
          "CONCAT"
      | Tokens.CONFIG ->
          "CONFIG"
      | Tokens.CONSTANT ->
          "CONSTANT"
      | Tokens.DEBUG ->
          "DEBUG"
      | Tokens.DIV ->
          "DIV"
      | Tokens.DIVRM ->
          "DIVRM"
      | Tokens.DO ->
          "DO"
      | Tokens.DOT ->
          "DOT"
      | Tokens.DOWNTO ->
          "DOWNTO"
      | Tokens.ELSE ->
          "ELSE"
      | Tokens.ELSIF ->
          "ELSIF"
      | Tokens.END ->
          "END"
      | Tokens.ENUMERATION ->
          "ENUMERATION"
      | Tokens.EOF ->
          "EOF"
      | Tokens.EOR ->
          "EOR"
      | Tokens.EQ ->
          "EQ"
      | Tokens.EQ_OP ->
          "EQ_OP"
      | Tokens.EXCEPTION ->
          "EXCEPTION"
      | Tokens.FOR ->
          "FOR"
      | Tokens.FUNC ->
          "FUNC"
      | Tokens.GEQ ->
          "GEQ"
      | Tokens.GETTER ->
          "GETTER"
      | Tokens.GT ->
          "GT"
      | Tokens.IDENTIFIER _ ->
          "IDENTIFIER"
      | Tokens.IF ->
          "IF"
      | Tokens.IMPL ->
          "IMPL"
      | Tokens.IN ->
          "IN"
      | Tokens.INTEGER ->
          "INTEGER"
      | Tokens.INT_LIT _ ->
          "INT_LIT"
      | Tokens.LBRACE ->
          "LBRACE"
      | Tokens.LBRACKET ->
          "LBRACKET"
      | Tokens.LEQ ->
          "LEQ"
      | Tokens.LET ->
          "LET"
      | Tokens.LLBRACKET ->
          "LLBRACKET"
      | Tokens.LOOPLIMIT ->
          "LOOPLIMIT"
      | Tokens.LPAR ->
          "LPAR"
      | Tokens.LT ->
          "LT"
      | Tokens.MASK_LIT _ ->
          "MASK_LIT"
      | Tokens.MINUS ->
          "MINUS"
      | Tokens.MOD ->
          "MOD"
      | Tokens.MUL ->
          "MUL"
      | Tokens.NEQ ->
          "NEQ"
      | Tokens.NOT ->
          "NOT"
      | Tokens.OF ->
          "OF"
      | Tokens.OR ->
          "OR"
      | Tokens.OTHERWISE ->
          "OTHERWISE"
      | Tokens.PASS ->
          "PASS"
      | Tokens.PLUS ->
          "PLUS"
      | Tokens.PLUS_COLON ->
          "PLUS_COLON"
      | Tokens.POW ->
          "POW"
      | Tokens.PRAGMA ->
          "PRAGMA"
      | Tokens.PRINT ->
          "PRINT"
      | Tokens.RBRACE ->
          "RBRACE"
      | Tokens.RBRACKET ->
          "RBRACKET"
      | Tokens.RDIV ->
          "RDIV"
      | Tokens.REAL ->
          "REAL"
      | Tokens.REAL_LIT _ ->
          "REAL_LIT"
      | Tokens.RECORD ->
          "RECORD"
      | Tokens.RECURSELIMIT ->
          "RECURSELIMIT"
      | Tokens.REPEAT ->
          "REPEAT"
      | Tokens.RETURN ->
          "RETURN"
      | Tokens.RPAR ->
          "RPAR"
      | Tokens.RRBRACKET ->
          "RRBRACKET"
      | Tokens.SEMI_COLON ->
          "SEMI_COLON"
      | Tokens.SETTER ->
          "SETTER"
      | Tokens.SHL ->
          "SHL"
      | Tokens.SHR ->
          "SHR"
      | Tokens.SLICING ->
          "SLICING"
      | Tokens.STAR_COLON ->
          "STAR_COLON"
      | Tokens.STRING ->
          "STRING"
      | Tokens.STRING_LIT _ ->
          "STRING_LIT"
      | Tokens.SUBTYPES ->
          "SUBTYPES"
      | Tokens.THEN ->
          "THEN"
      | Tokens.THROW ->
          "THROW"
      | Tokens.TO ->
          "TO"
      | Tokens.TRY ->
          "TRY"
      | Tokens.TYPE ->
          "TYPE"
      | Tokens.UNKNOWN ->
          "UNKNOWN"
      | Tokens.UNREACHABLE ->
          "UNREACHABLE"
      | Tokens.UNTIL ->
          "UNTIL"
      | Tokens.VAR ->
          "VAR"
      | Tokens.WHEN ->
          "WHEN"
      | Tokens.WHERE ->
          "WHERE"
      | Tokens.WHILE ->
          "WHILE"
      | Tokens.WITH ->
          "WITH"
  
  let _menhir_fail : unit -> 'a =
    fun () ->
      Printf.eprintf "Internal failure -- please contact the parser generator's developers.\n%!";
      assert false
  
  include struct
    
    [@@@ocaml.warning "-4-37"]
    
    let _menhir_goto_opn : type  ttv_stack. ttv_stack -> _ -> _menhir_box_opn =
      fun _menhir_stack _v ->
        MenhirBox_opn _v
    
    let _menhir_run_1096 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined2, _2_inlined2, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_383 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1092 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined2, _2_inlined2, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_384 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1376 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_stmts =
      fun _menhir_stack _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let _1 = _v in
            let _v = _menhir_action_454 _1 in
            MenhirBox_stmts _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1360 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
            let MenhirCell0_WHILE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3, _5) = ((), (), _endpos, (), ()) in
            let _v = _menhir_action_355 _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1346 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF, _menhir_box_opn) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_OF (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_CASE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3) = ((), (), _endpos, ()) in
            let _v = _menhir_action_346 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1308 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _1_inlined1, _3, _8) = ((), (), _endpos, (), (), ()) in
            let _v = _menhir_action_351 _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1304 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _1_inlined2, _1_inlined1, _3, _endpos__2_inlined2_, _8) = ((), (), (), (), (), _endpos, ()) in
            let _v = _menhir_action_352 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1296 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _1_inlined1, _3, _8) = ((), (), _endpos, (), (), ()) in
            let _v = _menhir_action_349 _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1292 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _1_inlined2, _1_inlined1, _3, _endpos__2_inlined2_, _8) = ((), (), (), (), (), _endpos, ()) in
            let _v = _menhir_action_350 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1208 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_s_else -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_s_else (_menhir_stack, _, s2, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
            let MenhirCell1_IF (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3) = ((), (), _endpos, ()) in
            let _v = _menhir_action_345 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ e s2 in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1137 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3) = ((), (), _endpos, ()) in
            let _v = _menhir_action_353 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ c in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1135 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_OTHERWISE (_menhir_stack, _, _, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _2_inlined1, _1_inlined2, _3, _endpos__2_inlined2_) = ((), (), (), (), (), _endpos) in
            let _v = _menhir_action_354 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ c in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1085 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _4, _endpos__2_inlined1_) = ((), (), (), _endpos) in
            let _v = _menhir_action_347 _1 _1_inlined1 _2 _4 _endpos__2_inlined1_ _startpos__1_ expr in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1081 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _4, _1_inlined1, _endpos__2_inlined2_) = ((), (), (), (), _endpos) in
            let _v = _menhir_action_348 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _4 _endpos__2_inlined2_ _startpos__1_ expr in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1071 : type  ttv_stack. ttv_stack -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let x = _v in
        let _v = _menhir_action_003 x in
        MenhirBox_ast _v
    
    let rec _menhir_run_0973 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _endpos__2_inlined1_) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_189 _1 _2_inlined1 _endpos__2_ _endpos__2_inlined1_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_goto_list_decl_ : type  ttv_stack. ttv_stack -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _v _menhir_s ->
        match _menhir_s with
        | MenhirState0000 ->
            _menhir_run_1071 _menhir_stack _v
        | MenhirState1069 ->
            _menhir_run_1070 _menhir_stack _v
        | MenhirState1065 ->
            _menhir_run_1066 _menhir_stack _v
        | MenhirState1061 ->
            _menhir_run_1062 _menhir_stack _v
        | MenhirState1057 ->
            _menhir_run_1058 _menhir_stack _v
        | MenhirState1053 ->
            _menhir_run_1054 _menhir_stack _v
        | MenhirState1048 ->
            _menhir_run_1049 _menhir_stack _v
        | MenhirState1044 ->
            _menhir_run_1045 _menhir_stack _v
        | MenhirState1040 ->
            _menhir_run_1041 _menhir_stack _v
        | MenhirState1036 ->
            _menhir_run_1037 _menhir_stack _v
        | MenhirState0355 ->
            _menhir_run_1033 _menhir_stack _v
        | MenhirState1030 ->
            _menhir_run_1031 _menhir_stack _v
        | MenhirState0375 ->
            _menhir_run_1008 _menhir_stack _v
        | MenhirState1006 ->
            _menhir_run_1007 _menhir_stack _v
        | MenhirState1004 ->
            _menhir_run_1005 _menhir_stack _v
        | MenhirState0998 ->
            _menhir_run_0999 _menhir_stack _v
        | MenhirState0996 ->
            _menhir_run_0997 _menhir_stack _v
        | MenhirState0986 ->
            _menhir_run_0987 _menhir_stack _v
        | MenhirState0984 ->
            _menhir_run_0985 _menhir_stack _v
        | MenhirState0978 ->
            _menhir_run_0979 _menhir_stack _v
        | MenhirState0781 ->
            _menhir_run_0977 _menhir_stack _v
        | MenhirState0975 ->
            _menhir_run_0976 _menhir_stack _v
        | MenhirState0784 ->
            _menhir_run_0973 _menhir_stack _v
        | MenhirState0971 ->
            _menhir_run_0972 _menhir_stack _v
        | MenhirState0965 ->
            _menhir_run_0966 _menhir_stack _v
        | MenhirState0959 ->
            _menhir_run_0960 _menhir_stack _v
        | MenhirState0954 ->
            _menhir_run_0955 _menhir_stack _v
        | MenhirState0948 ->
            _menhir_run_0949 _menhir_stack _v
        | MenhirState0789 ->
            _menhir_run_0943 _menhir_stack _v
        | MenhirState0941 ->
            _menhir_run_0942 _menhir_stack _v
        | MenhirState0939 ->
            _menhir_run_0940 _menhir_stack _v
        | MenhirState0931 ->
            _menhir_run_0932 _menhir_stack _v
        | MenhirState0799 ->
            _menhir_run_0930 _menhir_stack _v
        | MenhirState0928 ->
            _menhir_run_0929 _menhir_stack _v
        | MenhirState0926 ->
            _menhir_run_0927 _menhir_stack _v
        | MenhirState0922 ->
            _menhir_run_0923 _menhir_stack _v
        | MenhirState0920 ->
            _menhir_run_0921 _menhir_stack _v
        | MenhirState0912 ->
            _menhir_run_0913 _menhir_stack _v
        | MenhirState0910 ->
            _menhir_run_0911 _menhir_stack _v
        | MenhirState0895 ->
            _menhir_run_0896 _menhir_stack _v
        | MenhirState0893 ->
            _menhir_run_0894 _menhir_stack _v
        | MenhirState0889 ->
            _menhir_run_0890 _menhir_stack _v
        | MenhirState0887 ->
            _menhir_run_0888 _menhir_stack _v
        | MenhirState0879 ->
            _menhir_run_0880 _menhir_stack _v
        | MenhirState0807 ->
            _menhir_run_0878 _menhir_stack _v
        | MenhirState0876 ->
            _menhir_run_0877 _menhir_stack _v
        | MenhirState0870 ->
            _menhir_run_0871 _menhir_stack _v
        | MenhirState0864 ->
            _menhir_run_0865 _menhir_stack _v
        | MenhirState0859 ->
            _menhir_run_0860 _menhir_stack _v
        | MenhirState0853 ->
            _menhir_run_0854 _menhir_stack _v
        | MenhirState0812 ->
            _menhir_run_0848 _menhir_stack _v
        | MenhirState0846 ->
            _menhir_run_0847 _menhir_stack _v
        | MenhirState0840 ->
            _menhir_run_0841 _menhir_stack _v
        | MenhirState0834 ->
            _menhir_run_0835 _menhir_stack _v
        | MenhirState0829 ->
            _menhir_run_0830 _menhir_stack _v
        | MenhirState0823 ->
            _menhir_run_0824 _menhir_stack _v
        | MenhirState0817 ->
            _menhir_run_0818 _menhir_stack _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1070 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_177 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1066 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2) = ((), _v, ()) in
        let _v = _menhir_action_187 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1062 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_178 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1058 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2) = ((), _v, ()) in
        let _v = _menhir_action_188 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1054 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_176 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1049 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_174 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1045 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_185 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1041 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_175 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1037 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_186 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1033 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_173 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1031 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty_decl _menhir_cell0_option_subtype_ _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_option_subtype_ (_menhir_stack, _1_inlined1) = _menhir_stack in
        let MenhirCell1_ty_decl (_menhir_stack, _, t) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_TYPE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _3) = ((), _v, ()) in
        let _v = _menhir_action_159 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ t x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1008 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption___anonymous_22_ _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption___anonymous_22_ (_menhir_stack, _, _3, _endpos__3_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell0_SUBTYPES (_menhir_stack, _startpos__1_inlined1_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_TYPE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_160 _1 _1_inlined1 _2_inlined1 _3 _endpos__2_ _endpos__3_ _startpos__1_ _startpos__1_inlined1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1007 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_156 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1005 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_155 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0999 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_158 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0997 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_157 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0987 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_152 _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0985 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_151 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0979 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_154 _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0977 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_153 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0976 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_190 _1 _1_inlined1 _2_inlined1 _endpos__1_inlined1_ _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0972 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_165 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0966 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_166 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0960 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_164 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0955 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_162 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0949 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_163 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0943 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_161 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0942 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_150 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0940 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_149 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0932 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_148 _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0930 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_147 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0929 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_140 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0927 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_139 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0923 : type  ttv_stack. (((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _endpos__1_inlined6_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined5, _1_inlined4) = ((), _v, (), ()) in
        let _v = _menhir_action_142 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _2 _2_inlined1 _endpos__1_inlined6_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0921 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined7, _) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined5, _1_inlined4) = ((), _v, (), ()) in
        let _v = _menhir_action_141 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _1_inlined7 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0913 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_146 _1 _1_inlined1 _1_inlined3 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0911 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_145 _1 _1_inlined1 _1_inlined3 _1_inlined5 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0896 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_136 _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0894 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_135 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0890 : type  ttv_stack. (((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_138 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _2_inlined1 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0888 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_137 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0880 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined3_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_144 _1 _1_inlined1 _endpos__1_inlined3_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0878 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_, _menhir_box_ast) _menhir_cell1_end_semicolon -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined4, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_143 _1 _1_inlined1 _1_inlined4 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0877 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_171 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0871 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_172 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0865 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_170 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0860 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_168 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0854 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_169 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0848 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_167 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0847 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_183 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0841 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_184 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0835 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_182 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0830 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_180 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0824 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_181 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_0818 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_ast =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_179 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    let _menhir_run_1336 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _3, _3_inlined1, _5, _endpos__2_inlined2_, _endpos_args_, args) = ((), (), (), (), (), (), _endpos_0, _endpos, _v) in
                let _v = _menhir_action_380 _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1197 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _3, _3_inlined1, _5, _endpos__2_inlined2_, _endpos_args_, args) = ((), (), (), (), (), (), _endpos_0, _endpos, _v) in
                let _v = _menhir_action_377 _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1117 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _3, _3_inlined1, _5, _endpos__2_inlined2_, _endpos_args_, args) = ((), (), (), (), (), (), _endpos_0, _endpos, _v) in
                let _v = _menhir_action_387 _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1329 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__2_inlined2_, _endpos__3_inlined2_) = ((), (), (), _v, (), (), _endpos_1, _endpos_0) in
                    let _v = _menhir_action_379 _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1315 : type  ttv_stack. (ttv_stack, _menhir_box_opn) _menhir_cell1_DEBUG _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_DEBUG (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _endpos__2_inlined1_, _1_inlined2) = ((), (), _endpos_1, _v) in
                    let _v = _menhir_action_391 _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1190 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__2_inlined2_, _endpos__3_inlined2_) = ((), (), (), _v, (), (), _endpos_1, _endpos_0) in
                    let _v = _menhir_action_376 _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1174 : type  ttv_stack. (ttv_stack, _menhir_box_opn) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER -> _ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _) = _menhir_stack in
                let MenhirCell1_PRAGMA (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__1_inlined1_, _1_inlined1, _endpos__2_inlined2_) = ((), (), _endpos, _v, _endpos_0) in
                let _v = _menhir_action_399 _1 _1_inlined1 _2 _2_inlined1 _endpos__1_inlined1_ _endpos__2_inlined2_ _startpos__1_ in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1166 : type  ttv_stack. (ttv_stack, _menhir_box_opn) _menhir_cell1_PRINT _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_PRINT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _endpos__2_inlined1_, _1_inlined2) = ((), (), _endpos_1, _v) in
                    let _v = _menhir_action_389 _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1110 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__2_inlined2_, _endpos__3_inlined2_) = ((), (), (), _v, (), (), _endpos_1, _endpos_0) in
                    let _v = _menhir_action_386 _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let rec _menhir_run_0001 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0003 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1034 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1042 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1051 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1055 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1063 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0004 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_UNKNOWN (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0005 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0352 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0006 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_460 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_goto_ty : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1095 ->
            _menhir_run_1096 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1091 ->
            _menhir_run_1092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1063 ->
            _menhir_run_1064 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1055 ->
            _menhir_run_1056 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1042 ->
            _menhir_run_1043 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1034 ->
            _menhir_run_1035 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1009 ->
            _menhir_run_1032 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1000 ->
            _menhir_run_1001 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0992 ->
            _menhir_run_0993 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0980 ->
            _menhir_run_0981 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0967 ->
            _menhir_run_0968 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0961 ->
            _menhir_run_0962 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0950 ->
            _menhir_run_0951 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0944 ->
            _menhir_run_0945 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0935 ->
            _menhir_run_0936 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0914 ->
            _menhir_run_0915 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0902 ->
            _menhir_run_0903 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0898 ->
            _menhir_run_0899 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0881 ->
            _menhir_run_0882 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0872 ->
            _menhir_run_0873 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0866 ->
            _menhir_run_0867 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0855 ->
            _menhir_run_0856 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0849 ->
            _menhir_run_0850 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0842 ->
            _menhir_run_0843 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0836 ->
            _menhir_run_0837 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0825 ->
            _menhir_run_0826 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0819 ->
            _menhir_run_0820 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0795 ->
            _menhir_run_0796 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0745 ->
            _menhir_run_0746 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0724 ->
            _menhir_run_0731 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0726 ->
            _menhir_run_0727 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0431 ->
            _menhir_run_0432 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0423 ->
            _menhir_run_0424 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0421 ->
            _menhir_run_0422 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0415 ->
            _menhir_run_0416 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0413 ->
            _menhir_run_0414 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0409 ->
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0407 ->
            _menhir_run_0408 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0397 ->
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0386 ->
            _menhir_run_0387 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0382 ->
            _menhir_run_0383 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0367 ->
            _menhir_run_0368 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0363 ->
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0352 ->
            _menhir_run_0353 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0005 ->
            _menhir_run_0351 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0347 ->
            _menhir_run_0346 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0008 ->
            _menhir_run_0346 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0303 ->
            _menhir_run_0308 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0162 ->
            _menhir_run_0163 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0041 ->
            _menhir_run_0161 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0139 ->
            _menhir_run_0140 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0061 ->
            _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0123 ->
            _menhir_run_0135 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0079 ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1064 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1065
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1066 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1067 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0356 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_TYPE (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SUBTYPES ->
                let _startpos_1 = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_SUBTYPES (_menhir_stack, _startpos_1) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v_2 ->
                    let _startpos_3 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos_4 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_2, _startpos_3, _endpos_4) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WITH ->
                        _menhir_run_0360 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0359
                    | Tokens.SEMI_COLON ->
                        let _v_5 = _menhir_action_191 () in
                        _menhir_run_0374 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_4 _v_5 MenhirState0359 _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.OF ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | Tokens.RECORD ->
                    let _startpos_6 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACE ->
                        let _menhir_stack = MenhirCell1_RECORD (_menhir_stack, MenhirState1009, _startpos_6, _endpos_7) in
                        let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v_9 ->
                            _menhir_run_0362 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState1011
                        | Tokens.RBRACE ->
                            let _v_10 = _menhir_action_199 () in
                            _menhir_run_1012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10
                        | _ ->
                            _eRR ())
                    | Tokens.SEMI_COLON | Tokens.SUBTYPES ->
                        let (_menhir_s, _endpos__1_, _startpos__1_, _1) = (MenhirState1009, _endpos_7, _startpos_6, ()) in
                        let _v = _menhir_action_469 _1 _endpos__1_ _startpos__1_ in
                        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | Tokens.IDENTIFIER _v_12 ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState1009
                | Tokens.EXCEPTION ->
                    let _startpos_13 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos_14 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACE ->
                        let _menhir_stack = MenhirCell1_EXCEPTION (_menhir_stack, MenhirState1009, _startpos_13, _endpos_14) in
                        let _startpos_15 = _menhir_lexbuf.Lexing.lex_start_p in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_15) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v_16 ->
                            _menhir_run_0362 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState1015
                        | Tokens.RBRACE ->
                            let _v_17 = _menhir_action_199 () in
                            _menhir_run_1016 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17
                        | _ ->
                            _eRR ())
                    | Tokens.SEMI_COLON | Tokens.SUBTYPES ->
                        let (_menhir_s, _endpos__1_, _startpos__1_, _1) = (MenhirState1009, _endpos_14, _startpos_13, ()) in
                        let _v = _menhir_action_471 _1 _endpos__1_ _startpos__1_ in
                        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | Tokens.ENUMERATION ->
                    let _menhir_s = MenhirState1009 in
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _menhir_stack = MenhirCell1_ENUMERATION (_menhir_stack, _menhir_s, _startpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACE ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                        let _menhir_s = MenhirState1019 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_1020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1009
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0360 : type  ttv_stack. (ttv_stack _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_WITH (_menhir_stack, _menhir_s) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0362 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0361
            | Tokens.RBRACE ->
                let _v = _menhir_action_199 () in
                _menhir_run_0372 _menhir_stack _menhir_lexbuf _menhir_lexer _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0362 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0363 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0367 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0007 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_458 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0008 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let (_startpos__1_, _endpos__3_) = (_startpos, _endpos) in
            let _v = _menhir_action_463 _endpos__3_ _startpos__1_ in
            _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
        | Tokens.REAL ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | Tokens.INTEGER ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0008
        | Tokens.BOOLEAN ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | Tokens.BITS ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | Tokens.BIT ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | Tokens.ARRAY ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0008
        | _ ->
            _eRR ()
    
    and _menhir_run_0010 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0011
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0011
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0011
            | Tokens.RBRACE ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let (_startpos__1_, _1, _endpos__3_) = (_startpos, (), _endpos_1) in
                let _v = _menhir_action_455 _1 _endpos__3_ _startpos__1_ in
                _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0011
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0011
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0011
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0011
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0011
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0011
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0011
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0011
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0011
            | _ ->
                _eRR ())
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RECURSELIMIT | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.SUBTYPES | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
            let _v = _menhir_action_457 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0012 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_s_, _startpos_s_, s) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_020 _endpos_s_ _startpos_s_ s in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_s_ _startpos_s_ _v _menhir_s _tok
    
    and _menhir_goto_expr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1370 ->
            _menhir_run_1371 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1364 ->
            _menhir_run_1365 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1356 ->
            _menhir_run_1357 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1353 ->
            _menhir_run_1354 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1348 ->
            _menhir_run_1349 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1342 ->
            _menhir_run_1343 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1321 ->
            _menhir_run_1339 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1300 ->
            _menhir_run_1301 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1298 ->
            _menhir_run_1299 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1288 ->
            _menhir_run_1289 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1286 ->
            _menhir_run_1287 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1284 ->
            _menhir_run_1285 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1278 ->
            _menhir_run_1279 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1273 ->
            _menhir_run_1274 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1263 ->
            _menhir_run_1264 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1253 ->
            _menhir_run_1254 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1248 ->
            _menhir_run_1249 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1238 ->
            _menhir_run_1239 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1230 ->
            _menhir_run_1231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1225 ->
            _menhir_run_1226 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1215 ->
            _menhir_run_1216 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1203 ->
            _menhir_run_1204 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1182 ->
            _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1157 ->
            _menhir_run_1158 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1153 ->
            _menhir_run_1154 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1145 ->
            _menhir_run_1148 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1139 ->
            _menhir_run_1142 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1102 ->
            _menhir_run_1120 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1077 ->
            _menhir_run_1078 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1075 ->
            _menhir_run_1076 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1067 ->
            _menhir_run_1068 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1059 ->
            _menhir_run_1060 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1051 ->
            _menhir_run_1052 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1046 ->
            _menhir_run_1047 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1038 ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0969 ->
            _menhir_run_0970 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0963 ->
            _menhir_run_0964 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0957 ->
            _menhir_run_0958 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0952 ->
            _menhir_run_0953 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0946 ->
            _menhir_run_0947 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0916 ->
            _menhir_run_0917 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0883 ->
            _menhir_run_0884 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0874 ->
            _menhir_run_0875 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0868 ->
            _menhir_run_0869 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0862 ->
            _menhir_run_0863 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0857 ->
            _menhir_run_0858 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0851 ->
            _menhir_run_0852 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0844 ->
            _menhir_run_0845 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0838 ->
            _menhir_run_0839 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0832 ->
            _menhir_run_0833 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0827 ->
            _menhir_run_0828 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0821 ->
            _menhir_run_0822 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0815 ->
            _menhir_run_0816 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0810 ->
            _menhir_run_0811 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0787 ->
            _menhir_run_0788 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0752 ->
            _menhir_run_0770 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0439 ->
            _menhir_run_0719 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0441 ->
            _menhir_run_0715 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0710 ->
            _menhir_run_0711 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0706 ->
            _menhir_run_0707 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0478 ->
            _menhir_run_0692 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0671 ->
            _menhir_run_0672 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0666 ->
            _menhir_run_0667 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0661 ->
            _menhir_run_0662 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0651 ->
            _menhir_run_0652 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0641 ->
            _menhir_run_0642 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0636 ->
            _menhir_run_0637 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0626 ->
            _menhir_run_0627 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0618 ->
            _menhir_run_0619 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0613 ->
            _menhir_run_0614 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0603 ->
            _menhir_run_0604 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0592 ->
            _menhir_run_0593 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0590 ->
            _menhir_run_0591 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0507 ->
            _menhir_run_0575 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0545 ->
            _menhir_run_0546 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0539 ->
            _menhir_run_0540 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0529 ->
            _menhir_run_0530 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0526 ->
            _menhir_run_0527 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0522 ->
            _menhir_run_0523 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0519 ->
            _menhir_run_0520 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0514 ->
            _menhir_run_0515 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0498 ->
            _menhir_run_0499 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0496 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0494 ->
            _menhir_run_0495 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0485 ->
            _menhir_run_0486 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0457 ->
            _menhir_run_0458 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0402 ->
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0400 ->
            _menhir_run_0401 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0003 ->
            _menhir_run_0354 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0015 ->
            _menhir_run_0343 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0016 ->
            _menhir_run_0342 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0017 ->
            _menhir_run_0337 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0334 ->
            _menhir_run_0335 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0019 ->
            _menhir_run_0333 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0223 ->
            _menhir_run_0311 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0309 ->
            _menhir_run_0310 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0301 ->
            _menhir_run_0302 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0299 ->
            _menhir_run_0300 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0297 ->
            _menhir_run_0298 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0295 ->
            _menhir_run_0296 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0293 ->
            _menhir_run_0294 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0291 ->
            _menhir_run_0292 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0289 ->
            _menhir_run_0290 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0282 ->
            _menhir_run_0283 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0279 ->
            _menhir_run_0281 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0277 ->
            _menhir_run_0278 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0275 ->
            _menhir_run_0276 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0273 ->
            _menhir_run_0274 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0260 ->
            _menhir_run_0261 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0257 ->
            _menhir_run_0258 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0255 ->
            _menhir_run_0256 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0253 ->
            _menhir_run_0254 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0251 ->
            _menhir_run_0252 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0249 ->
            _menhir_run_0250 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0246 ->
            _menhir_run_0248 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0244 ->
            _menhir_run_0245 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0242 ->
            _menhir_run_0243 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0240 ->
            _menhir_run_0241 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0238 ->
            _menhir_run_0239 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0236 ->
            _menhir_run_0237 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0234 ->
            _menhir_run_0235 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0232 ->
            _menhir_run_0233 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0220 ->
            _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0210 ->
            _menhir_run_0211 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1323 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1333 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1325 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1311 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1257 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1211 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1184 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1194 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1186 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1171 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1162 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1104 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1114 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1106 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0783 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0754 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0764 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0756 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0445 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0449 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0480 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0686 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0482 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0645 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0489 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0502 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0509 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0569 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0511 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0339 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0021 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0196 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0199 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0205 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0194 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0189 ->
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0184 ->
            _menhir_run_0185 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0182 ->
            _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0180 ->
            _menhir_run_0181 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0178 ->
            _menhir_run_0179 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0176 ->
            _menhir_run_0177 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0172 ->
            _menhir_run_0173 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0168 ->
            _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0166 ->
            _menhir_run_0167 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0154 ->
            _menhir_run_0155 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0148 ->
            _menhir_run_0149 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0144 ->
            _menhir_run_0145 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0460 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0033 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0326 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0262 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0156 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0152 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0150 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0146 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0049 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0129 ->
            _menhir_run_0130 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0011 ->
            _menhir_run_0128 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0304 ->
            _menhir_run_0128 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0133 ->
            _menhir_run_0128 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0131 ->
            _menhir_run_0128 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0124 ->
            _menhir_run_0128 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0121 ->
            _menhir_run_0122 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0119 ->
            _menhir_run_0120 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0117 ->
            _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0114 ->
            _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0112 ->
            _menhir_run_0113 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0110 ->
            _menhir_run_0111 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0108 ->
            _menhir_run_0109 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0106 ->
            _menhir_run_0107 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0104 ->
            _menhir_run_0105 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0102 ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0100 ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0098 ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0096 ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0093 ->
            _menhir_run_0095 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0091 ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0089 ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0087 ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0085 ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0083 ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0081 ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0076 ->
            _menhir_run_0077 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0066 ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0064 ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0045 ->
            _menhir_run_0046 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0036 ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0034 ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0031 ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0029 ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0027 ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0024 ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1371 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_lexpr (_menhir_stack, _, le, _startpos_le_) = _menhir_stack in
                let (_2, _2_inlined1, _endpos__2_inlined2_, e) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_365 _2 _2_inlined1 _endpos__2_inlined2_ _startpos_le_ e le in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | _ ->
            _eRR ()
    
    and _menhir_run_0029 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_SHR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0029 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0013 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_r_, _startpos_r_, r) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_018 _endpos_r_ _startpos_r_ r in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_r_ _startpos_r_ _v _menhir_s _tok
    
    and _menhir_run_0015 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0015 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0016 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
        let _menhir_s = MenhirState0016 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0017 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0017 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0018 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_i_, _startpos_i_, i) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_016 _endpos_i_ _startpos_i_ i in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_i_ _startpos_i_ _v _menhir_s _tok
    
    and _menhir_run_0019 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0019 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0020 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACE ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0199 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_021 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0021 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0021
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0021
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
            let _endpos__3_ = _endpos in
            let _v = _menhir_action_051 _endpos__3_ _startpos_name_ name in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0021
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0021
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0021
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0021
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0021
        | Tokens.IF ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0021
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0021
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0021
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0021
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0021
        | _ ->
            _eRR ()
    
    and _menhir_run_0023 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_017 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0024 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0024 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0025 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_019 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0199 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0199
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0199
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0199
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
            let _endpos__3_ = _endpos in
            let _v = _menhir_action_068 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0199
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0199
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0199
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0199
        | Tokens.IF ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0199
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0199
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0199
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0199
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0199
        | _ ->
            _eRR ()
    
    and _menhir_run_0198 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACE ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0199 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EQ ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0210 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.SHL | Tokens.SHR ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_021 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0210 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0210 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0036 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_SHL (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0036 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0066 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_RDIV (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0066 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0031 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_POW (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0031 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0081 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_PLUS (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0081 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0091 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_OR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0091 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0093 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0093
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0093
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_064 _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_e_ _v _menhir_s _tok
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0093
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0093
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0093
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0093
        | _ ->
            _eRR ()
    
    and _menhir_run_0083 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_MUL (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0083 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0085 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_MOD (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0085 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0096 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
        let _menhir_s = MenhirState0096 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0098 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LT (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0098 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0027 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LLBRACKET (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0027 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0106 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0106 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0033 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0033 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.COLON ->
            _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0034 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_COLON (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0034 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0038 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_IN (_menhir_stack, _menhir_s) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0039 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BNOT ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_BNOT (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0323 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MASK_LIT _v ->
                    _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LEQ ->
                    _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LBRACE ->
                    _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.GEQ ->
                    _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0040 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_UNKNOWN (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0041 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0162 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0042 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_name_, _startpos_name_, name) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_465 _endpos_name_ _startpos_name_ name in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_name_ _v _menhir_s _tok
    
    and _menhir_run_0043 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_459 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0044 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_BITS (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0045 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0062 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_461 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0063 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ARRAY (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACKET ->
            let _menhir_s = MenhirState0064 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0164 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_s_, _startpos_s_, s) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_075 _endpos_s_ _startpos_s_ s in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_s_ _startpos_s_ _v _menhir_s _tok
    
    and _menhir_goto_expr_pattern : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0171 ->
            _menhir_run_0318 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0517 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0323 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0039 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0315 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0174 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0224 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0270 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0266 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0229 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0318 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SLICING ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0232 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0234 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0236 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (x, _endpos__3_) = (_v, _endpos_0) in
            let _v = _menhir_action_122 _endpos__3_ _startpos__1_ x in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0238 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0240 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0242 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0244 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0246 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0249 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0251 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0253 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0255 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0257 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0260 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0262 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0265 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0273 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0275 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0277 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0279 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0282 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0284 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0289 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0291 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0293 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0295 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0297 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0299 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0301 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0303 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0309 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COMMA ->
            let (_endpos_expr_pattern_, _startpos_expr_pattern_, expr_pattern) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_404 _endpos_expr_pattern_ _startpos_expr_pattern_ expr_pattern in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_pattern_ _startpos_expr_pattern_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0232 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0232 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0234 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0234 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0236 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0236 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0238 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0238 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0240 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0240 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0242 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0242 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0244 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0244 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0246 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_119 _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_e_ _v _menhir_s _tok
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | _ ->
            _eRR ()
    
    and _menhir_run_0249 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0249 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0251 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0251 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0253 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
        let _menhir_s = MenhirState0253 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0255 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0255 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0257 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0257 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0260 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell0_LEQ (_menhir_stack, _startpos) in
        let _menhir_s = MenhirState0260 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0262 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0262 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.COLON ->
            _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0265 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0266 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BNOT ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_BNOT (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0270 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MASK_LIT _v ->
                    _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LEQ ->
                    _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LBRACE ->
                    _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.GEQ ->
                    _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0165 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_r_, _startpos_r_, r) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_073 _endpos_r_ _startpos_r_ r in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_r_ _startpos_r_ _v _menhir_s _tok
    
    and _menhir_run_0166 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0166 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0168 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0168
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0168
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0168
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0168
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0168
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0168
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0168
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0168
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0168
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0168
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0168
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0168
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
            let _v = _menhir_action_406 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_pattern : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0171 ->
            _menhir_run_0314 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0517 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0323 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0039 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0315 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0174 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0270 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0266 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0229 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0224 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0314 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_s = MenhirState0315 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0170 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_410 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0171 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0171 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MASK_LIT _v ->
            _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LEQ ->
            _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.GEQ ->
            _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0172 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0172 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0174 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0174 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MASK_LIT _v ->
            _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LEQ ->
            _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.GEQ ->
            _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0175 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_i_, _startpos_i_, i) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_071 _endpos_i_ _startpos_i_ i in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_i_ _startpos_i_ _v _menhir_s _tok
    
    and _menhir_run_0176 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0176 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0188 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0189
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0189
            | Tokens.RPAR ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let (_endpos__3_, _startpos_name_, name) = (_endpos_2, _startpos, _v) in
                let _v = _menhir_action_106 _endpos__3_ _startpos_name_ name in
                _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
            | Tokens.REAL_LIT _v_3 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0189
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0189
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0189
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0189
            | Tokens.INT_LIT _v_4 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState0189
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0189
            | Tokens.IDENTIFIER _v_5 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0189
            | Tokens.BOOL_LIT _v_6 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0189
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0189
            | Tokens.BITVECTOR_LIT _v_7 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0189
            | _ ->
                _eRR ())
        | Tokens.LBRACE ->
            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0196
            | Tokens.STRING_LIT _v_9 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0196
            | Tokens.REAL_LIT _v_10 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState0196
            | Tokens.RBRACE ->
                let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_11, _endpos, _startpos, _v) in
                let _v = _menhir_action_123 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
                _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0196
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0196
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0196
            | Tokens.INT_LIT _v_12 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0196
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0196
            | Tokens.IDENTIFIER _v_13 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0196
            | Tokens.BOOL_LIT _v_14 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0196
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0196
            | Tokens.BITVECTOR_LIT _v_15 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0196
            | _ ->
                _eRR ())
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_076 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0220 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_GEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0220 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0222 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_072 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0223 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0223 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0224 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0225 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_074 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0228 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0229 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROW | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let (_endpos_x_, _startpos_x_, x) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_436 x in
            _menhir_goto_separated_nonempty_list_COMMA_pattern_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _startpos_x_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_pattern_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0517 ->
            _menhir_run_0518 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0323 ->
            _menhir_run_0324 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0039 ->
            _menhir_run_0320 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0315 ->
            _menhir_run_0316 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0174 ->
            _menhir_run_0312 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0270 ->
            _menhir_run_0271 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0266 ->
            _menhir_run_0267 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0229 ->
            _menhir_run_0230 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0224 ->
            _menhir_run_0226 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0518 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_WHEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHERE ->
            let _menhir_s = MenhirState0519 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0555 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0400 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0400 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0405 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0405
        | Tokens.LPAR ->
            _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0405
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.COMMA ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0405, _v, _startpos_0, _endpos) in
                let _menhir_s = MenhirState0429 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0405, _v, _startpos_0, _endpos) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer
            | Tokens.COLON ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0405, _v, _startpos_0, _endpos) in
                _menhir_run_0415 _menhir_stack _menhir_lexbuf _menhir_lexer
            | Tokens.EQ | Tokens.SEMI_COLON ->
                let _v =
                  let _1 = _v in
                  _menhir_action_010 _1
                in
                _menhir_run_0749 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0405 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0406 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _menhir_s = MenhirState0407 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _menhir_s = MenhirState0409 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR | Tokens.SEMI_COLON ->
            let _1 = () in
            let _v = _menhir_action_011 _1 in
            _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_decl_item : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1319 ->
            _menhir_run_1320 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1180 ->
            _menhir_run_1181 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1087 ->
            _menhir_run_1099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0405 ->
            _menhir_run_0749 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0505 ->
            _menhir_run_0506 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0476 ->
            _menhir_run_0477 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0426 ->
            _menhir_run_0425 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0418 ->
            _menhir_run_0425 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0411 ->
            _menhir_run_0417 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1320 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1321
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState1321
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState1321
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1321
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1321
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1321
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState1321
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1321
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1323
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState1323
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState1323
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.UNKNOWN ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1325
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState1325
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.EOF ->
                                        let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                                        let (_1, _2, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, _endpos__2_inlined2_, lhs, _endpos__3_inlined2_) = ((), (), (), (), (), _startpos, _v_3, _endpos_11, _v, _endpos_10) in
                                        let _v = _menhir_action_378 _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_opn _menhir_stack _v
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_12 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState1325
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1325
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1325
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1325
                            | Tokens.INT_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState1325
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1325
                            | Tokens.IDENTIFIER _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState1325
                            | Tokens.BOOL_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState1325
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1325
                            | Tokens.BITVECTOR_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1323, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState1325
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_17 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_068 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_1339 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_17 MenhirState1321 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1323
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1323
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1323
                    | Tokens.INT_LIT _v_18 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState1323
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1323
                    | Tokens.IDENTIFIER _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState1323
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState1323) in
                        let _menhir_s = MenhirState1333 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_26 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState1323
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1323
                    | Tokens.BITVECTOR_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1321, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState1323
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_28 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_021 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_1339 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_28 MenhirState1321 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_29 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState1321
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1321
            | Tokens.BITVECTOR_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState1321
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1339 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, x, _endpos__2_inlined1_, _3) = ((), (), _v, _endpos_0, ()) in
                let _v = _menhir_action_364 _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
        | _ ->
            _eRR ()
    
    and _menhir_run_0112 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_IMPL (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0112 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0108 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_GT (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0108 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0110 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_GEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0110 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0114 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0114
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0114
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0114
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0114
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0114
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_063 _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_sub_expr_ _v _menhir_s _tok
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0114
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0114
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0114
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0114
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0114
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0114
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0114
        | _ ->
            _eRR ()
    
    and _menhir_run_0100 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_EOR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0100 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0068 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_DOT (_menhir_stack, _menhir_s) in
            let _menhir_s = MenhirState0069 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _endpos_x_, x) = ((), _endpos, _v) in
            let _v = _menhir_action_056 _2 _endpos_x_ _startpos_e_ e x in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0070 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0071 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON | Tokens.COLON_COLON | Tokens.RBRACKET ->
            let x = _v in
            let _v = _menhir_action_416 x in
            _menhir_goto_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_IDENTIFIER_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1270 ->
            _menhir_run_1271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1245 ->
            _menhir_run_1246 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1222 ->
            _menhir_run_1223 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1089 ->
            _menhir_run_1090 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0658 ->
            _menhir_run_0659 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0633 ->
            _menhir_run_0634 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0610 ->
            _menhir_run_0611 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0464 ->
            _menhir_run_0466 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0429 ->
            _menhir_run_0430 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0285 ->
            _menhir_run_0286 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0069 ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0071 ->
            _menhir_run_0072 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1271 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1273 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1246 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1248 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1223 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1225 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1090 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState1091 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState1095 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0659 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0661 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0634 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0636 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0611 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0613 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0466 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_lexpr_atom_desc -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
            let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
            let _v = _menhir_action_133 _1_inlined1 _2 _endpos_desc_ _startpos_desc_ desc in
            _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_lexpr_atom_desc : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0457 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0460 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.RBRACKET ->
                    let _endpos_12 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    let (_2, _endpos__3_, _endpos_desc_, _startpos_desc_, desc) = ((), _endpos_12, _endpos, _startpos, _v) in
                    let _v = _menhir_action_132 _2 _endpos_desc_ _startpos_desc_ desc in
                    _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
                | Tokens.IDENTIFIER _v_13 ->
                    let _menhir_stack = MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                    _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0464
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v_14 ->
                let _endpos_16 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let (_2, _endpos_desc_, _startpos_desc_, desc, _endpos_field_, field) = ((), _endpos, _startpos, _v, _endpos_16, _v_14) in
                let _v = _menhir_action_131 _2 _endpos_desc_ _startpos_desc_ desc field in
                _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_field_ _startpos_desc_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR ->
            let (_endpos_desc_, _startpos_desc_, desc) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_125 _endpos_desc_ _startpos_desc_ desc in
            _menhir_goto_lexpr _menhir_stack _menhir_lexbuf _menhir_lexer _startpos_desc_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_lexpr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1074 ->
            _menhir_run_1369 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1374 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1362 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1358 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1306 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1302 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1294 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1290 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1205 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1151 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1128 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1133 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1083 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1079 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1002 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0994 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0982 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0399 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0937 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0797 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0924 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0918 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0908 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0891 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0885 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0805 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0776 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0778 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0404 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0774 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0771 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0768 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0762 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0758 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0750 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0747 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0433 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0437 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0438 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0741 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0737 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0739 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0732 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0728 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0720 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0440 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0716 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0442 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0443 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0712 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0708 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0703 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0447 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0698 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0450 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0452 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0693 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0690 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0684 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0484 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0487 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0679 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0673 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0675 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0668 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0663 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0653 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0649 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0643 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0638 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0628 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0624 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0620 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0615 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0605 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0491 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0598 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0600 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0594 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0596 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0586 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0588 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0500 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0584 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0581 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0504 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0576 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0573 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0567 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0513 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0562 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0555 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0558 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0521 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0552 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0524 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0543 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0547 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0531 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0541 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0536 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0474 ->
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0470 ->
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0454 ->
            _menhir_run_0469 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1369 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1370 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0538 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0539 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0473 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
            let _menhir_s = MenhirState0474 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0455 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let x = _v in
            let _v = _menhir_action_428 x in
            _menhir_goto_separated_nonempty_list_COMMA_lexpr_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0453 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_126 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_lexpr _menhir_stack _menhir_lexbuf _menhir_lexer _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0454 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0454 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0455 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0455 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_128 _1 in
        _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_goto_separated_nonempty_list_COMMA_lexpr_ : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0474 ->
            _menhir_run_0475 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0470 ->
            _menhir_run_0471 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0475 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr, ttv_result) _menhir_cell1_lexpr -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_lexpr (_menhir_stack, _menhir_s, x, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_429 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_lexpr_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0471 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_lexpr -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_lexpr (_menhir_stack, _, x, _) = _menhir_stack in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_127 _1_inlined1 _2 _endpos__3_ _startpos__1_ x in
        _menhir_goto_lexpr _menhir_stack _menhir_lexbuf _menhir_lexer _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0469 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_s = MenhirState0470 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0455 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0430 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0431 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0745 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0286 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
            let _v = _menhir_action_112 _1_inlined1 _2 _endpos__3_ _startpos_e_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0073 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOT -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_DOT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
            let _v = _menhir_action_057 _1_inlined1 _2 _endpos__3_ _startpos_e_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0072 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_417 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0087 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_DIVRM (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0087 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0089 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_DIV (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0089 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0076 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_CONCAT (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0076 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0102 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_COLON_COLON (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0102 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0117 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_BOR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0117 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0119 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_BEQ (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0119 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0121 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_BAND (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0121 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0123 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | Tokens.REAL ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0124
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0124
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0124
            | Tokens.RBRACE ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
                let (_startpos__1_, _2, _endpos__3_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_059 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr in
                _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0124
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0124
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0124
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0124
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0124
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0124
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0124
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0124
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0123, _startpos) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0124
            | _ ->
                _eRR ())
        | Tokens.INTEGER ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0123
        | Tokens.BOOLEAN ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | Tokens.BITS ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | Tokens.BIT ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | Tokens.ARRAY ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0123
        | _ ->
            _eRR ()
    
    and _menhir_run_0104 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_AND (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0104 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_1181 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_LET as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState1182
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState1182
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState1182
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1184
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState1184
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState1184
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.UNKNOWN ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1186
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState1186
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.EOF ->
                                        let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                                        let (_1, _2, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, _endpos__2_inlined2_, lhs, _endpos__3_inlined2_) = ((), (), (), (), (), _startpos, _v_3, _endpos_11, _v, _endpos_10) in
                                        let _v = _menhir_action_375 _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_opn _menhir_stack _v
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_12 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState1186
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1186
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1186
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1186
                            | Tokens.INT_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState1186
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1186
                            | Tokens.IDENTIFIER _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState1186
                            | Tokens.BOOL_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState1186
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1186
                            | Tokens.BITVECTOR_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1184, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState1186
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_17 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_068 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_17 MenhirState1182 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1184
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1184
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1184
                    | Tokens.INT_LIT _v_18 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState1184
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1184
                    | Tokens.IDENTIFIER _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState1184
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState1184) in
                        let _menhir_s = MenhirState1194 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_26 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState1184
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1184
                    | Tokens.BITVECTOR_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1182, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState1184
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_28 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_021 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_28 MenhirState1182 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_29 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState1182
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
            | Tokens.BITVECTOR_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState1182
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1200 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, x, _endpos__2_inlined1_, _3) = ((), (), _v, _endpos_0, ()) in
                let _v = _menhir_action_363 _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1200
        | _ ->
            _eRR ()
    
    and _menhir_run_1099 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, ldi) = ((), (), _endpos, _v) in
                let _v = _menhir_action_381 _1 _2 _endpos__2_inlined1_ _startpos__1_ ldi in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState1102
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState1102
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState1102
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1104
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState1104
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState1104
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.UNKNOWN ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1106
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState1106
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.EOF ->
                                        let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                                        let (_1, _2, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, _endpos__2_inlined2_, lhs, _endpos__3_inlined2_) = ((), (), (), (), (), _startpos, _v_3, _endpos_11, _v, _endpos_10) in
                                        let _v = _menhir_action_385 _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_opn _menhir_stack _v
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_12 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState1106
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1106
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1106
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1106
                            | Tokens.INT_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState1106
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1106
                            | Tokens.IDENTIFIER _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState1106
                            | Tokens.BOOL_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState1106
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1106
                            | Tokens.BITVECTOR_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState1104, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState1106
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_17 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_068 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_1120 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_17 MenhirState1102 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1104
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1104
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1104
                    | Tokens.INT_LIT _v_18 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState1104
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1104
                    | Tokens.IDENTIFIER _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState1104
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState1104) in
                        let _menhir_s = MenhirState1114 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_26 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState1104
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1104
                    | Tokens.BITVECTOR_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1102, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState1104
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_28 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_021 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_1120 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_28 MenhirState1102 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_29 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState1102
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
            | Tokens.BITVECTOR_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState1102
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1120 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined1, _endpos__2_inlined2_) = ((), (), _v, (), _endpos_0) in
                let _v = _menhir_action_382 _1 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _startpos__1_ ldi in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
        | _ ->
            _eRR ()
    
    and _menhir_run_0749 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_VAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0750
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0750
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, ldi) = ((), _endpos, _v) in
                let _v = _menhir_action_271 _1 _endpos__2_ _startpos__1_ ldi in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0752
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0752
            | Tokens.REAL_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0752
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0752
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0752
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0752
            | Tokens.INT_LIT _v_3 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0752
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0752
            | Tokens.IDENTIFIER _v_4 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_5 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0754
                    | Tokens.STRING_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0754
                    | Tokens.REAL_LIT _v_7 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0754
                    | Tokens.RBRACE ->
                        let _endpos_8 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_9 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.UNKNOWN ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0756
                            | Tokens.STRING_LIT _v_10 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState0756
                            | Tokens.RPAR ->
                                let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_12 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.WHILE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.VAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.UNREACHABLE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.TRY ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.THROW ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.RETURN ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.REPEAT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.PRINT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.PRAGMA ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.PASS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.MINUS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.LPAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.LET ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.IF ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.IDENTIFIER _v_13 ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0758
                                    | Tokens.FOR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.DEBUG ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.CONSTANT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.CASE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.ASSERT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.ARROBASE_LOOPLIMIT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0756, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0758
                                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                                        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                                        let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, lhs, _endpos__3_inlined2_) = ((), _endpos_12, (), (), (), _startpos, _v_4, _v, _endpos_11) in
                                        let _v = _menhir_action_275 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0756
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0756
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0756
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0756
                            | Tokens.INT_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0756
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0756
                            | Tokens.IDENTIFIER _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0756
                            | Tokens.BOOL_LIT _v_17 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17 MenhirState0756
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0756
                            | Tokens.BITVECTOR_LIT _v_18 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0754, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState0756
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_19 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_8, _endpos, _startpos, _v_4) in
                              _menhir_action_068 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0770 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_8 _startpos_5 _v_19 MenhirState0752 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0754
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0754
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0754
                    | Tokens.INT_LIT _v_20 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState0754
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0754
                    | Tokens.IDENTIFIER _v_21 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v_21 MenhirState0754
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0754) in
                        let _menhir_s = MenhirState0764 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_28 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0754
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0754
                    | Tokens.BITVECTOR_LIT _v_29 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0752, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState0754
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_30 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_4) in
                      _menhir_action_021 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0770 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_30 MenhirState0752 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_31 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState0752
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0752
            | Tokens.BITVECTOR_LIT _v_32 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_32 MenhirState0752
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0434 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.IDENTIFIER _v ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0437
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_startpos__1_, _1, _endpos__2_, _2_inlined1, _3) = (_startpos, (), _endpos_1, (), ()) in
                        let _v = _menhir_action_282 _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0438 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0438 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0439 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0439
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0440
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0439, _endpos) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0440
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_287 _1 _endpos__2_ _startpos__1_ in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0439
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0439
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0439
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0439
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0439
        | _ ->
            _eRR ()
    
    and _menhir_run_0441 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0441
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0442
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0441, _endpos) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0442
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_247 _1 _endpos__2_ _startpos__1_ in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0441
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0441
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0441
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0441
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0441
        | _ ->
            _eRR ()
    
    and _menhir_run_0443 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0443 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0444 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0445
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.IDENTIFIER _v ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0447
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0445, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0447
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos_1) in
                        let _v = _menhir_action_278 _1 _endpos__2_ _startpos__1_ in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0445
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0445
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0445
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0445
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0445
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0448 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0449
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0449
            | Tokens.SEMI_COLON ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.IDENTIFIER _v_3 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0450
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0449, _endpos_2) in
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let (_startpos__1_, _1, _endpos__2_, _endpos__2_inlined1_, _2_inlined1) = (_startpos, (), _endpos_2, _endpos, _v) in
                    let _v = _menhir_action_288 _1 _2_inlined1 _endpos__2_ _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v_4 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState0449
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0449
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0449
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0449
            | Tokens.INT_LIT _v_5 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0449
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0449
            | Tokens.IDENTIFIER _v_6 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0449
            | Tokens.BOOL_LIT _v_7 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0449
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0449
            | Tokens.BITVECTOR_LIT _v_8 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_8 MenhirState0449
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0451 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0452
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0452
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_246 _1 _endpos__2_ _startpos__1_ in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0476 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0476 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0411 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0411 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0412 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0415 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR ->
            let _1 = _v in
            let _v = _menhir_action_010 _1 in
            _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0413 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0413 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REAL ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INTEGER ->
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOLEAN ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITS ->
            _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BIT ->
            _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARRAY ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0415 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0415 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REAL ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INTEGER ->
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOLEAN ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITS ->
            _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BIT ->
            _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARRAY ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0485 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0485 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0488 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0489
            | Tokens.RPAR ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_3 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.IDENTIFIER _v_4 ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState0491
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_endpos__2_, _startpos_name_, name) = (_endpos_3, _startpos, _v) in
                        let _v = _menhir_action_249 _endpos__2_ _startpos_name_ name in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | Tokens.EQ ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                    let _menhir_s = MenhirState0603 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.DOT ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0489, _endpos_2) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACKET ->
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.COMMA ->
                                let _menhir_s = MenhirState0610 in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.IDENTIFIER _v ->
                                    _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | _ ->
                                    _eRR ())
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EQ ->
                            let _menhir_s = MenhirState0618 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.UNKNOWN ->
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.STRING_LIT _v ->
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.REAL_LIT _v ->
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.NOT ->
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.MINUS ->
                                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.LPAR ->
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.INT_LIT _v ->
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.IF ->
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BOOL_LIT _v ->
                                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BNOT ->
                                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BITVECTOR_LIT _v ->
                                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v_24 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_24 MenhirState0489
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
            | Tokens.INT_LIT _v_25 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_25 MenhirState0489
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
            | Tokens.IDENTIFIER _v_26 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState0489
            | Tokens.BOOL_LIT _v_27 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0489
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
            | Tokens.BITVECTOR_LIT _v_28 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0489
            | _ ->
                _eRR ())
        | Tokens.LBRACE ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0645 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT | Tokens.EQ | Tokens.LBRACKET | Tokens.LLBRACKET ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_128 _1 in
            _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0492 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0494 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0501 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0502
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0502
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.IDENTIFIER _v ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0504
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0502, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0504
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos_1) in
                        let _v = _menhir_action_280 _1 _endpos__2_ _startpos__1_ in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0502
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0502
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0502
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0502
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0502
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0502
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0502
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0502
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0502
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0502
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0505 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0505 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0514 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0514 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0522 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0522 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0525 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0526 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_goto_nonempty_list_stmt_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1374 ->
            _menhir_run_1376 _menhir_stack _v _tok
        | MenhirState1362 ->
            _menhir_run_1363 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1358 ->
            _menhir_run_1359 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1306 ->
            _menhir_run_1307 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1302 ->
            _menhir_run_1303 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1294 ->
            _menhir_run_1295 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1290 ->
            _menhir_run_1291 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1205 ->
            _menhir_run_1206 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1151 ->
            _menhir_run_1152 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1133 ->
            _menhir_run_1134 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1128 ->
            _menhir_run_1129 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1083 ->
            _menhir_run_1084 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1079 ->
            _menhir_run_1080 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1002 ->
            _menhir_run_1003 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0994 ->
            _menhir_run_0995 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0982 ->
            _menhir_run_0983 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0937 ->
            _menhir_run_0938 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0924 ->
            _menhir_run_0925 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0918 ->
            _menhir_run_0919 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0908 ->
            _menhir_run_0909 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0891 ->
            _menhir_run_0892 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0885 ->
            _menhir_run_0886 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0805 ->
            _menhir_run_0806 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0797 ->
            _menhir_run_0798 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0399 ->
            _menhir_run_0780 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0778 ->
            _menhir_run_0779 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0776 ->
            _menhir_run_0777 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0774 ->
            _menhir_run_0775 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0404 ->
            _menhir_run_0773 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0771 ->
            _menhir_run_0772 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0768 ->
            _menhir_run_0769 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0762 ->
            _menhir_run_0763 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0758 ->
            _menhir_run_0759 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0750 ->
            _menhir_run_0751 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0747 ->
            _menhir_run_0748 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0433 ->
            _menhir_run_0744 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0437 ->
            _menhir_run_0743 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0741 ->
            _menhir_run_0742 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0739 ->
            _menhir_run_0740 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0737 ->
            _menhir_run_0738 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0732 ->
            _menhir_run_0733 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0728 ->
            _menhir_run_0729 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0438 ->
            _menhir_run_0722 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0720 ->
            _menhir_run_0721 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0440 ->
            _menhir_run_0718 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0716 ->
            _menhir_run_0717 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0442 ->
            _menhir_run_0714 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0712 ->
            _menhir_run_0713 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0708 ->
            _menhir_run_0709 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0443 ->
            _menhir_run_0705 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0703 ->
            _menhir_run_0704 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0447 ->
            _menhir_run_0700 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0698 ->
            _menhir_run_0699 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0450 ->
            _menhir_run_0696 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0452 ->
            _menhir_run_0695 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0693 ->
            _menhir_run_0694 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0690 ->
            _menhir_run_0691 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0684 ->
            _menhir_run_0685 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0484 ->
            _menhir_run_0681 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0679 ->
            _menhir_run_0680 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0675 ->
            _menhir_run_0676 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0673 ->
            _menhir_run_0674 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0487 ->
            _menhir_run_0670 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0668 ->
            _menhir_run_0669 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0663 ->
            _menhir_run_0664 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0653 ->
            _menhir_run_0654 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0649 ->
            _menhir_run_0650 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0643 ->
            _menhir_run_0644 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0638 ->
            _menhir_run_0639 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0628 ->
            _menhir_run_0629 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0624 ->
            _menhir_run_0625 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0620 ->
            _menhir_run_0621 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0615 ->
            _menhir_run_0616 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0605 ->
            _menhir_run_0606 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0491 ->
            _menhir_run_0602 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0600 ->
            _menhir_run_0601 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0598 ->
            _menhir_run_0599 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0596 ->
            _menhir_run_0597 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0594 ->
            _menhir_run_0595 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0588 ->
            _menhir_run_0589 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0586 ->
            _menhir_run_0587 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0584 ->
            _menhir_run_0585 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0500 ->
            _menhir_run_0583 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0581 ->
            _menhir_run_0582 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0504 ->
            _menhir_run_0578 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0576 ->
            _menhir_run_0577 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0573 ->
            _menhir_run_0574 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0567 ->
            _menhir_run_0568 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0513 ->
            _menhir_run_0564 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0562 ->
            _menhir_run_0563 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0558 ->
            _menhir_run_0559 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0555 ->
            _menhir_run_0556 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0552 ->
            _menhir_run_0553 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0521 ->
            _menhir_run_0550 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0524 ->
            _menhir_run_0549 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0547 ->
            _menhir_run_0548 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0543 ->
            _menhir_run_0544 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0541 ->
            _menhir_run_0542 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0536 ->
            _menhir_run_0537 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0531 ->
            _menhir_run_0532 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1363 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState1364 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1359 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1359
        | _ ->
            _eRR ()
    
    and _menhir_run_0533 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_END (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let (_endpos_x_, x) = (_endpos, ()) in
            let _v = _menhir_action_401 x in
            _menhir_goto_option_SEMI_COLON_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _v _tok
        | Tokens.ARROBASE_LOOPLIMIT | Tokens.ASSERT | Tokens.CASE | Tokens.CATCH | Tokens.CONFIG | Tokens.CONSTANT | Tokens.DEBUG | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.FOR | Tokens.FUNC | Tokens.GETTER | Tokens.IDENTIFIER _ | Tokens.IF | Tokens.LET | Tokens.LPAR | Tokens.MINUS | Tokens.OTHERWISE | Tokens.PASS | Tokens.PRAGMA | Tokens.PRINT | Tokens.REPEAT | Tokens.RETURN | Tokens.SETTER | Tokens.THROW | Tokens.TRY | Tokens.TYPE | Tokens.UNREACHABLE | Tokens.UNTIL | Tokens.VAR | Tokens.WHEN | Tokens.WHILE ->
            let _v = _menhir_action_400 () in
            _menhir_goto_option_SEMI_COLON_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_option_SEMI_COLON_ : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_END -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_END (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_opt_, opt) = ((), _endpos, _v) in
        let _v = _menhir_action_015 _1 _endpos_opt_ _startpos__1_ opt in
        _menhir_goto_end_semicolon _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_opt_ _v _menhir_s _tok
    
    and _menhir_goto_end_semicolon : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1359 ->
            _menhir_run_1360 _menhir_stack _endpos _tok
        | MenhirState1345 ->
            _menhir_run_1346 _menhir_stack _endpos _tok
        | MenhirState1307 ->
            _menhir_run_1308 _menhir_stack _endpos _tok
        | MenhirState1303 ->
            _menhir_run_1304 _menhir_stack _endpos _tok
        | MenhirState1295 ->
            _menhir_run_1296 _menhir_stack _endpos _tok
        | MenhirState1291 ->
            _menhir_run_1292 _menhir_stack _endpos _tok
        | MenhirState1207 ->
            _menhir_run_1208 _menhir_stack _endpos _tok
        | MenhirState1131 ->
            _menhir_run_1137 _menhir_stack _endpos _tok
        | MenhirState1134 ->
            _menhir_run_1135 _menhir_stack _endpos _tok
        | MenhirState1084 ->
            _menhir_run_1085 _menhir_stack _endpos _tok
        | MenhirState1080 ->
            _menhir_run_1081 _menhir_stack _endpos _tok
        | MenhirState1002 ->
            _menhir_run_1006 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1003 ->
            _menhir_run_1004 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0994 ->
            _menhir_run_0998 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0995 ->
            _menhir_run_0996 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0982 ->
            _menhir_run_0986 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0983 ->
            _menhir_run_0984 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0399 ->
            _menhir_run_0978 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0937 ->
            _menhir_run_0941 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0938 ->
            _menhir_run_0939 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0797 ->
            _menhir_run_0931 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0924 ->
            _menhir_run_0928 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0925 ->
            _menhir_run_0926 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0918 ->
            _menhir_run_0922 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0919 ->
            _menhir_run_0920 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0908 ->
            _menhir_run_0912 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0909 ->
            _menhir_run_0910 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0891 ->
            _menhir_run_0895 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0892 ->
            _menhir_run_0893 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0885 ->
            _menhir_run_0889 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0886 ->
            _menhir_run_0887 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0805 ->
            _menhir_run_0879 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0806 ->
            _menhir_run_0807 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0798 ->
            _menhir_run_0799 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0780 ->
            _menhir_run_0781 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0777 ->
            _menhir_run_0778 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0773 ->
            _menhir_run_0774 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0735 ->
            _menhir_run_0741 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0738 ->
            _menhir_run_0739 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0678 ->
            _menhir_run_0679 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0599 ->
            _menhir_run_0600 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0595 ->
            _menhir_run_0596 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0587 ->
            _menhir_run_0588 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0583 ->
            _menhir_run_0584 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0561 ->
            _menhir_run_0562 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0532 ->
            _menhir_run_0536 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1006 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1006
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_1007 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0376 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                _menhir_run_0378 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0377
            | Tokens.LPAR ->
                let _v_1 = _menhir_action_195 () in
                _menhir_run_0392 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0377 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0378 : type  ttv_stack. (ttv_stack _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0378 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0379 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0379 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0380 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0379 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0382 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0386 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let x = _v in
            let _v = _menhir_action_430 x in
            _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0378 ->
            _menhir_run_0390 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0388 ->
            _menhir_run_0389 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0384 ->
            _menhir_run_0385 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0380 ->
            _menhir_run_0381 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0390 : type  ttv_stack. (ttv_stack _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_LBRACE -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let _1_inlined1 = _v in
        let _v = _menhir_action_196 _1_inlined1 in
        _menhir_goto_loption_braced_nclist_opt_typed_identifier___ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_goto_loption_braced_nclist_opt_typed_identifier___ : type  ttv_stack. (ttv_stack _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0801 ->
            _menhir_run_0802 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0791 ->
            _menhir_run_0792 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0377 ->
            _menhir_run_0392 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0802 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0803 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.BEGIN ->
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
                    let _menhir_s = MenhirState0805 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.VAR ->
                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.UNREACHABLE ->
                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.TRY ->
                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.THROW ->
                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.RETURN ->
                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REPEAT ->
                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.PRINT ->
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.PRAGMA ->
                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.PASS ->
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LET ->
                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.FOR ->
                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.END ->
                        _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.DEBUG ->
                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.CONSTANT ->
                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.CASE ->
                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ASSERT ->
                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.ARROW ->
                    let _menhir_s = MenhirState0881 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0897 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0897 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0898 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0902 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0792 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0793 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.ARROW ->
                    let _menhir_s = MenhirState0795 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0897 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0392 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0393 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.COLON_COLON ->
                            let _menhir_s = MenhirState0397 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING ->
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.REAL ->
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.LPAR ->
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.INTEGER ->
                                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BOOLEAN ->
                                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BITS ->
                                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BIT ->
                                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.ARRAY ->
                                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | _ ->
                                _eRR ())
                        | Tokens.COLON ->
                            let _menhir_s = MenhirState0980 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING ->
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.REAL ->
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.LPAR ->
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.INTEGER ->
                                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BOOLEAN ->
                                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BITS ->
                                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BIT ->
                                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.ARRAY ->
                                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0897 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0389 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_434 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0385 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_435 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0381 : type  ttv_stack. (ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_433 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0782 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0783
            | Tokens.STRING_LIT _v_1 ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0783
            | Tokens.SEMI_COLON ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0783, _endpos_2) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.VAR ->
                    _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.TYPE ->
                    _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.SETTER ->
                    _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.PRAGMA ->
                    _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.LET ->
                    _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.GETTER ->
                    _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.FUNC ->
                    _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.CONSTANT ->
                    _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.CONFIG ->
                    _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0784
                | Tokens.EOF ->
                    let _v_3 = _menhir_action_134 () in
                    _menhir_run_0973 _menhir_stack _v_3
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v_4 ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState0783
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0783
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0783
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0783
            | Tokens.INT_LIT _v_5 ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0783
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0783
            | Tokens.IDENTIFIER _v_6 ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0783
            | Tokens.BOOL_LIT _v_7 ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0783
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0783
            | Tokens.BITVECTOR_LIT _v_8 ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_8 MenhirState0783
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0785 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0787 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState0944 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState0950 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0957 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState0961 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState0967 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0790 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                _menhir_run_0378 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0791
            | Tokens.LPAR ->
                let _v_1 = _menhir_action_195 () in
                _menhir_run_0792 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0791 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0800 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                _menhir_run_0378 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
            | Tokens.LPAR ->
                let _v_1 = _menhir_action_195 () in
                _menhir_run_0802 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0801 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0808 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0810 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState0849 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState0855 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0862 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState0866 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState0872 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0813 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0815 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState0819 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState0825 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0832 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState0836 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState0842 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1004 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_1005 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0998 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0999 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0996 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0996
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0997 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0986 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0986
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0987 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0984 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0985 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0978 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0978
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0979 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0941 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0941
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0942 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0939 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0939
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0940 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0931 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0931
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0932 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0928 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0928
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0929 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0926 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0926
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0927 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0922 : type  ttv_stack. (((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0922
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0923 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0920 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0920
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0921 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0912 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0912
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0913 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0910 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0910
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0911 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0895 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0895
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0896 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0893 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0893
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0894 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0889 : type  ttv_stack. (((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0889
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0890 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0887 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0887
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0888 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0879 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0879
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0880 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0807 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0878 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0799 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0799
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0930 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0781 : type  ttv_stack. ((((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_ast) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0781
        | Tokens.EOF ->
            let _v_0 = _menhir_action_134 () in
            _menhir_run_0977 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0778 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0778
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0778
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _4) = ((), _endpos, ()) in
            let _v = _menhir_action_237 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0774 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0774
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0774
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _4, _1_inlined1) = ((), _endpos, (), ()) in
            let _v = _menhir_action_238 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0741 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0741
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0741
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3) = ((), _endpos, ()) in
            let _v = _menhir_action_243 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0739 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_OTHERWISE, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0739
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_OTHERWISE (_menhir_stack, _, _, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3) = ((), _endpos, (), (), ()) in
            let _v = _menhir_action_244 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0679 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_s_else as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0679
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_s_else (_menhir_stack, _, s2, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
            let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3) = ((), _endpos, ()) in
            let _v = _menhir_action_235 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0600 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0600
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0600
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined1, _3, _8) = ((), _endpos, (), (), ()) in
            let _v = _menhir_action_241 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0596 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0596
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, (), (), (), ()) in
            let _v = _menhir_action_242 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0588 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0588
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0588
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined1, _3, _8) = ((), _endpos, (), (), ()) in
            let _v = _menhir_action_239 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0584 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0584
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0584
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, (), (), (), ()) in
            let _v = _menhir_action_240 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0562 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CASE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OF, ttv_result) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0562
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0562
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_OF (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3) = ((), _endpos, ()) in
            let _v = _menhir_action_236 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0536 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0536
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0536
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
            let MenhirCell0_WHILE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3, _5) = ((), _endpos, (), ()) in
            let _v = _menhir_action_245 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_1307 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | _ ->
            _eRR ()
    
    and _menhir_run_1303 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1303
        | _ ->
            _eRR ()
    
    and _menhir_run_1295 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1295
        | _ ->
            _eRR ()
    
    and _menhir_run_1291 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1291
        | _ ->
            _eRR ()
    
    and _menhir_run_1206 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ELSIF ->
            _menhir_run_0671 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1206
        | Tokens.ELSE ->
            _menhir_run_0675 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1206
        | Tokens.END ->
            let _v_0 =
              let _endpos__0_ = _endpos in
              _menhir_action_414 _endpos__0_
            in
            _menhir_run_1207 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState1206 _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0671 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0671 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0675 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_ELSE (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0675 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_1207 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_s_else (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1207
        | _ ->
            _eRR ()
    
    and _menhir_run_1152 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState1153 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1134 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1134
        | _ ->
            _eRR ()
    
    and _menhir_run_1129 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.CATCH ->
            let _menhir_s = MenhirState1130 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0724 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0724 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | Tokens.REAL ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | Tokens.LPAR ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | Tokens.INTEGER ->
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.COLON ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0724, _v, _startpos_0, _endpos) in
                let _menhir_s = MenhirState0726 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.ARROW ->
                let _v =
                  let (_endpos_name_, _startpos_name_, name) = (_endpos, _startpos_0, _v) in
                  _menhir_action_465 _endpos_name_ _startpos_name_ name
                in
                _menhir_run_0731 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v MenhirState0724 _tok
            | _ ->
                _eRR ())
        | Tokens.BOOLEAN ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | Tokens.BITS ->
            _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | Tokens.BIT ->
            _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | Tokens.ARRAY ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0724
        | _ ->
            _eRR ()
    
    and _menhir_run_0731 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_WHEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0732 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1084 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | _ ->
            _eRR ()
    
    and _menhir_run_1080 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1080
        | _ ->
            _eRR ()
    
    and _menhir_run_1003 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1003
        | _ ->
            _eRR ()
    
    and _menhir_run_0995 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0995
        | _ ->
            _eRR ()
    
    and _menhir_run_0983 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0983
        | _ ->
            _eRR ()
    
    and _menhir_run_0938 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | _ ->
            _eRR ()
    
    and _menhir_run_0925 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0925
        | _ ->
            _eRR ()
    
    and _menhir_run_0919 : type  ttv_stack. (((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0919
        | _ ->
            _eRR ()
    
    and _menhir_run_0909 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0909
        | _ ->
            _eRR ()
    
    and _menhir_run_0892 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0892
        | _ ->
            _eRR ()
    
    and _menhir_run_0886 : type  ttv_stack. (((((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty, _menhir_box_ast) _menhir_cell1_expr, _menhir_box_ast) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0886
        | _ ->
            _eRR ()
    
    and _menhir_run_0806 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0806
        | _ ->
            _eRR ()
    
    and _menhir_run_0798 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0798
        | _ ->
            _eRR ()
    
    and _menhir_run_0780 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0780
        | _ ->
            _eRR ()
    
    and _menhir_run_0779 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _4) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_292 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0777 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0777
        | _ ->
            _eRR ()
    
    and _menhir_run_0775 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _4, _1_inlined1) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_293 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0773 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0773
        | _ ->
            _eRR ()
    
    and _menhir_run_0772 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined1) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_327 _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0769 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1, _5) = ((), _endpos, _v, (), (), (), ()) in
        let _v = _menhir_action_332 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0763 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_331 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0759 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_330 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0751 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_326 _1 _endpos__2_ _startpos__1_ ldi xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0748 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, ttv_result) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined2, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _1_inlined2) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_328 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0744 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, ttv_result) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined2, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _1_inlined2) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_329 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0743 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_UNREACHABLE _menhir_cell0_LPAR _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_337 _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0742 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_298 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0740 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_OTHERWISE, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
        let MenhirCell1_OTHERWISE (_menhir_stack, _, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _1_inlined2, _3) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_299 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0738 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_OTHERWISE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0738
        | _ ->
            _eRR ()
    
    and _menhir_run_0733 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0724 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.END | Tokens.OTHERWISE ->
            let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_1, _4, _1_inlined1) = ((), (), _v) in
            let _v = _menhir_action_231 _1 _1_inlined1 _4 ty in
            _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_nonempty_list_catcher_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1130 ->
            _menhir_run_1131 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0723 ->
            _menhir_run_0735 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0733 ->
            _menhir_run_0734 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0729 ->
            _menhir_run_0730 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1131 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.OTHERWISE ->
            let _menhir_s = MenhirState1131 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState1133 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0735 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.OTHERWISE ->
            let _menhir_s = MenhirState0735 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState0737 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0735
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0734 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_ty, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_233 _1 _1_inlined1 _4 ty xs in
        _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0730 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_ty, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_234 _1 _1_inlined1 _4 ty x xs in
        _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0729 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0724 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.END | Tokens.OTHERWISE ->
            let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_1, _4, _1_inlined1) = ((), (), _v) in
            let _v = _menhir_action_232 _1 _1_inlined1 _4 ty x in
            _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0722 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_TRY as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.CATCH ->
            let _menhir_s = MenhirState0723 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0724 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0721 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_THROW, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_341 _1 _endpos__2_ _startpos__1_ e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0718 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_THROW, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_342 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0717 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_RETURN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_303 _1 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0714 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_RETURN, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_302 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0713 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined2, _3) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_339 _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0709 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_338 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0705 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState0706 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0704 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRINT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_334 _1 _1_inlined2 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0700 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRINT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_333 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0699 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_344 _1 _1_inlined1 _2_inlined1 _endpos__1_inlined1_ _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0696 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _endpos__2_inlined1_) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_343 _1 _2_inlined1 _endpos__2_ _endpos__2_inlined1_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0695 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_PASS _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_301 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0694 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_308 _1 _3 _endpos__2_ _startpos__1_ decl_item x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0691 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1, _5) = ((), _endpos, _v, (), (), (), ()) in
        let _v = _menhir_action_322 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0685 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_321 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0681 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_320 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0680 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_s_else, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_s_else (_menhir_stack, _, s2, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_290 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0676 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_ELSE (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_1, _endpos__1_inlined1_, _1_inlined1) = ((), _endpos, _v) in
        let _v = _menhir_action_415 _1 _1_inlined1 in
        _menhir_goto_s_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_inlined1_ _v _menhir_s _tok
    
    and _menhir_goto_s_else : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1206 ->
            _menhir_run_1207 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0670 ->
            _menhir_run_0678 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0674 ->
            _menhir_run_0677 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0678 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_s_else (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | _ ->
            _eRR ()
    
    and _menhir_run_0677 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_s2_, s2) = ((), (), _endpos, _v) in
        let _v = _menhir_action_413 _1 _1_inlined1 _3 _endpos_s2_ _startpos__1_ e s2 in
        _menhir_goto_s_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_s2_ _v _menhir_s _tok
    
    and _menhir_run_0674 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ELSIF ->
            _menhir_run_0671 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.ELSE ->
            _menhir_run_0675 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.END ->
            let _v_0 =
              let _endpos__0_ = _endpos in
              _menhir_action_414 _endpos__0_
            in
            _menhir_run_0677 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0670 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ELSIF ->
            _menhir_run_0671 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.ELSE ->
            _menhir_run_0675 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.END ->
            let _v_0 =
              let _endpos__0_ = _endpos in
              _menhir_action_414 _endpos__0_
            in
            _menhir_run_0678 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0670 _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0669 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _4, _2_inlined1) = (_endpos, _v, (), ()) in
        let _v = _menhir_action_316 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0664 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _4, _2_inlined1, _2_inlined2) = (_endpos, _v, (), (), ()) in
        let _v = _menhir_action_319 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0654 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _2_inlined1) = (_endpos, _v, ()) in
        let _v = _menhir_action_313 _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0650 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_306 _1_inlined1 _endpos__2_ _startpos_name_ args name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0644 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _4, _2_inlined1) = (_endpos, _v, (), ()) in
        let _v = _menhir_action_315 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0639 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _4, _2_inlined1, _2_inlined2) = (_endpos, _v, (), (), ()) in
        let _v = _menhir_action_318 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0629 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _2_inlined1) = (_endpos, _v, ()) in
        let _v = _menhir_action_312 _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0625 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_305 _1_inlined1 _endpos__2_ _startpos_name_ name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0621 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _4, _2_inlined1) = (_endpos, _v, (), ()) in
        let _v = _menhir_action_314 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0616 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _4, _2_inlined1, _2_inlined2) = (_endpos, _v, (), (), ()) in
        let _v = _menhir_action_317 _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0606 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs, _2_inlined1) = (_endpos, _v, ()) in
        let _v = _menhir_action_311 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0602 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_304 _endpos__2_ _startpos_name_ name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0601 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_296 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0599 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0599
        | _ ->
            _eRR ()
    
    and _menhir_run_0597 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), (), ()) in
        let _v = _menhir_action_297 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0595 : type  ttv_stack ttv_result. ((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0595
        | _ ->
            _eRR ()
    
    and _menhir_run_0589 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_294 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0587 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
        | _ ->
            _eRR ()
    
    and _menhir_run_0585 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), (), ()) in
        let _v = _menhir_action_295 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0583 : type  ttv_stack ttv_result. ((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
        | _ ->
            _eRR ()
    
    and _menhir_run_0582 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_DEBUG _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_336 _1 _1_inlined2 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0578 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_DEBUG _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_335 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0577 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_309 _1 _3 _endpos__2_ _startpos__1_ decl_item x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0574 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1, _5) = ((), _endpos, _v, (), (), (), ()) in
        let _v = _menhir_action_325 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0568 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_324 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0564 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _2_inlined1, _3, _3_inlined1) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_323 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0563 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CASE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OF, ttv_result) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_OF (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_291 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0559 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_OTHERWISE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_OTHERWISE (_menhir_stack, _, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _endpos__1_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2, _4, _endpos__1_inlined3_, _1_inlined3, desc) = ((), (), (), _endpos, _v, ()) in
        let _v = _menhir_action_206 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _endpos__1_inlined3_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1344 ->
            _menhir_run_1345 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0516 ->
            _menhir_run_0561 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0556 ->
            _menhir_run_0560 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0550 ->
            _menhir_run_0554 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1345 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1345
        | _ ->
            _eRR ()
    
    and _menhir_run_0561 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_CASE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OF as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0561
        | _ ->
            _eRR ()
    
    and _menhir_run_0560 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _endpos__1_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _4, l) = ((), (), _v) in
        let _v = _menhir_action_208 _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ l in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0554 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ARROW, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _endpos__1_inlined3_) = _menhir_stack in
        let MenhirCell1_ARROW (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2, _, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _4, _1_inlined2, l) = ((), (), (), _v) in
        let _v = _menhir_action_209 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ l in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0556 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0517 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0556
        | Tokens.OTHERWISE ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0556 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState0558 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _4, _1_inlined2, _endpos__1_inlined2_) = ((), (), _v, _endpos) in
            let _v = _menhir_action_204 _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ in
            _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0517 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0517 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0166 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MASK_LIT _v ->
            _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LEQ ->
            _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.GEQ ->
            _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0222 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0553 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ARROW, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_OTHERWISE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_OTHERWISE (_menhir_stack, _, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _endpos__1_inlined3_) = _menhir_stack in
        let MenhirCell1_ARROW (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2, _, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _4, _2_inlined1, _1_inlined2, _1_inlined4, _endpos__1_inlined4_, desc) = ((), (), (), (), _v, _endpos, ()) in
        let _v = _menhir_action_207 _1 _1_inlined1 _1_inlined2 _1_inlined3 _1_inlined4 _2 _2_inlined1 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _endpos__1_inlined4_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0550 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ARROW as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0517 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.OTHERWISE ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0550 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState0552 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            let MenhirCell1_ARROW (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2, _, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _4, _1_inlined2, _endpos__1_inlined3_, _1_inlined3) = ((), (), (), _endpos, _v) in
            let _v = _menhir_action_205 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ in
            _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0549 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_ASSERT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_307 _1 _endpos__2_ _startpos__1_ e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0548 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell0_REPEAT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3, _5) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_340 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0544 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState0545 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0542 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_lexpr, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_lexpr (_menhir_stack, _menhir_s, le, _startpos_le_) = _menhir_stack in
        let (_endpos_xs_, xs, _2_inlined1) = (_endpos, _v, ()) in
        let _v = _menhir_action_310 _2_inlined1 _endpos__2_ _startpos_le_ e le xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0537 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
        let MenhirCell0_WHILE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3, _5) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_300 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0532 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
        | _ ->
            _eRR ()
    
    and _menhir_run_0770 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0771
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0770, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0771
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined1) = ((), _endpos_0, _v, ()) in
                let _v = _menhir_action_272 _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0770
        | _ ->
            _eRR ()
    
    and _menhir_run_0506 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_CONSTANT as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0507
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0507
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0507
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0507
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0507
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0507
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0507
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0507
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0509
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0509
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0509
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.UNKNOWN ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0511
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0511
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.WHILE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.VAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.UNREACHABLE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.TRY ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.THROW ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.RETURN ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.REPEAT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.PRINT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.PRAGMA ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.PASS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.MINUS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.LPAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.LET ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.IF ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.IDENTIFIER _v_12 ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0513
                                    | Tokens.FOR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.DEBUG ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.CONSTANT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.CASE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.ASSERT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.ARROBASE_LOOPLIMIT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0511, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0513
                                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                                        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                                        let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, lhs, _endpos__3_inlined2_) = ((), _endpos_11, (), (), (), _startpos, _v_3, _v, _endpos_10) in
                                        let _v = _menhir_action_268 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0511
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0511
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0511
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0511
                            | Tokens.INT_LIT _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0511
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0511
                            | Tokens.IDENTIFIER _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0511
                            | Tokens.BOOL_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0511
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0511
                            | Tokens.BITVECTOR_LIT _v_17 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0509, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17 MenhirState0511
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_18 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_068 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0575 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_18 MenhirState0507 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0509
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0509
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0509
                    | Tokens.INT_LIT _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState0509
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0509
                    | Tokens.IDENTIFIER _v_20 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState0509
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0509) in
                        let _menhir_s = MenhirState0569 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0509
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0509
                    | Tokens.BITVECTOR_LIT _v_28 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0507, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0509
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_29 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_021 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0575 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_29 MenhirState0507 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState0507
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0507
            | Tokens.BITVECTOR_LIT _v_31 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState0507
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0575 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0576
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0575, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0576
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, x, _3) = ((), _endpos_0, _v, ()) in
                let _v = _menhir_action_254 _1 _3 _endpos__2_ _startpos__1_ decl_item x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0575
        | _ ->
            _eRR ()
    
    and _menhir_run_0477 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LET as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0478
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0478
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0478
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0478
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0478
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0478
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0478
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0478
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0480
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0480
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0480
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.UNKNOWN ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0482
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.WHILE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.VAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.UNREACHABLE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.TRY ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.THROW ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.RETURN ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.REPEAT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.PRINT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.PRAGMA ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.PASS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.MINUS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.LPAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.LET ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.IF ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.IDENTIFIER _v_12 ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0484
                                    | Tokens.FOR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.DEBUG ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.CONSTANT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.CASE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.ASSERT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.ARROBASE_LOOPLIMIT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0482, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0484
                                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                                        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                                        let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, lhs, _endpos__3_inlined2_) = ((), _endpos_11, (), (), (), _startpos, _v_3, _v, _endpos_10) in
                                        let _v = _menhir_action_265 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0482
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
                            | Tokens.INT_LIT _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0482
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
                            | Tokens.IDENTIFIER _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0482
                            | Tokens.BOOL_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0482
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
                            | Tokens.BITVECTOR_LIT _v_17 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0480, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17 MenhirState0482
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_18 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_068 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0692 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_18 MenhirState0478 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0480
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0480
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0480
                    | Tokens.INT_LIT _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState0480
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0480
                    | Tokens.IDENTIFIER _v_20 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState0480
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0480) in
                        let _menhir_s = MenhirState0686 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0480
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0480
                    | Tokens.BITVECTOR_LIT _v_28 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0478, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0480
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_29 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_021 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0692 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_29 MenhirState0478 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState0478
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0478
            | Tokens.BITVECTOR_LIT _v_31 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState0478
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0692 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0693
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0692, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0693
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, x, _3) = ((), _endpos_0, _v, ()) in
                let _v = _menhir_action_253 _1 _3 _endpos__2_ _startpos__1_ decl_item x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | _ ->
            _eRR ()
    
    and _menhir_run_0425 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
            let _menhir_s = MenhirState0426 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let x = _v in
            let _v = _menhir_action_418 x in
            _menhir_goto_separated_nonempty_list_COMMA_decl_item_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_decl_item_ : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0426 ->
            _menhir_run_0427 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0418 ->
            _menhir_run_0419 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0427 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_decl_item -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_decl_item (_menhir_stack, _menhir_s, x) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_419 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_decl_item_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0419 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _menhir_s, _v) in
            let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0421 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _menhir_s, _v) in
            let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0423 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR | Tokens.SEMI_COLON ->
            let MenhirCell1_decl_item (_menhir_stack, _, x) = _menhir_stack in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_2, li) = ((), _v) in
            let _v = _menhir_action_012 _2 li x in
            _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0417 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_s = MenhirState0418 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0324 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_IN _menhir_cell0_BNOT _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell0_BNOT (_menhir_stack, _startpos__1_) = _menhir_stack in
            let MenhirCell1_IN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_1, _2, _1_inlined2, _endpos__3_, _endpos__1_inlined2_, _startpos__1_inlined2_) = ((), (), _v, _endpos_0, _endpos, _startpos) in
            let _v = _menhir_action_062 _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0320 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_IN _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_IN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_2, _endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1, _endpos__3_) = ((), _endpos, _startpos, _v, _endpos_0) in
            let _v = _menhir_action_061 _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0316 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_pattern -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_pattern (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_2, _endpos__3_, li) = ((), _endpos_0, _v) in
            let _v = _menhir_action_409 _2 _endpos__3_ _startpos__1_ li x in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0312 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1, _endpos__3_) = (_endpos, _startpos, _v, _endpos_0) in
            let _v = _menhir_action_411 _1_inlined1 _endpos__1_inlined1_ _startpos__1_inlined1_ in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0271 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_BNOT _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell0_BNOT (_menhir_stack, _startpos__1_) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_1, _2, _1_inlined2, _endpos__3_, _endpos__1_inlined2_, _startpos__1_inlined2_) = ((), (), _v, _endpos_0, _endpos, _startpos) in
            let _v = _menhir_action_117 _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0267 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_2, _endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1, _endpos__3_) = ((), _endpos, _startpos, _v, _endpos_0) in
            let _v = _menhir_action_116 _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0230 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_pattern -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_pattern (_menhir_stack, _menhir_s, x, _startpos_x_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_437 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_pattern_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _startpos_x_ _v _menhir_s _tok
    
    and _menhir_run_0226 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BNOT, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LBRACE (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _1_inlined2, _endpos__3_, _endpos__1_inlined2_, _startpos__1_inlined2_) = ((), _v, _endpos_0, _endpos, _startpos) in
            let _v = _menhir_action_412 _1 _1_inlined2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0273 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0273 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0275 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0275 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0277 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell0_GEQ (_menhir_stack, _startpos) in
        let _menhir_s = MenhirState0277 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0279 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_118 _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_sub_expr_ _v _menhir_s _tok
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | _ ->
            _eRR ()
    
    and _menhir_run_0282 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0282 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0284 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACKET ->
            let _menhir_s = MenhirState0285 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _endpos_x_, x) = ((), _endpos, _v) in
            let _v = _menhir_action_111 _2 _endpos_x_ _startpos_e_ e x in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0289 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0289 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0291 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0291 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0293 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0293 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0295 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0295 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0297 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0297 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0299 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0299 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0301 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0301 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0303 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.REAL ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.LPAR ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.RBRACE ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
                let (_startpos__1_, _2, _endpos__3_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_114 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr in
                _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | _ ->
                _eRR ())
        | Tokens.INTEGER ->
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0303
        | Tokens.BOOLEAN ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.BITS ->
            _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.BIT ->
            _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.ARRAY ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | _ ->
            _eRR ()
    
    and _menhir_run_0309 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0309 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0231 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SLICING ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0232 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0234 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0236 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0238 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0240 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0242 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0244 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0246 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0249 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0251 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0253 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0255 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0257 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0260 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0262 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0265 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0273 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0275 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0277 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0279 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0282 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0284 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0289 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0291 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0293 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0295 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0297 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0299 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0301 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0303 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0309 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let (_endpos_expr_pattern_, _startpos_expr_pattern_, expr_pattern) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_404 _endpos_expr_pattern_ _startpos_expr_pattern_ expr_pattern in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_pattern_ _startpos_expr_pattern_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_1365 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
                let MenhirCell0_REPEAT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, _3, _5, cond) = ((), (), _endpos_0, (), (), _v) in
                let _v = _menhir_action_395 _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1365
        | _ ->
            _eRR ()
    
    and _menhir_run_1357 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1357) in
            let _menhir_s = MenhirState1358 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1357
        | _ ->
            _eRR ()
    
    and _menhir_run_1354 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.RPAR ->
            let _menhir_s = MenhirState1354 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_WHILE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState1356 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.REPEAT ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_REPEAT (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState1362 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1354
        | _ ->
            _eRR ()
    
    and _menhir_run_1349 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_ASSERT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_ASSERT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, e) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_362 _1 _2 _endpos__2_inlined1_ _startpos__1_ e in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1349
        | _ ->
            _eRR ()
    
    and _menhir_run_1343 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_CASE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.OF ->
            let _menhir_stack = MenhirCell1_OF (_menhir_stack, MenhirState1343) in
            let _menhir_s = MenhirState1344 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0517 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
        | _ ->
            _eRR ()
    
    and _menhir_run_1301 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1301) in
            let _menhir_s = MenhirState1302 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | _ ->
            _eRR ()
    
    and _menhir_run_1299 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState1299) in
            let _menhir_s = MenhirState1300 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1299) in
            let _menhir_s = MenhirState1306 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | _ ->
            _eRR ()
    
    and _menhir_run_1289 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1289) in
            let _menhir_s = MenhirState1290 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | _ ->
            _eRR ()
    
    and _menhir_run_1287 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState1287) in
            let _menhir_s = MenhirState1288 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1287) in
            let _menhir_s = MenhirState1294 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | _ ->
            _eRR ()
    
    and _menhir_run_1285 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.TO ->
            let _menhir_stack = MenhirCell1_TO (_menhir_stack, MenhirState1285) in
            let _menhir_s = MenhirState1286 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.DOWNTO ->
            let _menhir_stack = MenhirCell1_DOWNTO (_menhir_stack, MenhirState1285) in
            let _menhir_s = MenhirState1298 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1285
        | _ ->
            _eRR ()
    
    and _menhir_run_1279 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _4, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), (), _endpos_0, _v) in
                let _v = _menhir_action_371 _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args fld name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1279
        | _ ->
            _eRR ()
    
    and _menhir_run_1274 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _4, _2_inlined1, _2_inlined2, rhs, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_374 _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos_args_ _startpos_name_ args name rhs x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
        | _ ->
            _eRR ()
    
    and _menhir_run_1264 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_368 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1264
        | _ ->
            _eRR ()
    
    and _menhir_run_1254 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _4, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), (), _endpos_0, _v) in
                let _v = _menhir_action_370 _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1254
        | _ ->
            _eRR ()
    
    and _menhir_run_1249 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _4, _2_inlined1, _2_inlined2, rhs, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_373 _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1249
        | _ ->
            _eRR ()
    
    and _menhir_run_1239 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_367 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1239
        | _ ->
            _eRR ()
    
    and _menhir_run_1231 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _4, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), (), _endpos_0, _v) in
                let _v = _menhir_action_369 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | _ ->
            _eRR ()
    
    and _menhir_run_1226 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _4, _2_inlined1, _2_inlined2, rhs, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_372 _1_inlined2 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1226
        | _ ->
            _eRR ()
    
    and _menhir_run_1216 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_366 _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1216
        | _ ->
            _eRR ()
    
    and _menhir_run_1204 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState1204) in
            let _menhir_s = MenhirState1205 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1204
        | _ ->
            _eRR ()
    
    and _menhir_run_1158 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined2, _3, _endpos__2_inlined2_) = ((), (), _v, (), (), _endpos_0) in
                let _v = _menhir_action_394 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ expr in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1158
        | _ ->
            _eRR ()
    
    and _menhir_run_1154 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, _3, expr) = ((), (), _endpos_0, (), _v) in
                let _v = _menhir_action_393 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState1154) in
            let _menhir_s = MenhirState1157 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1154
        | _ ->
            _eRR ()
    
    and _menhir_run_1148 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_RETURN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_RETURN (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, x, _endpos__2_inlined1_) = ((), (), _v, _endpos_0) in
                let _v = _menhir_action_358 _1 _2 _endpos__2_inlined1_ _startpos__1_ x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | _ ->
            _eRR ()
    
    and _menhir_run_1142 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_THROW as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_THROW (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, e) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_396 _1 _2 _endpos__2_inlined1_ _startpos__1_ e in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | _ ->
            _eRR ()
    
    and _menhir_run_1078 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1078) in
            let _menhir_s = MenhirState1079 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1078
        | _ ->
            _eRR ()
    
    and _menhir_run_1076 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState1076) in
            let _menhir_s = MenhirState1077 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1076) in
            let _menhir_s = MenhirState1083 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1076
        | _ ->
            _eRR ()
    
    and _menhir_run_1068 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1068, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1069
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1070 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1068
        | _ ->
            _eRR ()
    
    and _menhir_run_1060 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1060, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1061
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1062 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
        | _ ->
            _eRR ()
    
    and _menhir_run_1052 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1052, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1053
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1054 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1052
        | _ ->
            _eRR ()
    
    and _menhir_run_1047 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1047, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1048
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1049 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1047
        | _ ->
            _eRR ()
    
    and _menhir_run_1039 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1039, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1040
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1041 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1039
        | _ ->
            _eRR ()
    
    and _menhir_run_0970 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0970, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0971
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0972 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
        | _ ->
            _eRR ()
    
    and _menhir_run_0964 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0964, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0965
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0966 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0964
        | _ ->
            _eRR ()
    
    and _menhir_run_0958 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0958, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0959
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0960 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | _ ->
            _eRR ()
    
    and _menhir_run_0953 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0953, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0955 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0953
        | _ ->
            _eRR ()
    
    and _menhir_run_0947 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0947, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0949 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0947
        | _ ->
            _eRR ()
    
    and _menhir_run_0917 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.BEGIN ->
            let _menhir_s = MenhirState0917 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_BEGIN (_menhir_stack, _menhir_s, _endpos) in
            let _menhir_s = MenhirState0918 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0917
        | _ ->
            _eRR ()
    
    and _menhir_run_0884 : type  ttv_stack. (((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.BEGIN ->
            let _menhir_s = MenhirState0884 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_BEGIN (_menhir_stack, _menhir_s, _endpos) in
            let _menhir_s = MenhirState0885 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0884
        | _ ->
            _eRR ()
    
    and _menhir_run_0875 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0875, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0876
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0877 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | _ ->
            _eRR ()
    
    and _menhir_run_0869 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0869, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0871 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0869
        | _ ->
            _eRR ()
    
    and _menhir_run_0863 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0863, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0864
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0865 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | _ ->
            _eRR ()
    
    and _menhir_run_0858 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0858, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0860 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0858
        | _ ->
            _eRR ()
    
    and _menhir_run_0852 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0852, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0853
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0854 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0852
        | _ ->
            _eRR ()
    
    and _menhir_run_0845 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0845, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0846
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0847 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
        | _ ->
            _eRR ()
    
    and _menhir_run_0839 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0839, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0840
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0841 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0839
        | _ ->
            _eRR ()
    
    and _menhir_run_0833 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0833, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0834
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0835 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0833
        | _ ->
            _eRR ()
    
    and _menhir_run_0828 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0828, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0829
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0830 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0828
        | _ ->
            _eRR ()
    
    and _menhir_run_0822 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_ast) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0822, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0823
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0824 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0822
        | _ ->
            _eRR ()
    
    and _menhir_run_0816 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0816, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0818 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0816
        | _ ->
            _eRR ()
    
    and _menhir_run_0811 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0811, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0812
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0848 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0811
        | _ ->
            _eRR ()
    
    and _menhir_run_0788 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0788, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0789
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0943 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0788
        | _ ->
            _eRR ()
    
    and _menhir_run_0719 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_THROW as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0720
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0719, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0720
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, e) = ((), _endpos_0, _v) in
                let _v = _menhir_action_286 _1 _endpos__2_ _startpos__1_ e in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0719
        | _ ->
            _eRR ()
    
    and _menhir_run_0715 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_RETURN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0716
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0715, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0716
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, x) = ((), _endpos_0, _v) in
                let _v = _menhir_action_248 _1 _endpos__2_ _startpos__1_ x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
        | _ ->
            _eRR ()
    
    and _menhir_run_0711 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0712
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0711, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0712
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3) = ((), _endpos_0, _v, (), ()) in
                let _v = _menhir_action_284 _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
        | _ ->
            _eRR ()
    
    and _menhir_run_0707 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0708
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0707, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0708
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _3, expr) = ((), _endpos_0, (), _v) in
                let _v = _menhir_action_283 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0707) in
            let _menhir_s = MenhirState0710 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0707
        | _ ->
            _eRR ()
    
    and _menhir_run_0672 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSIF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0672) in
            let _menhir_s = MenhirState0673 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0672
        | _ ->
            _eRR ()
    
    and _menhir_run_0667 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0668
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0667, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0668
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _4, _2_inlined1, rhs) = (_endpos_0, (), (), _v) in
                let _v = _menhir_action_261 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
        | _ ->
            _eRR ()
    
    and _menhir_run_0662 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0663
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0662, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0663
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _4, _2_inlined1, _2_inlined2, rhs) = (_endpos_0, (), (), (), _v) in
                let _v = _menhir_action_264 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
        | _ ->
            _eRR ()
    
    and _menhir_run_0652 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0653
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0652, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0653
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, rhs) = (_endpos_0, (), _v) in
                let _v = _menhir_action_258 _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
        | _ ->
            _eRR ()
    
    and _menhir_run_0642 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0643
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0642, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _4, _2_inlined1, rhs) = (_endpos_0, (), (), _v) in
                let _v = _menhir_action_260 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0642
        | _ ->
            _eRR ()
    
    and _menhir_run_0637 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0638
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0637, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _4, _2_inlined1, _2_inlined2, rhs) = (_endpos_0, (), (), (), _v) in
                let _v = _menhir_action_263 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | _ ->
            _eRR ()
    
    and _menhir_run_0627 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0628
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0627, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0628
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, rhs) = (_endpos_0, (), _v) in
                let _v = _menhir_action_257 _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
        | _ ->
            _eRR ()
    
    and _menhir_run_0619 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0620
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0619, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0620
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _4, _2_inlined1, rhs) = (_endpos_0, (), (), _v) in
                let _v = _menhir_action_259 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0619
        | _ ->
            _eRR ()
    
    and _menhir_run_0614 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0615
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0614, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0615
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _4, _2_inlined1, _2_inlined2, rhs) = (_endpos_0, (), (), (), _v) in
                let _v = _menhir_action_262 _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0614
        | _ ->
            _eRR ()
    
    and _menhir_run_0604 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0605
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0604, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0605
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, rhs) = (_endpos_0, (), _v) in
                let _v = _menhir_action_256 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0604
        | _ ->
            _eRR ()
    
    and _menhir_run_0593 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0593) in
            let _menhir_s = MenhirState0594 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0593
        | _ ->
            _eRR ()
    
    and _menhir_run_0591 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0591) in
            let _menhir_s = MenhirState0592 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0591) in
            let _menhir_s = MenhirState0598 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0591
        | _ ->
            _eRR ()
    
    and _menhir_run_0546 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0547
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0546, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
                let MenhirCell0_REPEAT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _3, _5, cond) = ((), _endpos_0, (), (), _v) in
                let _v = _menhir_action_285 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | _ ->
            _eRR ()
    
    and _menhir_run_0540 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0541
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0540, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0541
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_lexpr (_menhir_stack, _menhir_s, le, _startpos_le_) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, e) = (_endpos_0, (), _v) in
                let _v = _menhir_action_255 _2_inlined1 _endpos__2_ _startpos_le_ e le in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
        | _ ->
            _eRR ()
    
    and _menhir_run_0530 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0530) in
            let _menhir_s = MenhirState0531 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
        | _ ->
            _eRR ()
    
    and _menhir_run_0527 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.RPAR ->
            let _menhir_s = MenhirState0527 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_WHILE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0529 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.REPEAT ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_REPEAT (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0543 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0527
        | _ ->
            _eRR ()
    
    and _menhir_run_0523 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ASSERT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0524
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0523, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0524
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, e) = ((), _endpos_0, _v) in
                let _v = _menhir_action_252 _1 _endpos__2_ _startpos__1_ e in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0523
        | _ ->
            _eRR ()
    
    and _menhir_run_0520 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | Tokens.ARROW ->
            let _menhir_stack = MenhirCell1_ARROW (_menhir_stack, MenhirState0520) in
            let _menhir_s = MenhirState0521 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | _ ->
            _eRR ()
    
    and _menhir_run_0515 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_CASE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.OF ->
            let _menhir_stack = MenhirCell1_OF (_menhir_stack, MenhirState0515) in
            let _menhir_s = MenhirState0516 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0517 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | _ ->
            _eRR ()
    
    and _menhir_run_0499 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0499) in
            let _menhir_s = MenhirState0500 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | _ ->
            _eRR ()
    
    and _menhir_run_0497 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0497) in
            let _menhir_s = MenhirState0498 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0497) in
            let _menhir_s = MenhirState0586 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0497
        | _ ->
            _eRR ()
    
    and _menhir_run_0495 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.TO ->
            let _menhir_stack = MenhirCell1_TO (_menhir_stack, MenhirState0495) in
            let _menhir_s = MenhirState0496 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.DOWNTO ->
            let _menhir_stack = MenhirCell1_DOWNTO (_menhir_stack, MenhirState0495) in
            let _menhir_s = MenhirState0590 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | _ ->
            _eRR ()
    
    and _menhir_run_0486 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0486) in
            let _menhir_s = MenhirState0487 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | _ ->
            _eRR ()
    
    and _menhir_run_0458 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr_atom_desc as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.RRBRACKET ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
            let (_2, _4, expr, _endpos__4_) = ((), (), _v, _endpos_0) in
            let _v = _menhir_action_130 _2 _4 _endpos_desc_ _startpos_desc_ desc expr in
            _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__4_ _startpos_desc_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | _ ->
            _eRR ()
    
    and _menhir_run_0403 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0403) in
            let _menhir_s = MenhirState0404 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0403
        | _ ->
            _eRR ()
    
    and _menhir_run_0401 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0401) in
            let _menhir_s = MenhirState0402 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0401) in
            let _menhir_s = MenhirState0776 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
        | _ ->
            _eRR ()
    
    and _menhir_run_0354 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0354, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1033 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | _ ->
            _eRR ()
    
    and _menhir_run_0343 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0343
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0343
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0343
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0343
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_049 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0342 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_048 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0337 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (x, _endpos__3_) = (_v, _endpos_0) in
            let _v = _menhir_action_067 _endpos__3_ _startpos__1_ x in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0337) in
            let _menhir_s = MenhirState0339 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0337
        | _ ->
            _eRR ()
    
    and _menhir_run_0335 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.ELSIF ->
            _menhir_run_0180 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.ELSE ->
            _menhir_run_0184 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0335
        | _ ->
            _eRR ()
    
    and _menhir_run_0180 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0180 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0184 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_ELSE (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0184 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0333 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0333) in
            let _menhir_s = MenhirState0334 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0333
        | _ ->
            _eRR ()
    
    and _menhir_run_0311 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BNOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_102 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0310 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0310
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_077 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0302 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_078 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0300 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_080 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0298 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_079 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0296 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_100 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0294 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_101 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0292 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_081 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0290 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_082 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0283 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_083 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0281 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_084 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0278 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_GEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell0_GEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_087 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0276 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_086 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0274 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_088 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0261 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_LEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell0_LEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_090 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0258 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.RRBRACKET ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_2, _4, e2, _endpos__4_) = ((), (), _v, _endpos_0) in
            let _v = _menhir_action_110 _2 _4 _endpos__4_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__4_ _startpos_e1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | _ ->
            _eRR ()
    
    and _menhir_run_0256 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_089 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0254 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_092 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0252 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_093 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0250 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_094 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0248 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_085 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0245 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_095 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0243 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_091 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0241 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_099 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0239 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_096 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0237 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_097 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0235 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_098 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0233 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_2, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_405 _2 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0221 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_GEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0221
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let MenhirCell1_GEQ (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_expr_, expr) = ((), _endpos, _v) in
            let _v = _menhir_action_408 _1 _endpos_expr_ _startpos__1_ expr in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0211 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0211) in
            let _menhir_s = MenhirState0212 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    _menhir_run_0210 _menhir_stack _menhir_lexbuf _menhir_lexer
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0211
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let y = _v in
            let _v = _menhir_action_422 x y in
            _menhir_goto_separated_nonempty_list_COMMA_field_assign_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_field_assign_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0196 ->
            _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0212 ->
            _menhir_run_0214 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1323 ->
            _menhir_run_0201 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1184 ->
            _menhir_run_0201 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1104 ->
            _menhir_run_0201 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0754 ->
            _menhir_run_0201 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0480 ->
            _menhir_run_0201 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0509 ->
            _menhir_run_0201 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0199 ->
            _menhir_run_0201 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0215 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_124 _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
    
    and _menhir_run_0214 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, y, _, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_423 x xs y in
        _menhir_goto_separated_nonempty_list_COMMA_field_assign_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0201 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_069 _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
    
    and _menhir_run_0193 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0193) in
            let _menhir_s = MenhirState0194 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0193
        | Tokens.RBRACE | Tokens.RPAR | Tokens.SEMI_COLON ->
            let (_endpos_x_, x) = (_endpos, _v) in
            let _v = _menhir_action_420 x in
            _menhir_goto_separated_nonempty_list_COMMA_expr_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_expr_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1333 ->
            _menhir_run_1334 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1325 ->
            _menhir_run_1329 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1311 ->
            _menhir_run_1315 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1257 ->
            _menhir_run_1258 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1211 ->
            _menhir_run_1234 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1194 ->
            _menhir_run_1195 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1186 ->
            _menhir_run_1190 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1171 ->
            _menhir_run_1174 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState1162 ->
            _menhir_run_1166 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1114 ->
            _menhir_run_1115 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1106 ->
            _menhir_run_1110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0783 ->
            _menhir_run_0974 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0764 ->
            _menhir_run_0765 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0756 ->
            _menhir_run_0760 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0445 ->
            _menhir_run_0701 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0449 ->
            _menhir_run_0697 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0686 ->
            _menhir_run_0687 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0482 ->
            _menhir_run_0682 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0645 ->
            _menhir_run_0646 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0489 ->
            _menhir_run_0622 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0502 ->
            _menhir_run_0579 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0569 ->
            _menhir_run_0570 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0511 ->
            _menhir_run_0565 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0339 ->
            _menhir_run_0340 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0021 ->
            _menhir_run_0331 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0196 ->
            _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0205 ->
            _menhir_run_0207 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1323 ->
            _menhir_run_0203 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1184 ->
            _menhir_run_0203 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1104 ->
            _menhir_run_0203 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0754 ->
            _menhir_run_0203 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0480 ->
            _menhir_run_0203 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0509 ->
            _menhir_run_0203 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0199 ->
            _menhir_run_0203 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0194 ->
            _menhir_run_0195 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0189 ->
            _menhir_run_0191 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1334 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1335
            | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_1336 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0205 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0205
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0205
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let _endpos__3_ = _endpos in
            let _v = _menhir_action_202 () in
            _menhir_goto_loption_plist_expr__ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0205
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0205
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0205
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0205
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0205
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0205
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0205
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0205
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0205
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0205
        | _ ->
            _eRR ()
    
    and _menhir_goto_loption_plist_expr__ : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1335 ->
            _menhir_run_1336 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState1259 ->
            _menhir_run_1260 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1196 ->
            _menhir_run_1197 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState1116 ->
            _menhir_run_1117 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0766 ->
            _menhir_run_0767 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0688 ->
            _menhir_run_0689 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0647 ->
            _menhir_run_0648 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0571 ->
            _menhir_run_0572 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0218 ->
            _menhir_run_0219 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0204 ->
            _menhir_run_0209 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1260 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _endpos__2_inlined1_, args) = ((), _endpos_0, _v) in
                let _v = _menhir_action_361 _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ args name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState1263 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.COMMA ->
                        let _menhir_s = MenhirState1270 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _menhir_s = MenhirState1278 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0767 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0768
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0768
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _5, _endpos_args_, args) = ((), _endpos_0, (), (), (), (), _endpos, _v) in
                let _v = _menhir_action_277 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0689 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0690
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0690
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _5, _endpos_args_, args) = ((), _endpos_0, (), (), (), (), _endpos, _v) in
                let _v = _menhir_action_267 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0648 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0649
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, args) = (_endpos_0, _v) in
                let _v = _menhir_action_251 _1_inlined1 _endpos__2_ _startpos_name_ args name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0651 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.COMMA ->
                        let _menhir_s = MenhirState0658 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _menhir_s = MenhirState0666 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0572 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0573
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _5, _endpos_args_, args) = ((), _endpos_0, (), (), (), (), _endpos, _v) in
                let _v = _menhir_action_270 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0219 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_args_, args) = (_endpos, _v) in
        let _v = _menhir_action_108 _1_inlined1 _endpos_args_ _startpos_name_ args name in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_args_ _startpos_name_ _v _menhir_s _tok
    
    and _menhir_run_0209 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_args_, args) = (_endpos, _v) in
        let _v = _menhir_action_053 _1_inlined1 _endpos_args_ _startpos_name_ args name in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_args_ _startpos_name_ _v _menhir_s _tok
    
    and _menhir_run_1258 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1259
            | Tokens.DOT | Tokens.EQ | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_1260 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 MenhirState1259 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1234 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let (_2, _endpos__2_inlined1_, _1_inlined1) = ((), _endpos_1, _v) in
                    let _v = _menhir_action_360 _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | Tokens.EQ ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                let _menhir_s = MenhirState1238 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.DOT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.COMMA ->
                            let _menhir_s = MenhirState1245 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.EQ ->
                        let _menhir_s = MenhirState1253 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1195 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
            | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_1197 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1115 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1116
            | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_1117 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0974 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0975
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_0976 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0765 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0766
            | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_0767 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 MenhirState0766 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0760 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.IDENTIFIER _v_2 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0762
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0762
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__3_inlined2_) = ((), _endpos_1, (), _v, (), (), _endpos_0) in
                    let _v = _menhir_action_276 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0701 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRINT _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.IDENTIFIER _v_2 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0703
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0703
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _1_inlined2) = ((), _endpos_1, _v) in
                    let _v = _menhir_action_279 _1 _1_inlined2 _endpos__2_ _startpos__1_ in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0697 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0698
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _) = _menhir_stack in
                let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _endpos__1_inlined1_, _1_inlined1) = ((), _endpos_0, _endpos, _v) in
                let _v = _menhir_action_289 _1 _1_inlined1 _2_inlined1 _endpos__1_inlined1_ _endpos__2_ _startpos__1_ in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0687 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0688
            | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_0689 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 MenhirState0688 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0682 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.IDENTIFIER _v_2 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0684
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0684
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__3_inlined2_) = ((), _endpos_1, (), _v, (), (), _endpos_0) in
                    let _v = _menhir_action_266 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0646 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0647
            | Tokens.DOT | Tokens.EQ | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_0648 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 MenhirState0647 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0622 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.IDENTIFIER _v_2 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0624
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0624
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                    let (_endpos__2_, _1_inlined1) = (_endpos_1, _v) in
                    let _v = _menhir_action_250 _1_inlined1 _endpos__2_ _startpos_name_ name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | Tokens.EQ ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                let _menhir_s = MenhirState0626 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.DOT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.COMMA ->
                            let _menhir_s = MenhirState0633 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.EQ ->
                        let _menhir_s = MenhirState0641 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0579 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_DEBUG _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.IDENTIFIER _v_2 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0581
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0581
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _1_inlined2) = ((), _endpos_1, _v) in
                    let _v = _menhir_action_281 _1 _1_inlined2 _endpos__2_ _startpos__1_ in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0570 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0571
            | Tokens.SEMI_COLON ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_0572 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 MenhirState0571 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0565 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.IDENTIFIER _v_2 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0567
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos_0) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                    _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0567
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__3_inlined2_) = ((), _endpos_1, (), _v, (), (), _endpos_0) in
                    let _v = _menhir_action_269 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0340 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_2, _endpos__3_, li) = ((), _endpos_0, _v) in
            let _v = _menhir_action_070 _2 _endpos__3_ _startpos__1_ li x in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0331 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
            let (_1_inlined1, _endpos__3_) = (_v, _endpos_0) in
            let _v = _menhir_action_052 _1_inlined1 _endpos__3_ _startpos_name_ name in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0217 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0218
            | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_0219 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0207 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_LPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_1_inlined1, _endpos__3_) = (_v, _endpos_0) in
            let _v = _menhir_action_203 _1_inlined1 in
            _menhir_goto_loption_plist_expr__ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0203 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0204
            | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
                let _v_1 = _menhir_action_201 () in
                _menhir_run_0209 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_0 _v_1 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0195 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_421 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_expr_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0191 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
            let (_1_inlined1, _endpos__3_) = (_v, _endpos_0) in
            let _v = _menhir_action_107 _1_inlined1 _endpos__3_ _startpos_name_ name in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0185 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0185
        | Tokens.ARROW | Tokens.BEGIN | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_ELSE (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_1, _endpos__2_, _2) = ((), _endpos, _v) in
            let _v = _menhir_action_013 _1 _2 in
            _menhir_goto_e_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_e_else : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0335 ->
            _menhir_run_0336 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0179 ->
            _menhir_run_0187 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0183 ->
            _menhir_run_0186 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0336 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e1, _, _) = _menhir_stack in
        let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_e_else_, e_else) = ((), (), _endpos, _v) in
        let _v = _menhir_action_050 _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_else_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0187 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e1, _, _) = _menhir_stack in
        let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_e_else_, e_else) = ((), (), _endpos, _v) in
        let _v = _menhir_action_105 _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_else_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0186 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, c, _, _) = _menhir_stack in
        let MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_e_else_, e_else) = ((), (), _endpos, _v) in
        let _v = _menhir_action_014 _1 _3 _endpos_e_else_ _startpos__1_ c e e_else in
        _menhir_goto_e_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_else_ _v _menhir_s _tok
    
    and _menhir_run_0183 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.ELSIF ->
            _menhir_run_0180 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.ELSE ->
            _menhir_run_0184 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0183
        | _ ->
            _eRR ()
    
    and _menhir_run_0181 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSIF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0181) in
            let _menhir_s = MenhirState0182 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0181
        | _ ->
            _eRR ()
    
    and _menhir_run_0179 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.ELSIF ->
            _menhir_run_0180 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.ELSE ->
            _menhir_run_0184 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0179
        | _ ->
            _eRR ()
    
    and _menhir_run_0177 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0177) in
            let _menhir_s = MenhirState0178 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0177
        | _ ->
            _eRR ()
    
    and _menhir_run_0173 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0173
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let MenhirCell1_LEQ (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_expr_, expr) = ((), _endpos, _v) in
            let _v = _menhir_action_407 _1 _endpos_expr_ _startpos__1_ expr in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0169 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0169
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0169
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0169
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0169
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_103 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0167 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0167
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0167
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0167
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0167
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_104 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0155 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0155) in
            let _menhir_s = MenhirState0156 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0155
        | Tokens.RBRACKET ->
            let MenhirCell1_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_439 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_slice_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0460 ->
            _menhir_run_0461 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0033 ->
            _menhir_run_0328 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0326 ->
            _menhir_run_0327 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0262 ->
            _menhir_run_0263 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0156 ->
            _menhir_run_0157 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0152 ->
            _menhir_run_0153 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0150 ->
            _menhir_run_0151 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0146 ->
            _menhir_run_0147 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0049 ->
            _menhir_run_0050 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0461 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_lexpr_atom_desc -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_129 _1_inlined1 _endpos_desc_ _startpos_desc_ desc in
        _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
    
    and _menhir_run_0328 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LBRACKET -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_054 _1_inlined1 _endpos__3_ _startpos_e_ e in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
    
    and _menhir_run_0327 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_COLON (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_446 _1 e xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0263 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_109 _1_inlined1 _endpos__3_ _startpos_e_ e in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
    
    and _menhir_run_0157 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_COLON (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (xs, _2_inlined1) = (_v, ()) in
        let _v = _menhir_action_444 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0153 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, expr, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_443 expr xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0151 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_PLUS_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_PLUS_COLON (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (xs, _2_inlined1) = (_v, ()) in
        let _v = _menhir_action_445 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0147 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_STAR_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_STAR_COLON (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (xs, _2_inlined1) = (_v, ()) in
        let _v = _menhir_action_447 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0050 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRACKET as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v_0 ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                let _startpos_2 = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_2) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    _menhir_run_0049 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0053
                | Tokens.RBRACE ->
                    let _v_3 = _menhir_action_197 () in
                    _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0053
                | _ ->
                    _eRR ())
            | Tokens.COMMA ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                    _menhir_run_0049 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0059
                | Tokens.RBRACE ->
                    let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                    let (x, _1_inlined1, x_inlined1) = (_v_0, _v, ()) in
                    let _v = _menhir_action_214 _1_inlined1 x x_inlined1 in
                    _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                let _menhir_s = MenhirState0061 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                let _menhir_s = MenhirState0139 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.RBRACE ->
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (x, _1_inlined1) = (_v_0, _v) in
                let _v = _menhir_action_213 _1_inlined1 x in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0049 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0049 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNKNOWN ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.STRING_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.COLON ->
            _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0055 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _menhir_stack = MenhirCell1_loption_non_empty_trailing_list_COMMA_bitfield__ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
                _menhir_run_0049 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0057
            | Tokens.RBRACE ->
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (_1_inlined3, x_inlined1) = (_v, ()) in
                let _v = _menhir_action_216 _1_inlined1 _1_inlined3 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
            let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
            let _1_inlined3 = _v in
            let _v = _menhir_action_215 _1_inlined1 _1_inlined3 x in
            _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0141 ->
            _menhir_run_0142 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0137 ->
            _menhir_run_0138 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0059 ->
            _menhir_run_0060 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0057 ->
            _menhir_run_0058 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0048 ->
            _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState0053 ->
            _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0142 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_ty -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, _1_inlined2, t) = ((), (), _v) in
        let _v = _menhir_action_223 _1_inlined1 _1_inlined2 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0138 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_ty -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, _1_inlined2, t) = ((), (), _v) in
        let _v = _menhir_action_224 _1_inlined1 _1_inlined2 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0060 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, t) = ((), _v) in
        let _v = _menhir_action_221 _1_inlined1 _2 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0058 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_loption_non_empty_trailing_list_COMMA_bitfield__ _menhir_cell0_RBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_non_empty_trailing_list_COMMA_bitfield__ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, t) = ((), _v) in
        let _v = _menhir_action_222 _1_inlined1 _1_inlined3 _2 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0054 : type  ttv_stack ttv_result. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let x = _v in
        let _v = _menhir_action_198 x in
        _menhir_goto_loption_non_empty_trailing_list_COMMA_bitfield__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_goto_loption_non_empty_trailing_list_COMMA_bitfield__ : type  ttv_stack ttv_result. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0048 ->
            _menhir_run_0158 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0053 ->
            _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0158 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_BITS _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_194 _1_inlined1 in
        _menhir_goto_loption_bitfields_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _tok
    
    and _menhir_goto_loption_bitfields_ : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_BITS _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_BITS (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__1_inlined2_) = ((), _v, _endpos) in
        let _v = _menhir_action_462 _1 _1_inlined2 _endpos__1_inlined2_ _startpos__1_ x in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_inlined2_ _v _menhir_s _tok
    
    and _menhir_run_0149 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_PLUS_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0149) in
            let _menhir_s = MenhirState0150 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0149
        | Tokens.RBRACKET ->
            let MenhirCell1_PLUS_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_440 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0145 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_STAR_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0145) in
            let _menhir_s = MenhirState0146 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0145
        | Tokens.RBRACKET ->
            let MenhirCell1_STAR_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_442 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0143 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.STAR_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_STAR_COLON (_menhir_stack, MenhirState0143) in
            let _menhir_s = MenhirState0144 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.PLUS_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_PLUS_COLON (_menhir_stack, MenhirState0143) in
            let _menhir_s = MenhirState0148 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0143) in
            let _menhir_s = MenhirState0152 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COLON (_menhir_stack, MenhirState0143) in
            let _menhir_s = MenhirState0154 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0143
        | Tokens.RBRACKET ->
            let expr = _v in
            let _v = _menhir_action_438 expr in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0130 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SLICING as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0130) in
            let _menhir_s = MenhirState0131 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0130
        | Tokens.RBRACE ->
            let MenhirCell1_SLICING (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_425 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0011 ->
            _menhir_run_0344 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0304 ->
            _menhir_run_0306 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0133 ->
            _menhir_run_0134 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0131 ->
            _menhir_run_0132 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0124 ->
            _menhir_run_0126 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0344 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_INTEGER _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_456 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
    
    and _menhir_run_0306 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
        let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_115 _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0134 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, expr, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_426 expr xs in
        _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0132 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SLICING, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_SLICING (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (xs, _2_inlined1) = (_v, ()) in
        let _v = _menhir_action_427 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0126 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_AS, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
        let MenhirCell1_AS (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_060 _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0128 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SLICING ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_SLICING (_menhir_stack, MenhirState0128) in
            let _menhir_s = MenhirState0129 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0128) in
            let _menhir_s = MenhirState0133 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0128
        | Tokens.RBRACE ->
            let expr = _v in
            let _v = _menhir_action_424 expr in
            _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0122 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_BAND as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BAND (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_023 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0120 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_BEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_025 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0118 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_BOR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0118
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BOR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_024 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0116 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_EQ_OP as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0116
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_EQ_OP (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_029 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0113 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_IMPL as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0113
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_IMPL (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_033 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0111 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_GEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_GEQ (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_032 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0109 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_GT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_GT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_031 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0107 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0107
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_LEQ (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_035 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0105 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_AND as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_AND (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_022 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0103 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COLON_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_COLON_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_045 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0101 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_EOR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0101
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_EOR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_028 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0099 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0099
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_LT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_034 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0097 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0097
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MINUS (_menhir_stack, _, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_037 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0095 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_NEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0095
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_NEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_030 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0092 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0092
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_OR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_040 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0090 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DIV as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0090
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0090
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0090
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0090
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0090
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0090
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_DIV (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_026 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0088 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DIVRM as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0088
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0088
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0088
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0088
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0088
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0088
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_DIVRM (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_027 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0086 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_MOD as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0086
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0086
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0086
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0086
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0086
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0086
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MOD (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_038 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0084 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_MUL as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0084
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0084
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0084
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0084
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0084
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0084
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MUL (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_039 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0082 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_PLUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0082
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_PLUS (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_036 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0077 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_CONCAT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_CONCAT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_046 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0067 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RDIV as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0067
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0067
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0067
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0067
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0067
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0067
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_RDIV (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_041 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0065 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ARRAY as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.RBRACKET ->
            let _menhir_s = MenhirState0065 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RBRACKET (_menhir_stack, _menhir_s, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.OF ->
                let _menhir_s = MenhirState0079 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0065
        | _ ->
            _eRR ()
    
    and _menhir_run_0046 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BITS _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.SHL ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0046, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _startpos_1 = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_1) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    _menhir_run_0049 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0048
                | Tokens.RBRACE ->
                    let _v_2 = _menhir_action_197 () in
                    _menhir_run_0158 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2
                | _ ->
                    _eRR ())
            | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RECURSELIMIT | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.SUBTYPES | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
                let _endpos = _endpos_0 in
                let _v = _menhir_action_193 () in
                _menhir_goto_loption_bitfields_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.POW ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.PLUS ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.OR ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.NEQ ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.MUL ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.MOD ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.MINUS ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.LT ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.LLBRACKET ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.LEQ ->
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.LBRACKET ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.IN ->
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.IMPL ->
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.GT ->
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.GEQ ->
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.EQ_OP ->
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.EOR ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.DOT ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.DIVRM ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.DIV ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.CONCAT ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.COLON_COLON ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.BOR ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.BEQ ->
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.BAND ->
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.AS ->
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | Tokens.AND ->
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0046
        | _ ->
            _eRR ()
    
    and _menhir_run_0037 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SHL as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0037
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0037
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0037
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0037
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0037
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0037
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_SHL (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_042 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0035 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0035) in
            let _menhir_s = MenhirState0326 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0035
        | Tokens.RBRACKET ->
            let MenhirCell1_COLON (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_1, e) = ((), _v) in
            let _v = _menhir_action_441 _1 e in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0032 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_POW as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0032
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0032
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0032
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0032
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_POW (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_044 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0030 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SHR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0030
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0030
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0030
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0030
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0030
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0030
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_SHR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_043 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0028 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LLBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.RRBRACKET ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LLBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_2, _4, e2, _endpos__4_) = ((), (), _v, _endpos_0) in
            let _v = _menhir_action_055 _2 _4 _endpos__4_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__4_ _startpos_e1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0106 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0112 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0108 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0110 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0119 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0121 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0104 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0028
        | _ ->
            _eRR ()
    
    and _menhir_run_0026 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BNOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0026
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0026
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0038 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0026
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0026
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_047 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0372 : type  ttv_stack. (ttv_stack _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_WITH _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_WITH (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_192 _1 _1_inlined2 in
        _menhir_goto_loption___anonymous_22_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
    
    and _menhir_goto_loption___anonymous_22_ : type  ttv_stack. (ttv_stack _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1027 ->
            _menhir_run_1028 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0359 ->
            _menhir_run_0374 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1028 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty_decl _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER -> _ -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2, _, _) = _menhir_stack in
        let MenhirCell0_SUBTYPES (_menhir_stack, _) = _menhir_stack in
        let (_1, _3) = ((), _v) in
        let _v = _menhir_action_403 _1 _2 _3 in
        _menhir_goto_option_subtype_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
    
    and _menhir_goto_option_subtype_ : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty_decl -> _ -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let _menhir_stack = MenhirCell0_option_subtype_ (_menhir_stack, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
            | Tokens.EOF ->
                let _v_0 = _menhir_action_134 () in
                _menhir_run_1031 _menhir_stack _v_0
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0374 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption___anonymous_22_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0375
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1008 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1012 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_RECORD _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RECORD (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_470 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_goto_ty_decl : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty_decl (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SUBTYPES ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_SUBTYPES (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _startpos_1 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos_1, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WITH ->
                    _menhir_run_0360 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1027
                | Tokens.SEMI_COLON ->
                    let _v_2 = _menhir_action_191 () in
                    _menhir_run_1028 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.SEMI_COLON ->
            let _v = _menhir_action_402 () in
            _menhir_goto_option_subtype_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_1016 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_EXCEPTION _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_EXCEPTION (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_472 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_1020 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                _menhir_run_1020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState1021
            | Tokens.RBRACE ->
                let (x, x_inlined1) = (_v, ()) in
                let _v = _menhir_action_211 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let x = _v in
            let _v = _menhir_action_210 x in
            _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1019 ->
            _menhir_run_1023 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1021 ->
            _menhir_run_1022 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1023 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ENUMERATION _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ENUMERATION (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_468 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_1022 : type  ttv_stack. (ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, h, _, _) = _menhir_stack in
        let (_2, t) = ((), _v) in
        let _v = _menhir_action_212 _2 h t in
        _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1056 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1057
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1058 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1059 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1043 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1044
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1045 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1046 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1035 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_VAR _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.TYPE ->
                _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.SETTER ->
                _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.PRAGMA ->
                _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.LET ->
                _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.GETTER ->
                _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.FUNC ->
                _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.CONSTANT ->
                _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.CONFIG ->
                _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1036
            | Tokens.EOF ->
                let _v_1 = _menhir_action_134 () in
                _menhir_run_1037 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1038 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1032 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _1 = _v in
        let _v = _menhir_action_467 _1 in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_1001 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1002 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0993 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0994 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0981 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0982 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0968 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0969 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0962 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0963 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0951 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0952 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0945 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_LET _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0946 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0936 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0937 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0915 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RECURSELIMIT ->
            let _menhir_s = MenhirState0916 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0924 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0903 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0904 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0897 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_450 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0393 ->
            _menhir_run_0988 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState0793 ->
            _menhir_run_0933 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState0803 ->
            _menhir_run_0906 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState0904 ->
            _menhir_run_0905 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0900 ->
            _menhir_run_0901 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0988 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _menhir_s, _v) in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.COLON_COLON ->
                    let _menhir_s = MenhirState0992 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.COLON ->
                    let _menhir_s = MenhirState1000 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0933 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _menhir_s, _v) in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0935 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0906 : type  ttv_stack. (((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _menhir_s, _v) in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0908 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0914 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0905 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_452 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0901 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_453 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0899 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0900 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0897 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_451 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0882 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RECURSELIMIT ->
            let _menhir_s = MenhirState0883 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0891 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0873 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0874 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0867 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0868 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0856 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0857 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0850 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONSTANT _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0851 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0843 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0844 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0837 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0838 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0826 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0827 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0820 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_CONFIG _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0821 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0796 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0797 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0746 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0747
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0747
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _2_inlined2) = ((), _endpos_0, (), (), _v) in
                let _v = _menhir_action_273 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0727 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0728 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0432 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0433
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0433
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _2_inlined2) = ((), _endpos_0, (), (), _v) in
                let _v = _menhir_action_274 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0424 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _, li) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, x) = _menhir_stack in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_2, _2_inlined1, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_008 _1_inlined1 _2 _2_inlined1 li x in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0422 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _, li) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, x) = _menhir_stack in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_2, _2_inlined1, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_009 _1_inlined1 _2 _2_inlined1 li x in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0416 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _1, _, _) = _menhir_stack in
        let (_2, _1_inlined1) = (_v, ()) in
        let _v = _menhir_action_004 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0414 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _1, _, _) = _menhir_stack in
        let (_2, _1_inlined1) = (_v, ()) in
        let _v = _menhir_action_005 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0410 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_MINUS -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _, _) = _menhir_stack in
        let (_1, _2, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_006 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0408 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_MINUS -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _, _) = _menhir_stack in
        let (_1, _2, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_007 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0398 : type  ttv_stack. ((((ttv_stack, _menhir_box_ast) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_ast) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_ast) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0399 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0387 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0388 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0379 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_431 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0383 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0384 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0379 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_432 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0368 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_0362 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0369
            | Tokens.RBRACE ->
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
                let (_1, _2_inlined1, x_inlined1) = ((), _v, ()) in
                let _v = _menhir_action_226 _1 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2_inlined1) = ((), _v) in
            let _v = _menhir_action_225 _1 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1015 ->
            _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState1011 ->
            _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState0361 ->
            _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState0369 ->
            _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0365 ->
            _menhir_run_0366 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0371 : type  ttv_stack. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let x = _v in
        let _v = _menhir_action_200 x in
        _menhir_goto_loption_non_empty_trailing_list_COMMA_typed_identifier__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_goto_loption_non_empty_trailing_list_COMMA_typed_identifier__ : type  ttv_stack. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1015 ->
            _menhir_run_1016 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1011 ->
            _menhir_run_1012 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0361 ->
            _menhir_run_0372 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0370 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, _2, t) = ((), (), _v) in
        let _v = _menhir_action_229 _1 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0366 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER, _menhir_box_ast) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, _2, t) = ((), (), _v) in
        let _v = _menhir_action_230 _1 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0364 : type  ttv_stack. ((ttv_stack, _menhir_box_ast) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_ast) _menhir_state -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_0362 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0365
            | Tokens.RBRACE ->
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
                let (_1, _2_inlined1, x_inlined1) = ((), _v, ()) in
                let _v = _menhir_action_228 _1 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2_inlined1) = ((), _v) in
            let _v = _menhir_action_227 _1 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0353 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_UNKNOWN -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_UNKNOWN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_065 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0351 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_UNKNOWN -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_UNKNOWN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_066 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0346 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0347 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0044 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let x = _v in
            let _v = _menhir_action_448 x in
            _menhir_goto_separated_nonempty_list_COMMA_ty_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_ty_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0008 ->
            _menhir_run_0349 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0347 ->
            _menhir_run_0348 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0349 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LPAR -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_464 _1_inlined1 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
    
    and _menhir_run_0348 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_ty -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _menhir_s, x, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_449 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_ty_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0308 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _endpos_ty_, ty) = ((), _endpos, _v) in
        let _v = _menhir_action_113 _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0163 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_UNKNOWN -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_UNKNOWN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_120 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0161 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_UNKNOWN -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_UNKNOWN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_121 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0140 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_0049 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0141
            | Tokens.RBRACE ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (_2_inlined1, _1_inlined2, x_inlined1) = (_v, (), ()) in
                let _v = _menhir_action_218 _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
            let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_2_inlined1, _1_inlined2) = (_v, ()) in
            let _v = _menhir_action_217 _1_inlined1 _1_inlined2 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0136 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_0049 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0137
            | Tokens.RBRACE ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (_2_inlined1, _1_inlined2, x_inlined1) = (_v, (), ()) in
                let _v = _menhir_action_220 _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
            let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_2_inlined1, _1_inlined2) = (_v, ()) in
            let _v = _menhir_action_219 _1_inlined1 _1_inlined2 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0135 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_AS -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_AS (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _endpos_ty_, ty) = ((), _endpos, _v) in
        let _v = _menhir_action_058 _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0080 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_ARRAY, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RBRACKET -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_RBRACKET (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ARRAY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_t_, t, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_466 _1 _3 _endpos_t_ _startpos__1_ t x in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_t_ _v _menhir_s _tok
    
    let _menhir_run_0000 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_ast =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_0001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.TYPE ->
            _menhir_run_0356 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.SETTER ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.PRAGMA ->
            _menhir_run_0782 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.LET ->
            _menhir_run_0785 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.GETTER ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.FUNC ->
            _menhir_run_0800 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.CONSTANT ->
            _menhir_run_0808 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.CONFIG ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.EOF ->
            let _v = _menhir_action_134 () in
            _menhir_run_1071 _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1074 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1075 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.VAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_VAR (_menhir_stack, MenhirState1074, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
            | Tokens.LPAR ->
                _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
            | Tokens.IDENTIFIER _v ->
                let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.COMMA ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1087, _v, _startpos_0, _endpos) in
                    let _menhir_s = MenhirState1089 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.COLON_COLON ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1087, _v, _startpos_0, _endpos) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.COLON ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1087, _v, _startpos_0, _endpos) in
                    _menhir_run_0415 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.EQ | Tokens.SEMI_COLON ->
                    let _v =
                      let _1 = _v in
                      _menhir_action_010 _1
                    in
                    _menhir_run_1099 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1087 _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.UNREACHABLE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.RPAR ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_3 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_startpos__1_, _1, _2, _2_inlined1, _3, _endpos__2_inlined2_) = (_startpos, (), (), (), (), _endpos_3) in
                            let _v = _menhir_action_392 _1 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.TRY ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1128 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.THROW ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1139
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1139
            | Tokens.SEMI_COLON ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos) in
                    let _v = _menhir_action_397 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1139
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1139
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1139
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1139
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1139
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1139
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1139
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1139
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1139
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1139
            | _ ->
                _eRR ())
        | Tokens.RETURN ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1145
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1145
            | Tokens.SEMI_COLON ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos) in
                    let _v = _menhir_action_357 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1145
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1145
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1145
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1145
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1145
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1145
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1145
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1145
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1145
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState1074, _startpos) in
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1145
            | _ ->
                _eRR ())
        | Tokens.REPEAT ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1151 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.PRINT ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
                | Tokens.STRING_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1162
                | Tokens.RPAR ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_5 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos_5) in
                            let _v = _menhir_action_388 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1162
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
                | Tokens.INT_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1162
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
                | Tokens.IDENTIFIER _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1162
                | Tokens.BOOL_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1162
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
                | Tokens.BITVECTOR_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1162
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.PRAGMA ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos_6 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1171
                | Tokens.STRING_LIT _v_7 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState1171
                | Tokens.SEMI_COLON ->
                    let _endpos_8 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.EOF ->
                        let (_startpos__1_, _1, _2, _endpos__2_inlined1_, _2_inlined1, _endpos__2_inlined2_) = (_startpos, (), (), _endpos, _v, _endpos_8) in
                        let _v = _menhir_action_398 _1 _2 _2_inlined1 _endpos__2_inlined1_ _endpos__2_inlined2_ _startpos__1_ in
                        _menhir_goto_opn _menhir_stack _v
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v_9 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState1171
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1171
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1171
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1171
                | Tokens.INT_LIT _v_10 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState1171
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1171
                | Tokens.IDENTIFIER _v_11 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_11 MenhirState1171
                | Tokens.BOOL_LIT _v_12 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState1171
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1171
                | Tokens.BITVECTOR_LIT _v_13 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState1171
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.PASS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos) in
                    let _v = _menhir_action_356 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.MINUS ->
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1074
        | Tokens.LPAR ->
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1074
        | Tokens.LET ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1180 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IF ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1203 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos_14 = _menhir_lexbuf.Lexing.lex_start_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1211
                | Tokens.STRING_LIT _v_15 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState1211
                | Tokens.RPAR ->
                    let _endpos_16 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_17 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_2, _endpos__2_inlined1_, _startpos_name_, name) = ((), _endpos_17, _startpos, _v) in
                            let _v = _menhir_action_359 _2 _endpos__2_inlined1_ _startpos_name_ name in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | Tokens.EQ ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState1211, _endpos_16) in
                        let _menhir_s = MenhirState1215 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.UNKNOWN ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.DOT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState1211, _endpos_16) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LBRACKET ->
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.IDENTIFIER _v ->
                                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.COMMA ->
                                    let _menhir_s = MenhirState1222 in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.IDENTIFIER _v ->
                                        _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | _ ->
                                _eRR ())
                        | Tokens.IDENTIFIER _v ->
                            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.EQ ->
                                let _menhir_s = MenhirState1230 in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.UNKNOWN ->
                                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.STRING_LIT _v ->
                                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.REAL_LIT _v ->
                                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.NOT ->
                                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.MINUS ->
                                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.LPAR ->
                                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.INT_LIT _v ->
                                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.IF ->
                                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.IDENTIFIER _v ->
                                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.BOOL_LIT _v ->
                                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.BNOT ->
                                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.BITVECTOR_LIT _v ->
                                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | _ ->
                                    _eRR ())
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v_37 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v_37 MenhirState1211
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1211
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1211
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1211
                | Tokens.INT_LIT _v_38 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v_38 MenhirState1211
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1211
                | Tokens.IDENTIFIER _v_39 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v_39 MenhirState1211
                | Tokens.BOOL_LIT _v_40 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_40 MenhirState1211
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1211
                | Tokens.BITVECTOR_LIT _v_41 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_41 MenhirState1211
                | _ ->
                    _eRR ())
            | Tokens.LBRACE ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState1074, _v, _startpos, _endpos) in
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState1257 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.DOT | Tokens.EQ | Tokens.LBRACKET | Tokens.LLBRACKET ->
                let (_menhir_s, _endpos__1_, _startpos__1_, _1) = (MenhirState1074, _endpos, _startpos, _v) in
                let _v = _menhir_action_128 _1 in
                _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.FOR ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _menhir_s = MenhirState1284 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.UNKNOWN ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.DEBUG ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos_56 = _menhir_lexbuf.Lexing.lex_start_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1311
                | Tokens.STRING_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1311
                | Tokens.RPAR ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_57 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos_57) in
                            let _v = _menhir_action_390 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1311
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1311
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1311
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1311
                | Tokens.INT_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1311
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1311
                | Tokens.IDENTIFIER _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1311
                | Tokens.BOOL_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1311
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1311
                | Tokens.BITVECTOR_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState1074, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1311
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.CONSTANT ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1319 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0406 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0411 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.CASE ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1342 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ASSERT ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1348 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.UNKNOWN ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.STRING_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_s = MenhirState1074 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState1353 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.UNKNOWN ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0016 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0020 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0024 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1374 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_stmts =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState1374 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0405 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0438 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0439 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0441 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0443 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0448 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0453 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0485 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0488 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0514 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0525 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
  end
  
  let stmts =
    fun _menhir_lexer _menhir_lexbuf ->
      let _menhir_stack = () in
      let MenhirBox_stmts v = _menhir_run_1374 _menhir_stack _menhir_lexbuf _menhir_lexer in
      v
  
  let opn =
    fun _menhir_lexer _menhir_lexbuf ->
      let _menhir_stack = () in
      let MenhirBox_opn v = _menhir_run_1074 _menhir_stack _menhir_lexbuf _menhir_lexer in
      v
  
  let ast =
    fun _menhir_lexer _menhir_lexbuf ->
      let _menhir_stack = () in
      let MenhirBox_ast v = _menhir_run_0000 _menhir_stack _menhir_lexbuf _menhir_lexer in
      v
  
end
